using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;

using Evaluant.Uss;

namespace Evaluant.Uss.DataBaseGenerator
{
	/// <summary>
	/// Description rsume de Class1.
	/// </summary>
	class Program
	{
		/// <summary>
		/// Point d'entre principal de l'application.
		/// </summary>
		[STAThread]
		static int Main(string[] args)
		{
			string buildDB = String.Empty;
			string config = String.Empty;

			ArgParser parser = new ArgParser(args);
			
			if(parser.Exists("?"))
			{
				Help();
				return 0;
			}

			if(!parser.Exists("config"))
			{
				Console.WriteLine("Incorrect parameters : no configuration file is defined");
				Help();
				return 1;
			}
			
			config = parser["config"].Value;

			if(!File.Exists(config))
			{
				Console.WriteLine("The configuration file does not exist.");
				return 2;           
			}

			try
			{
				IPersistenceProvider pef;

				if(parser.Exists("engine"))
				{
					pef = XmlConfigLoader.LoadXmlConfig(config, parser["engine"].Value);
				}
				else
				{
					pef = XmlConfigLoader.LoadXmlConfig(config);
				}

				IPersistenceEngine pe = pef.CreatePersistenceEngine();

                pe.InitializeRepository();
			}
			catch(FileNotFoundException fex)
			{
				Console.WriteLine(String.Concat("File not found : ", fex.FileName));
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex.Message);
			}

			return 0;
		}

		static public void Help()
		{
			Console.WriteLine(@"

Help for initialize.exe

Initializes a repository

initialize.exe /config:config_filename [/engine:engine_name]

	config_filename
		Specify the Euss Configuration File which contains the parameters for SQLMapper provider

	engine_name
		Specify the engine configuration name which is to use. If not provided the default engine is used.
");

		}
	}
}
