using System;
using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.Diagnostics;
using System.Windows.Forms;

using Microsoft.Win32;

using System.IO;

namespace TemplatesManager
{
	/// <summary>
	/// Description rsume de TemplatesInstaller.
	/// </summary>
	[RunInstaller(true)]
	public class TemplatesInstaller : System.Configuration.Install.Installer
	{
		/// <summary>
		/// Variable ncessaire au concepteur.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public TemplatesInstaller()
		{
			// Cet appel est requis par le concepteur.
			InitializeComponent();

			// TODO: ajoutez les initialisations aprs l'appel  InitializeComponent
		}

		/// <summary> 
		/// Nettoyage des ressources utilises.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		protected override void OnBeforeInstall(IDictionary savedState)
		{
			base.OnBeforeInstall (savedState);
			Process[] processes = Process.GetProcessesByName("devenv");

			if(processes != null)
			{
				foreach(Process p in processes)
				{
					while(!p.HasExited)
					{
						DialogResult dr = MessageBox.Show("Please close your Visual Studio instance", "An instance of Visual Studio is running", MessageBoxButtons.RetryCancel, MessageBoxIcon.Exclamation);
						if(dr == DialogResult.Cancel)
							base.Rollback(savedState);
					}
				}
			}
		}


		#region Code gnr par le Concepteur de composants
		/// <summary>
		/// Mthode requise pour la prise en charge du concepteur - ne modifiez pas
		/// le contenu de cette mthode avec l'diteur de code.
		/// </summary>
		private void InitializeComponent()
		{

		}
		#endregion

		private string GetItemTemplatesDir(RegistryKey rk, string vsVersion, Languages lang)
		{
			rk = rk.OpenSubKey(vsVersion).OpenSubKey("Projects");

			if(lang == Languages.CS)
				return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindCSharpProject).GetValue(@"ItemTemplatesDir") + @"\";
			if(lang == Languages.VB)
				return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindVBProject).GetValue(@"ItemTemplatesDir") + @"\";

			return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindCSharpProject).GetValue(@"ItemTemplatesDir") + @"\";
		}

		private string GetWizardsTemplatesDir(RegistryKey rk, string vsVersion, Languages lang)
		{
			rk = rk.OpenSubKey(vsVersion).OpenSubKey("Projects");

			if(lang == Languages.CS)
				return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindCSharpProject).GetValue(@"WizardsTemplatesDir");
			if(lang == Languages.VB)
				return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindVBProject).GetValue(@"WizardsTemplatesDir");

			return (string)rk.OpenSubKey(VSLangProj.PrjKind.prjKindCSharpProject).GetValue(@"WizardsTemplatesDir");
		}

		private string GetShemasDir(RegistryKey rk, string vsVersion,string vsCommonDir)
		{
			rk = rk.OpenSubKey(vsVersion).OpenSubKey("Setup").OpenSubKey("VS");

			string commonDir = (string)rk.GetValue(@vsCommonDir);

			return String.Concat(commonDir, @"\Packages\schemas\xml\");
		}

		public RegistryKey GetVSKey()
		{
			RegistryKey rk;
			rk = Registry.LocalMachine.OpenSubKey("SOFTWARE").OpenSubKey("Microsoft").OpenSubKey("VisualStudio");
			if(rk != null)
			{
				return rk;
			}

			rk = Registry.CurrentUser.OpenSubKey("SOFTWARE").OpenSubKey("Microsoft").OpenSubKey("VisualStudio");
			if(rk != null)
			{
				return rk;
			}

			throw new Exception("Cannot find Visual Studio");
		}

		private const string ENGINECONFIGURATIONFILE_CONFIG = "Euss Engine Configuration File.config";
		private const string DOMAINMODELFILE_XML = "Euss Domain Model.xml";
		private const string EUSSMETADATAFILE_XML = "Euss MetaData File.xml";
		private const string SQLMAPPERFILE_XML = "Euss SQLMapper File.xml";
		private const string WEAVINGMODEL_XML = "Euss Weaving Model.xml";
		private const string SQLMAPPEROPTIONFILE_XML = "Euss SQLMapper Option File.xml";

		private const string ENGINECONFIGURATIONFILE_XSD = "Engine Configuration File.xsd";
		private const string DOMAINMODELFILE_XSD = "Euss Domain Model.xsd";
		private const string SQLMAPPERFILE_XSD = "Euss SQLMapper File.xsd";
		private const string WEAVINGMODEL_XSD = "Euss Weaving Model.xsd";
		private const string SQLMAPPEROPTIONFILE_XSD = "Euss SQLMapper Option File.xsd";
		private const string EUSSMETADATAFILE_XSD = "Euss MetaData File.xsd";

		private const string EUSSLOCALPROJECTITEMS = "EussLocalProjectItems.vsdir";
		private const string EUSSWEBPROJECTITEMS = "EussWebProjectItems.vsdir";
		private const string EUSSLOCALITEMS = "EussLocalItems.vsdir";
		private const string EUSSITEMS = "EussItems.vsdir";

		protected override void OnAfterInstall(IDictionary savedState)
		{
			base.OnAfterInstall (savedState);

			try
			{
				RegistryKey rk = GetVSKey();

				CreateTemplateFiles("7.1", Languages.CS, rk);
				CreateTemplateFiles("7.1", Languages.VB, rk);

				CreateShemaFiles("7.1", rk, "VS7CommonDir");
 
				CreateVszFile("Euss Domain Model.vsz", "7.1", Languages.CS, rk);
				CreateVSWizard("CSharpAddDomainModelWiz", "Euss Domain Model.inf",String.Concat("ItemInstaller.Wizards.", "domainmodel.js"), String.Concat("ItemInstaller.Templates.", DOMAINMODELFILE_XML), "EUSS Domain Model.xml", "7.1", Languages.CS, rk); 

				CreateVszFile("Euss Domain Model.vsz", "7.1", Languages.VB, rk);
				CreateVSWizard("VBAddDomainModelWiz", "Euss Domain Model.inf",String.Concat("ItemInstaller.Wizards.", "domainmodel.js"), String.Concat("ItemInstaller.Templates.", DOMAINMODELFILE_XML), "EUSS Domain Model.xml", "7.1", Languages.VB, rk);

                // manager.CreateEussVars(Context.Parameters["installdir"]);

			}
			catch(Exception e)
			{
                Context.LogMessage(e.Message);
			}
		}

		private void CreateVszFile(string vszFile, string vsVersion, Languages lang, RegistryKey rk)
		{
			string destTemplatesDir = GetItemTemplatesDir(rk, vsVersion, lang);

			string eussTemplatesDir = destTemplatesDir + @"\Euss\";

			if(!Directory.Exists(eussTemplatesDir))
				Directory.CreateDirectory(eussTemplatesDir);

			ExtractResource(String.Concat("ItemInstaller.VszFiles.", vszFile), String.Concat(eussTemplatesDir, vszFile));
		}

		private void DeleteVszFile(string vszFile, string vsVersion, Languages lang, RegistryKey rk)
		{
			string destTemplatesDir = GetItemTemplatesDir(rk, vsVersion, lang);

			string eussTemplatesDir = destTemplatesDir + @"\Euss\";

			if(File.Exists(String.Concat(eussTemplatesDir, vszFile)))
				File.Delete(String.Concat(eussTemplatesDir, vszFile));
		}

		private void CreateVSWizard(string wizardName, string templateInfName, string scriptRessourceLocation, string templateRessourceLocation, string templateName, string vsVersion, Languages lang,RegistryKey rk)
		{
			string destWizardDir = GetWizardsTemplatesDir(rk, vsVersion, lang);

			string destWizardNameDir = String.Concat(destWizardDir, @"\", wizardName);

			if(!Directory.Exists(destWizardDir))
				Directory.CreateDirectory(destWizardDir);

			string dirName = String.Empty;

			if(lang == Languages.CS)
				dirName = "1036";
			if(lang == Languages.VB)
				dirName = "1033";

			string scriptDir = String.Concat(destWizardNameDir, @"\Scripts\" + dirName + @"\");
			string templateDir = String.Concat(destWizardNameDir, @"\Templates\" + dirName + @"\");

			if(!Directory.Exists(scriptDir))
				Directory.CreateDirectory(scriptDir);

			ExtractResource(scriptRessourceLocation, String.Concat(scriptDir, DEFAULTSCRIPTNAME));

			if(!Directory.Exists(templateDir))
				Directory.CreateDirectory(templateDir);

			ExtractResource(templateRessourceLocation, String.Concat(templateDir, templateName));
			ExtractResource(String.Concat("ItemInstaller.Templates.", templateInfName), String.Concat(templateDir, DEFAULTTEMPLATENAME));
		}

		private void DeleteVSWizard(string wizardName, string templateName, string vsVersion, Languages lang,RegistryKey rk)
		{
			string destWizardDir = GetWizardsTemplatesDir(rk, vsVersion, lang);

			string destWizardNameDir = String.Concat(destWizardDir, @"\", wizardName);

			string dirName = String.Empty;

			if(lang == Languages.CS)
				dirName = "1036";
			if(lang == Languages.VB)
				dirName = "1033";

			string scriptDir = String.Concat(destWizardNameDir, @"\Scripts\" + dirName + @"\");
			string templateDir = String.Concat(destWizardNameDir, @"\Templates\" + dirName + @"\");

			if(File.Exists(String.Concat(scriptDir, DEFAULTSCRIPTNAME)))
				File.Delete(String.Concat(scriptDir, DEFAULTSCRIPTNAME));

			if(File.Exists(String.Concat(templateDir, templateName)))
				File.Delete(String.Concat(templateDir, templateName));

			if(File.Exists(String.Concat(templateDir, DEFAULTTEMPLATENAME)))
				File.Delete(String.Concat(templateDir, DEFAULTTEMPLATENAME));

			if(Directory.Exists(destWizardNameDir))
				Directory.Delete(destWizardNameDir, true);
		}

		private const string DEFAULTSCRIPTNAME = "default.js";
		private const string DEFAULTTEMPLATENAME = "Templates.inf";

		private void CreateTemplateFiles(string vsVersion, Languages lang, RegistryKey rk)
		{
			string destTemplatesDir = GetItemTemplatesDir(rk, vsVersion, lang);

			string eussTemplatesDir = destTemplatesDir + @"\" + EUSSTEMPLATESDIR + @"\";

			if(!Directory.Exists(eussTemplatesDir))
				Directory.CreateDirectory(eussTemplatesDir);

			ExtractResource(String.Concat("ItemInstaller.VsdirFiles.", EUSSITEMS), String.Concat(eussTemplatesDir, EUSSITEMS));

			ExtractResource(String.Concat("ItemInstaller.Templates.", EUSSMETADATAFILE_XML), String.Concat(destTemplatesDir + @"\", EUSSMETADATAFILE_XML));
			ExtractResource(String.Concat("ItemInstaller.Templates.", SQLMAPPERFILE_XML), String.Concat(destTemplatesDir + @"\", SQLMAPPERFILE_XML));
			ExtractResource(String.Concat("ItemInstaller.Templates.", WEAVINGMODEL_XML), String.Concat(destTemplatesDir + @"\", WEAVINGMODEL_XML));
			ExtractResource(String.Concat("ItemInstaller.Templates.", SQLMAPPEROPTIONFILE_XML), String.Concat(destTemplatesDir + @"\", SQLMAPPEROPTIONFILE_XML));
			ExtractResource(String.Concat("ItemInstaller.Templates.", ENGINECONFIGURATIONFILE_CONFIG), String.Concat(destTemplatesDir + @"\", ENGINECONFIGURATIONFILE_CONFIG));
		}

		public void CreateShemaFiles(string vsVersion, RegistryKey rk, string vsCommonDir)
		{
			string destSchemasDir = GetShemasDir(rk, vsVersion, vsCommonDir);

			ExtractResource(String.Concat("ItemInstaller.Schemas.", ENGINECONFIGURATIONFILE_XSD), String.Concat(destSchemasDir, ENGINECONFIGURATIONFILE_XSD));
			ExtractResource(String.Concat("ItemInstaller.Schemas.", DOMAINMODELFILE_XSD), String.Concat(destSchemasDir, DOMAINMODELFILE_XSD));
			ExtractResource(String.Concat("ItemInstaller.Schemas.", SQLMAPPERFILE_XSD), String.Concat(destSchemasDir, SQLMAPPERFILE_XSD));
			ExtractResource(String.Concat("ItemInstaller.Schemas.", WEAVINGMODEL_XSD), String.Concat(destSchemasDir, WEAVINGMODEL_XSD));
			ExtractResource(String.Concat("ItemInstaller.Schemas.", SQLMAPPEROPTIONFILE_XSD), String.Concat(destSchemasDir, SQLMAPPEROPTIONFILE_XSD));
			ExtractResource(String.Concat("ItemInstaller.Schemas.", EUSSMETADATAFILE_XSD), String.Concat(destSchemasDir, EUSSMETADATAFILE_XSD));
		}

		private void DeleteTemplateFiles(RegistryKey rk, string vsVersion, Languages lang)
		{
			string destTemplatesDir = GetItemTemplatesDir(rk, vsVersion, lang);

			string eussTemplatesDir = destTemplatesDir + @"\" + EUSSTEMPLATESDIR + @"\";

			if(File.Exists(String.Concat(eussTemplatesDir, EUSSITEMS)))
				File.Delete(String.Concat(eussTemplatesDir, EUSSITEMS));

			if(File.Exists(String.Concat(eussTemplatesDir, EUSSMETADATAFILE_XML)))
				File.Delete(String.Concat(eussTemplatesDir, EUSSMETADATAFILE_XML));

			if(File.Exists(String.Concat(destTemplatesDir, SQLMAPPERFILE_XML)))
				File.Delete(String.Concat(destTemplatesDir, SQLMAPPERFILE_XML));

			if(File.Exists(String.Concat(destTemplatesDir, WEAVINGMODEL_XML)))
				File.Delete(String.Concat(destTemplatesDir, WEAVINGMODEL_XML));

			if(File.Exists(String.Concat(destTemplatesDir, SQLMAPPEROPTIONFILE_XML)))
				File.Delete(String.Concat(destTemplatesDir, SQLMAPPEROPTIONFILE_XML));

			if(File.Exists(String.Concat(destTemplatesDir, ENGINECONFIGURATIONFILE_CONFIG)))
				File.Delete(String.Concat(destTemplatesDir, ENGINECONFIGURATIONFILE_CONFIG));
		}

		private void DeleteShemaFiles(RegistryKey rk, string vsVersion, string vsCommonDir)
		{
			string destSchemasDir = GetShemasDir(rk, vsVersion, vsCommonDir);

			if(File.Exists(String.Concat(destSchemasDir, EUSSMETADATAFILE_XSD)))
				File.Delete(String.Concat(destSchemasDir, EUSSMETADATAFILE_XSD));

			if(File.Exists(String.Concat(destSchemasDir, ENGINECONFIGURATIONFILE_XSD)))
				File.Delete(String.Concat(destSchemasDir, ENGINECONFIGURATIONFILE_XSD));

			if(File.Exists(String.Concat(destSchemasDir, DOMAINMODELFILE_XSD)))
				File.Delete(String.Concat(destSchemasDir, DOMAINMODELFILE_XSD));
			
			if(File.Exists(String.Concat(destSchemasDir, SQLMAPPERFILE_XSD)))
				File.Delete(String.Concat(destSchemasDir, SQLMAPPERFILE_XSD));

			if(File.Exists(String.Concat(destSchemasDir, WEAVINGMODEL_XSD)))
				File.Delete(String.Concat(destSchemasDir, WEAVINGMODEL_XSD));

			if(File.Exists(String.Concat(destSchemasDir, SQLMAPPEROPTIONFILE_XSD)))
				File.Delete(String.Concat(destSchemasDir, SQLMAPPEROPTIONFILE_XSD));
		}

		protected override void OnBeforeUninstall(IDictionary savedState)
		{
			base.OnBeforeUninstall (savedState);

			try
			{
				RegistryKey rk = GetVSKey();

				DeleteVszFile("Euss Domain Model.vsz", "7.1", Languages.CS, rk);
				DeleteVszFile("Euss Domain Model.vsz", "7.1", Languages.VB, rk);

				DeleteTemplateFiles(rk, "7.1", Languages.CS);
				DeleteTemplateFiles(rk, "7.1", Languages.VB);

				DeleteVSWizard("CSharpAddDomainModelWiz", "EUSS Domain Model.xml", "7.1", Languages.CS, rk); 
				DeleteVSWizard("VBAddDomainModelWiz", "EUSS Domain Model.xml", "7.1", Languages.VB, rk); 

				DeleteTemplateDir(rk, "7.1", Languages.CS, EUSSTEMPLATESDIR);
				DeleteTemplateDir(rk, "7.1", Languages.VB, EUSSTEMPLATESDIR);

				DeleteShemaFiles(rk, "7.1", "VS7CommonDir");

//                manager.ExecuteDevEnvSetup();
//                manager.CreateEussVars(Context.Parameters["installdir"]);
			}
			catch(Exception e)
			{
                Context.LogMessage(e.Message);
			}
		}

		private const string EUSSTEMPLATESDIR = "Euss";

		private void DeleteTemplateDir(RegistryKey rk, string vsVersion, Languages lang, string dirName)
		{
			string destTemplatesDir = GetItemTemplatesDir(rk, vsVersion, lang);
			string eussTemplatesDir = destTemplatesDir + @"\" + dirName + @"\";

			if(Directory.Exists(eussTemplatesDir))
				Directory.Delete(eussTemplatesDir);
		}

		private void ExtractResource(string resource, string filename)
		{
			StreamReader sr = new StreamReader(this.GetType().Assembly.GetManifestResourceStream(resource));
			string content = sr.ReadToEnd();
			sr.Close();

			StreamWriter sw = new StreamWriter(filename);
			sw.Write(content);
			sw.Close();
		}
	}

    public enum  Languages 
	{
		CS,
		VB
	}
}
