using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;

namespace Evaluant.Uss.DomainModelGenerator
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class Program
	{
		[STAThread]
		public static int Main(string[] args)
		{
            try
            {
                ArgParser parser = new ArgParser(args);

                string xmiFilename = string.Empty;
                string path = string.Empty;
                string modelFileName = string.Empty;

                if (parser.Exists("?"))
                {
                    Help();
                    return 0;
                }

                if (!parser.Exists("xmi"))
                {
                    Help();
                    return 1;
                }

                if (parser.Exists("xmi"))
                    xmiFilename = parser["xmi"].Value;

                path = Path.GetDirectoryName(xmiFilename);
                if (path == string.Empty)
                    path = Environment.CurrentDirectory;

                path += "\\";

                if (parser.Exists("model"))
                {
                    modelFileName = parser["model"].Value;

                    string modelPath = Path.GetDirectoryName(modelFileName);
                    if (modelPath == string.Empty)
                        modelFileName = String.Concat(path, Path.GetFileName(modelFileName));
                }
                else
                    modelFileName = string.Concat(path, "model.xml");

                if (!File.Exists(xmiFilename))
                    throw new FileNotFoundException(String.Format("Cannot find the file \"{0}\"", xmiFilename));

                DomainModelGenerator modelGen = new DomainModelGenerator(xmiFilename, modelFileName);
                modelGen.GenerateDomainModel();

                Console.WriteLine(String.Format("\"{0}\" successfully created", modelFileName));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine("Generation aborted !");
                return -1;
            }
			return 0;
		}

		static public void Help()
		{
			Console.WriteLine(@"

Help for xmi2domain.exe

Converts an XMI file to the euss xml domain model format

xmi2domain.exe /xmi:xmi_filename [/model:model_filename]

	xmi_filename
		The xmi file describing the domain model in xmi format

	model_filename
		(Optional) The file containing the domain model in xml format. 
        If not defined the model file is generated close to the xmi file ('model.xml')
");

		}
	}
}
