using System;

namespace Evaluant.OPath.Expressions
{
	/// <summary>
	/// Description rsume de Path.
	/// </summary>
	[Serializable]
    public class BinaryOperator : Expression, ICloneable
	{
		/// <summary>
		/// Creates a new <see cref="BinaryOperator"/> instance.
		/// </summary>
		public BinaryOperator() : base()
		{
			type = BinaryOperatorEnum.Unknown;
		}

        public BinaryOperator(BinaryOperatorEnum type, Constraint left, Constraint right)
        {
            this.leftOperand = left;
            this.rightOperand = right;
            this.type = type;
        }

        [System.Diagnostics.DebuggerStepThrough]
        public override void Accept(OPathVisitor visitor)
		{
			visitor.Visit(this);
		}

		private Constraint leftOperand;

		public Constraint LeftOperand
		{
            [System.Diagnostics.DebuggerStepThrough]
            get { return leftOperand; }
			set { leftOperand = value; }
		}

		private Constraint rightOperand;
        public Constraint RightOperand
		{
            [System.Diagnostics.DebuggerStepThrough]
            get { return rightOperand; }
			set { rightOperand = value; }
		}

		private BinaryOperatorEnum type;
		/// <summary>
		/// Gets or sets the type.
		/// </summary>
		/// <value></value>
        public BinaryOperatorEnum Type
		{
            [System.Diagnostics.DebuggerStepThrough]
            get { return type; }
			set { type = value; }
		}

        public override void ToString(System.Text.StringBuilder sb)
        {
            leftOperand.ToString(sb);

            switch (Type)
            {
                case BinaryOperatorEnum.And:
                    sb.Append(" and ");
                    break;
                case BinaryOperatorEnum.Or:
                    sb.Append(" or ");
                    break;
                case BinaryOperatorEnum.NotEqual:
                    sb.Append(" != ");
                    break;
                case BinaryOperatorEnum.LesserOrEqual:
                    sb.Append(" <= ");
                    break;
                case BinaryOperatorEnum.GreaterOrEqual:
                    sb.Append(" >= ");
                    break;
                case BinaryOperatorEnum.Lesser:
                    sb.Append(" < ");
                    break;
                case BinaryOperatorEnum.Greater:
                    sb.Append(" > ");
                    break;
                case BinaryOperatorEnum.Equal:
                    sb.Append(" = ");
                    break;
                case BinaryOperatorEnum.Minus:
                    sb.Append(" - ");
                    break;
                case BinaryOperatorEnum.Plus:
                    sb.Append(" + ");
                    break;
                case BinaryOperatorEnum.Modulo:
                    sb.Append(" % ");
                    break;
                case BinaryOperatorEnum.Div:
                    sb.Append(" / ");
                    break;
                case BinaryOperatorEnum.Times:
                    sb.Append(" * ");
                    break;
                case BinaryOperatorEnum.Contains:
                    sb.Append(" ~ ");
                    break;
                case BinaryOperatorEnum.BeginsWith:
                    sb.Append(" ^ ");
                    break;
                case BinaryOperatorEnum.EndsWith:
                    sb.Append(" $ ");
                    break;
            }

            rightOperand.ToString(sb);
        }

		#region Membres de ICloneable

		public override object Clone()
		{
            BinaryOperator clone = new BinaryOperator();
			clone.Type = this.Type;

			clone.LeftOperand = this.LeftOperand == null ? null : (Constraint)this.LeftOperand.Clone();
			clone.RightOperand = this.RightOperand == null ? null : (Constraint)this.RightOperand.Clone();

			return clone;
		}

		#endregion

	}

	public enum BinaryOperatorEnum
	{
		Unknown,
		And,
		Or,
		NotEqual,
		LesserOrEqual,
		GreaterOrEqual,
		Lesser,
		Greater,
		Equal,
		Minus,
		Plus,
		Modulo,
		Div,
		Times,
		Contains,
		BeginsWith,
		EndsWith
	}
}
