using System;
using System.Text;

namespace Evaluant.OPath.Expressions
{
	/// <summary>
	/// Description rsume de Path.
	/// </summary>
	[Serializable]
    public class Identifier : IOPathElement, ICloneable
	{
		/// <summary>
		/// Creates a new <see cref="Identifier"/> instance.
		/// </summary>
		public Identifier() : base()
		{
			value = String.Empty;
		}

        public Identifier(string value, Constraint constraint)
        {
            this.value = value;
            this.constraint = constraint;
        }

        [System.Diagnostics.DebuggerStepThrough]
        public virtual void Accept(OPathVisitor visitor)
		{
			visitor.Visit(this);
		}

		private Constraint constraint;
		public Constraint Constraint
		{
			get { return constraint; }
			set { constraint = value; }
		}

		private string value;
		/// <summary>
		/// Gets or sets the value.
		/// </summary>
		/// <value></value>
		public string Value
		{
			get { return value; }
			set { this.value = value; }
		}

        public void ToString(StringBuilder sb)
        {
            sb.Append(this.Value);
            if (constraint != null)
            {
                sb.Append("[");
                constraint.ToString(sb);
                sb.Append("]");
            }
        }

		#region Membres de ICloneable

		public virtual object Clone()
		{
			Identifier clone = new Identifier();

			clone.Value = this.Value;
			clone.Constraint = this.Constraint == null ? null : (Constraint)this.Constraint.Clone();

			return clone;
			
		}

		#endregion
	}
}
