using System;

namespace Evaluant.OPath.Expressions
{
	/// <summary>
	/// Description rsume de Path.
	/// </summary>
	[Serializable]
    public class UnaryOperator : Expression, ICloneable
	{
		/// <summary>
		/// Creates a new <see cref="UnaryOperator"/> instance.
		/// </summary>
		public UnaryOperator() : base()
		{
			type = UnaryOperatorEnum.Unknown;
		}

        public UnaryOperator(UnaryOperatorEnum type, Constraint operand)
        {
            this.operand = operand;
            this.type = type;
        }

        [System.Diagnostics.DebuggerStepThrough]
        public override void Accept(OPathVisitor visitor)
		{
			visitor.Visit(this);
		}

		private Constraint operand;
		public Constraint Operand
		{
			get { return operand; }
			set { operand = value; }
		}

		private UnaryOperatorEnum type;
		/// <summary>
		/// Gets or sets the type.
		/// </summary>
		/// <value></value>
		public UnaryOperatorEnum Type
		{
			get { return type; }
			set { type = value; }
		}

        public override void ToString(System.Text.StringBuilder sb)
        {
            switch (type)
            {
                case UnaryOperatorEnum.Not:
                    sb.Append(" not ");
                    break;
                case UnaryOperatorEnum.Minus:
                    sb.Append(" - ");
                    break;
            }

            operand.ToString(sb);
        }

		#region Membres de ICloneable

		public override object Clone()
		{
			UnaryOperator clone = new UnaryOperator();
			clone.Type = this.Type;
			clone.Operand = this.Operand == null ? null : (Constraint)this.Operand.Clone();

			return clone;
		}

		#endregion
	}

	public enum UnaryOperatorEnum
	{
		Unknown,
		Not,
		Minus
	}
}
