using System;
using System.Collections;
using System.Collections.Specialized;

namespace Evaluant.Uss.ObjectContext
{
    /// <summary>
    /// Description rsume de Query.
    /// </summary>
    [Serializable]
    public class Query
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="Query"/> class.
        /// </summary>
        /// <param name="from">The type where the query starts from.</param>
        /// <param name="constraint">The constraint.</param>
        public Query(Type from, string constraint)
        {
            if (from == null)
                throw new ArgumentNullException("from");

            if (constraint == null)
                throw new ArgumentNullException("constraint");

            _fromType = from;
            _constraint = constraint;
            _firstResult = 1;
            _maxResults = 0;

        }

        private string _constraint;

        public string Constraint
        {
            get { return _constraint; }
            set { _constraint = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Query"/> class.
        /// </summary>
        /// <param name="from">The type where the query starts from.</param>
        public Query(Type from)
            : this(from, String.Empty)
        {
        }

        private int _maxResults;

        /// <summary>
        /// Gets or sets the max results.
        /// </summary>
        /// <value>The max results.</value>
        /// <remarks>If set to zero, all the entities will be loaded</remarks>
        public int MaxResults
        {
            get { return _maxResults; }
            set { _maxResults = value; }
        }

        private int _firstResult;

        /// <summary>
        /// Gets or sets the first result.
        /// </summary>
        /// <value>The first result.</value>
        public int FirstResult
        {
            get { return _firstResult; }
            set { _firstResult = value; }
        }

        private Hashtable _parameters = new Hashtable();

        /// <summary>
        /// Sets the parameter.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="value">The value.</param>
        /// <returns>The current Query instance</returns>
        public Query SetParameter(string name, object value)
        {
            _parameters[name] = value;
            return this;
        }

        private StringCollection _inferredAttributes = new StringCollection();

        /// <summary>
        /// Gets the inferred attributes.
        /// </summary>
        /// <value>The inferred attributes.</value>
        /// <remarks>If no item is specified all the attributes are loaded</remarks>
        public StringCollection InferredAttributes
        {
            get { return _inferredAttributes; }
        }

        private StringCollection _inferredRelationships = new StringCollection();

        /// <summary>
        /// Gets the inferred relationships.
        /// </summary>
        /// <value>The inferred relations.</value>
        /// <remarks>If no item is specified no relation will be loaded</remarks>
        public StringCollection InferredRelationships
        {
            get { return _inferredRelationships; }
        }

        private StringCollection _orderBy = new StringCollection();

        /// <summary>
        /// Gets the order by fields.
        /// </summary>
        /// <value>The order by.</value>
        public StringCollection OrderBy
        {
            get { return _orderBy; }
        }

        private Type _fromType = null;

        /// <summary>
        /// Gets the entrypoint type.
        /// </summary>
        /// <value>The entrypoint type.</value>
        public Type FromType
        {
            get { return _fromType; }
        }
    }

    public class OQuery : Query
    {
        public OQuery(Type from) : base(from) { }

        public OQuery(Type from, string constraint)
            : base(from, constraint)
        {
            query = new Evaluant.OPath.OPathQuery(ObjectContext.ConvertNamespaceDomainToEuss(from, constraint));
        }

        protected bool compiled = false;

        protected OPath.OPathQuery query;
        public OPath.OPathQuery Query
        {
            get
            {
                if (!compiled && query != null)
                    query.Compile();
                return query;
            }
            set { query = value; }
        }
    }
}
