using System.Data;
using System.Data.SqlTypes;

namespace Evaluant.Uss.SqlMapper
{
	/// <summary>
	/// Description rsume de DataFactory.
	/// </summary>
	/// 
	public enum Algo {Where, Embeded, Linear}


	public interface IDriver
	{
		/// <summary>
		/// The Type used to create an IDbConnection
		/// </summary>
		System.Type ConnectionType { get; }

		/// <summary>
		/// The Type used to create an IDbCommand
		/// </summary>
		System.Type CommandType { get; }

		/// <summary>
		/// The Type used to create an IDbDataParameter
		/// </summary>
		System.Type ParameterType { get; }

		/// <summary>
		/// Creates an uninitialized IDbConnection object for the specific Driver
		/// </summary>
		IDbConnection CreateConnection();
		IDbConnection CreateConnection(string connStr);

		/// <summary>
		/// Creates an empty IDbCommand object for the specific Driver
		/// </summary>
		/// <remarks>
		/// The reason for having this method is that Interfaces in ADO.NET require 
		/// the use of a IDbCommand.CreateCommand - when we are making the IDbCommand
		/// objects we might not have a particular connection to create the commands
		/// from.
		/// </remarks>
		IDbCommand CreateCommand();
        IDbCommand CreateCommand(string query, IDbConnection connection);
        IDbCommand CreateCommand(string query, IDbConnection connection, CommandType type);
        IDbCommand CreateCommand(string query, IDbConnection connection, IDbTransaction transaction);

		/// <summary>
		/// Does this Driver require the use of the Named Prefix when trying
		/// to reference the Parameter in the Command's Parameter collection.  
		/// </summary>
		/// <remarks>
		/// This is really only useful when the UseNamedPrefixInSql == true.  When this is true the
		/// code will look like:
		/// <code>IDbParameter param = cmd.Parameters["@paramName"]</code>
		/// if this is false the code will be 
		/// <code>IDbParameter param = cmd.Parameters["paramName"]</code>.
		/// </remarks>
		bool UseNamedPrefixInParameter { get; }

		/// <summary>
		/// The Named Prefix for parameters.  
		/// </summary>
		/// <remarks>
		/// Sql Server uses <c>"@"</c> and Oracle uses <c>":"</c>.
		/// </remarks>
		string NamedPrefix { get; }

		/// <summary>
		/// Changes the parameterName into the correct format for an IDbParameter
		/// for the Driver.
		/// </summary>
		/// <remarks>
		/// For SqlServerConnectionProvider it will change <c>id</c> to <c>@id</c>
		/// </remarks>
		/// <param name="parameterName">The unformatted name of the parameter</param>
		/// <returns>A parameter formatted for an IDbParameter.</returns>
		string FormatParameter( string parameterName );

		/// <summary>
		/// Changes the parameterName into the correct format for an IDbParameter
		/// for the Driver.
		/// </summary>
		/// <remarks>
		/// For SqlServerConnectionProvider it will change <c>id</c> to <c>@id</c>
		/// </remarks>
		/// <param name="tableAlias">The Alias for the Table.</param>
		/// <param name="parameterName">The unformatted name of the parameter</param>
		/// <returns>A parameter formatted for an IDbParameter.</returns>
		string FormatParameter( string tableAlias, string parameterName );

		IDbDataParameter CreateParameter();
      
        IDbDataParameter CreateParameter(string name, DbType dbtype);
		IDbDataParameter CreateParameter(string name, DbType dbtype, object value);
        IDbDataParameter CreateParameter(string name, object value, DbType dbtype, ParameterDirection direction);

		IDbDataParameter CreateParameter(string name, object value, DbType dbtype, int size);
		IDbDataParameter CreateParameter(string name, object value, DbType dbtype, byte precision, byte scale);

		Algo GetAlgorithm();

        DBDialect Dialect { get; set;}

        void PreProcessQuery(IDbCommand command);

        IDbTransaction BeginTransaction(IDbConnection connection);
    }
}
