using System;
using System.Collections;

namespace Evaluant.Uss.Common
{
	/// <summary>
	///     <para>
	///       A collection that stores <see cref="AttributeDescriptor"/> objects.
	///    </para>
	/// </summary>
	/// <seealso cref="AttributeDescriptorCollection"/>
	[Serializable()]
	public class AttributeDescriptorCollection : CollectionBase 
	{
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref="AttributeDescriptorCollection"/>.
		///    </para>
		/// </summary>
		public AttributeDescriptorCollection()
		{
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref="AttributeDescriptorCollection"/> based on another <see cref="AttributeDescriptorCollection"/>.
		///    </para>
		/// </summary>
		/// <param name="value">
		///       A <see cref="AttributeDescriptorCollection"/> from which the contents are copied
		/// </param>
		public AttributeDescriptorCollection(AttributeDescriptorCollection value)
		{
			this.AddRange(value);
		}
		
		/// <summary>
		///     <para>
		///       Initializes a new instance of <see cref="AttributeDescriptorCollection"/> containing any array of <see cref="AttributeDescriptor"/> objects.
		///    </para>
		/// </summary>
		/// <param name="value">
		///       A array of <see cref="AttributeDescriptor"/> objects with which to intialize the collection
		/// </param>
		public AttributeDescriptorCollection(AttributeDescriptor[] value)
		{
			this.AddRange(value);
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref="AttributeDescriptor"/>.</para>
		/// </summary>
		/// <param name="index"><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		///    <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref="System.ArgumentOutOfRangeException"><paramref name="index"/> is outside the valid range of indexes for the collection.</exception>
		public AttributeDescriptor this[int index] 
		{
			get 
			{
				return ((AttributeDescriptor)(List[index]));
			}
			set 
			{
				List[index] = value;
			}
		}
		
		/// <summary>
		///    <para>Adds a <see cref="AttributeDescriptor"/> with the specified value to the 
		///    <see cref="AttributeDescriptorCollection"/> .</para>
		/// </summary>
		/// <param name="value">The <see cref="AttributeDescriptor"/> to add.</param>
		/// <returns>
		///    <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref="AttributeDescriptorCollection.AddRange"/>
		public int Add(AttributeDescriptor value)
		{
			return List.Add(value);
		}
		
		/// <summary>
		/// <para>Copies the elements of an array to the end of the <see cref="AttributeDescriptorCollection"/>.</para>
		/// </summary>
		/// <param name="value">
		///    An array of type <see cref="AttributeDescriptor"/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref="AttributeDescriptorCollection.Add"/>
		public void AddRange(AttributeDescriptor[] value)
		{
			for (int i = 0; i < value.Length; i++) 
			{
				this.Add(value[i]);
			}
		}
		
		/// <summary>
		///     <para>
		///       Adds the contents of another <see cref="AttributeDescriptorCollection"/> to the end of the collection.
		///    </para>
		/// </summary>
		/// <param name="value">
		///    A <see cref="AttributeDescriptorCollection"/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <seealso cref="AttributeDescriptorCollection.Add"/>
		public void AddRange(AttributeDescriptorCollection value)
		{
			for (int i = 0; i < value.Count; i++)
			{
				this.Add(value[i]);
			}
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		///    <see cref="AttributeDescriptorCollection"/> contains the specified <see cref="AttributeDescriptor"/>.</para>
		/// </summary>
		/// <param name="value">The <see cref="AttributeDescriptor"/> to locate.</param>
		/// <returns>
		/// <para><see langword="true"/> if the <see cref="AttributeDescriptor"/> is contained in the collection; 
		///   otherwise, <see langword='false"/>.</para>
		/// </returns>
		/// <seealso cref="AttributeDescriptorCollection.IndexOf"/>
		public bool Contains(AttributeDescriptor value)
		{
			return List.Contains(value);
		}
		
		/// <summary>
		/// <para>Copies the <see cref="AttributeDescriptorCollection"/> values to a one-dimensional <see cref="System.Array"/> instance at the 
		///    specified index.</para>
		/// </summary>
		/// <param name="array"><para>The one-dimensional <see cref="System.Array"/> that is the destination of the values copied from <see cref="AttributeDescriptorCollection"/> .</para></param>
		/// <param name="index">The index in <paramref name="array"/> where copying begins.</param>
		/// <returns>
		///   <para>None.</para>
		/// </returns>
		/// <exception cref="System.ArgumentException"><para><paramref name="array"/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref="AttributeDescriptorCollection"/> is greater than the available space between <paramref name="index"/> and the end of <paramref name="array"/>.</para></exception>
		/// <exception cref="System.ArgumentNullException"><paramref name="array"/> is <see langword="null"/>. </exception>
		/// <exception cref="System.ArgumentOutOfRangeException"><paramref name="index"/> is less than <paramref name="array"/>'s lowbound. </exception>
		/// <seealso cref="System.Array"/>
		public void CopyTo(AttributeDescriptor[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		///    <para>Returns the index of a <see cref="AttributeDescriptor"/> in 
		///       the <see cref="AttributeDescriptorCollection"/> .</para>
		/// </summary>
		/// <param name="value">The <see cref="AttributeDescriptor"/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref="AttributeDescriptor"/> of <paramref name="value"/> in the 
		/// <see cref="AttributeDescriptorCollection"/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref="AttributeDescriptorCollection.Contains"/>
		public int IndexOf(AttributeDescriptor value)
		{
			return List.IndexOf(value);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref="AttributeDescriptor"/> into the <see cref="AttributeDescriptorCollection"/> at the specified index.</para>
		/// </summary>
		/// <param name="index">The zero-based index where <paramref name="value"/> should be inserted.</param>
		/// <param name="value">The <see cref="AttributeDescriptor"/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref="AttributeDescriptorCollection.Add"/>
		public void Insert(int index, AttributeDescriptor value)
		{
			List.Insert(index, value);
		}
		
		/// <summary>
		///    <para>Returns an enumerator that can iterate through 
		///       the <see cref="AttributeDescriptorCollection"/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref="System.Collections.IEnumerator"/>
		public new AttributeDescriptorEnumerator GetEnumerator()
		{
			return new AttributeDescriptorEnumerator(this);
		}
		
		/// <summary>
		///    <para> Removes a specific <see cref="AttributeDescriptor"/> from the 
		///    <see cref="AttributeDescriptorCollection"/> .</para>
		/// </summary>
		/// <param name="value">The <see cref="AttributeDescriptor"/> to remove from the <see cref="AttributeDescriptorCollection"/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref="System.ArgumentException"><paramref name="value"/> is not found in the Collection. </exception>
		public void Remove(AttributeDescriptor value)
		{
			List.Remove(value);
		}
		
		public class AttributeDescriptorEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public AttributeDescriptorEnumerator(AttributeDescriptorCollection mappings)
			{
				this.temp = ((IEnumerable)(mappings));
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public AttributeDescriptor Current 
			{
				get 
				{
					return ((AttributeDescriptor)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}
}