using System;
using Evaluant.Uss.MetaData;
using Evaluant.Uss.Models;

namespace Evaluant.Uss.Common
{
    class ModelMetaDataVisitor : IMetaDataVisitor
    {
        private Model _Model;

        public ModelMetaDataVisitor(Model model)
        {
            _Model = model;
        }

        public void Process(ReferenceMetaData reference)
        {
            Models.Entity entity = _Model.GetEntity(reference.ParentType);

            if (entity == null)
            {
                entity = new Models.Entity(reference.ParentType);
                _Model.Entities.Add(entity);
            }

            Reference mref = _Model.GetReference(reference.ParentType, reference.Name);

            if (mref != null && reference.Ignore)
            {
                entity.References.Remove(mref);
                _Model.ClearReferenceCache();
                return;
            }

            if (mref == null)
            {
                mref = new Reference(reference.Name, reference.ParentType, reference.ChildType, reference.IsComposition, reference.FromMany, reference.ToMany);
                entity.References.Add(mref);
            }

            // To override a previous value
            if (mref.IsComposition != reference.IsComposition)
                mref.IsComposition = reference.IsComposition;

            if (mref.FromMany != reference.FromMany)
                mref.FromMany = reference.FromMany;

            if (mref.ToMany != reference.ToMany)
                mref.ToMany = reference.ToMany;

            if (mref.ChildType != reference.ChildType)
                mref.ChildType = reference.ChildType;
        }

        void Evaluant.Uss.MetaData.IMetaDataVisitor.Process(InterfaceMetadata interf)
        {
            Models.Entity entity = _Model.GetEntity(interf.Name);

            if (entity == null)
            {
                entity = new Models.Entity(interf.Name);
                _Model.Entities.Add(entity);
            }

            entity.IsInterface = true;
        }

        void Evaluant.Uss.MetaData.IMetaDataVisitor.Process(TypeMetaData type)
        {
            Models.Entity entity = _Model.GetEntity(type.Name);

            if (type.Ignore)
            {
                if (entity != null)
                {
                    _Model.Entities.Remove(entity);
                    _Model.ClearEntityCache();
                }

                return;
            }

            if (entity == null)
            {
                entity = new Models.Entity(type.Name);
                if (!type.Ignore)
                    _Model.Entities.Add(entity);
            }

            if (type.IsInterface)
            {
                if (entity.Implement != null && entity.Implement != String.Empty)
                    entity.Implement += ",";

                entity.Implement += type.ParentName;

                Models.Entity parent = _Model.GetEntity(type.ParentName);

                if (parent == null)
                {
                    parent = new Models.Entity(type.ParentName);
                    _Model.Entities.Add(parent);
                }

                parent.IsInterface = true;
            }
            else
            {
                // To override a previous value
                if (type.ParentName != String.Empty && type.ParentName != null && entity.Inherit != type.ParentName)
                    entity.Inherit = type.ParentName;
            }


        }

        void Evaluant.Uss.MetaData.IMetaDataVisitor.Process(PropertyMetaData property)
        {
            Models.Entity entity = _Model.GetEntity(property.Type);

            if (entity == null)
            {
                entity = new Models.Entity(property.Type);
                _Model.Entities.Add(entity);
            }

            Models.Attribute attribute = _Model.GetAttribute(property.Type, property.PropertyName);

            if (attribute != null && property.Ignore)
            {
                entity.Attributes.Remove(attribute);
                _Model.ClearAttributeCache();
                return;
            }

            if (attribute == null)
            {
                attribute = new Models.Attribute(property.PropertyName, property.PropertyType);
                attribute.Values = property.Values;
                entity.Attributes.Add(attribute);
            }

            // To override a previous value
            if (attribute.Type != property.PropertyType)
                attribute.Type = property.PropertyType;

            if (attribute.Values != property.Values)
                attribute.Values = property.Values;
        }
    }
}
