using System;
using System.Collections;

using Evaluant.OPath;

namespace Evaluant.Uss
{
	/// <summary>
	/// Description rsume de NavigationComparer.
	/// </summary>
	public class NavigationComparer : IComparer
	{
		OPathQuery[] _Queries;
		Navigator _Navigator;
		int[] _Ascendings;


		public NavigationComparer(string[] pathes, bool[] ascendings)
		{
			ArrayList queries = new ArrayList(pathes.Length);
			ArrayList ascs = new ArrayList(ascendings.Length);

			foreach(string path in pathes)
			{
				OPathQuery query = new OPathQuery(String.Format("this[{0}]", path));
				query.Compile(false); // don't optimize not to convert this[Name] in this[Name = true]
				queries.Add(query);
			}

			foreach(bool ascending in ascendings)
				ascs.Add(ascending ? 1 : -1);

			_Queries = (OPathQuery[])queries.ToArray(typeof(OPathQuery));
			_Ascendings = (int[])ascs.ToArray(typeof(int));

			_Navigator = new Navigator();
		}

		public NavigationComparer(string[] pathes)
		{
			ArrayList queries = new ArrayList(pathes.Length);
			ArrayList ascendings = new ArrayList(pathes.Length);

			foreach(string path in pathes)
			{
				OPathQuery query = new OPathQuery(String.Format("this[{0}]", path));
                query.Compile(false); // don't optimize not to convert this[Name] in this[Name = true]
				queries.Add(query);
				ascendings.Add(1);
			}

			_Queries = (OPathQuery[])queries.ToArray(typeof(OPathQuery));
			_Ascendings = (int[])ascendings.ToArray(typeof(int));

			_Navigator = new Navigator();
		}

		public NavigationComparer(string path, bool ascending) : this(new string[] { path }, new bool[] { ascending })
		{
		}

		public NavigationComparer(string path) : this(new string[] { path }, new bool[] { true })
		{
		}

		#region Membres de IComparer

		public int Compare(object x, object y)
		{
			if(x == y)
				return 0;

			if( x as Entity == null || y as Entity == null)
				throw new ArgumentException("The compared items must be of type Entity");

			for(int i=0; i<_Queries.Length; i++)
			{
				_Navigator.Roots.Clear();
				_Navigator.Roots.Add((Entity)x);
				object xValue = _Navigator.Evaluate(_Queries[i].Path);

				_Navigator.Roots.Clear();
				_Navigator.Roots.Add((Entity)y);
				object yValue = _Navigator.Evaluate(_Queries[i].Path);

				if(xValue == null && yValue == null)
					continue;

				if(xValue == null)
					return -1 * _Ascendings[i] ;

				if(yValue == null)
					return _Ascendings[i];

				int compare = Comparer.Default.Compare(xValue, yValue.GetType() == xValue.GetType() ? yValue : Convert.ChangeType(yValue, xValue.GetType()));

				// Goes to the next comparison criteria if they are same
				if(compare == 0)
					continue;

				return _Ascendings[i] * compare;
			}

			return 0;
		}

		#endregion
	}
}
