using System;
using System.Collections.Generic;
using System.Text;

using DslModeling = global::Microsoft.VisualStudio.Modeling;
using DslDiagrams = global::Microsoft.VisualStudio.Modeling.Diagrams;

namespace Evaluant.EussDSL
{
    public partial class EntityReferencesTargets : DslModeling::ElementLink
    {
        /// <summary>
        /// Constructor
        /// Creates a EntityReferencesTargets link in the same Partition as the given Entity
        /// </summary>
        /// <param name="source">Entity to use as the source of the relationship.</param>
        /// <param name="target">Entity to use as the target of the relationship.</param>
        public EntityReferencesTargets(Entity source, Entity target)
            : base((source != null ? source.Partition : null), new DslModeling::RoleAssignment[] { new DslModeling::RoleAssignment(EntityReferencesTargets.SourceDomainRoleId, source), new DslModeling::RoleAssignment(EntityReferencesTargets.TargetDomainRoleId, target) }, null)
        {
            
            //utilis lors de la cration initiale

            //on remplit avec un nom par dfaut
            if (!string.IsNullOrEmpty(target.type))
            {
                name = target.type.Substring(target.type.LastIndexOf(':') + 1) + "s";
            }

            // on essaie de remplir parentField avec le premier ID de l'entit source s'il existe
            if (source.Identifiers != null && source.Identifiers.Count > 0 && !string.IsNullOrEmpty(source.Identifiers[0].field))
            {
                parentField = source.Identifiers[0].field;
            }

            // on remplit par dfaut childTable avec la table de l'entit enfant
            if (!string.IsNullOrEmpty(target.table))
            {
                childTable = target.table;
            }

            // on essaie de remplir childField 
            if (source.Identifiers != null && source.Identifiers.Count > 0 && !string.IsNullOrEmpty(source.Identifiers[0].field))
            {
                childField = "FK_" + source.Identifiers[0].field;
            }

            // on essaie de remplir parentTable
            if (!string.IsNullOrEmpty(source.table))
            {
                parentTable = source.table;
            }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="store">Store where new link is to be created.</param>
        /// <param name="roleAssignments">List of relationship role assignments.</param>
        public EntityReferencesTargets(DslModeling::Store store, params DslModeling::RoleAssignment[] roleAssignments)
            : base(store != null ? store.DefaultPartition : null, roleAssignments, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="store">Store where new link is to be created.</param>
        /// <param name="roleAssignments">List of relationship role assignments.</param>
        /// <param name="propertyAssignments">List of properties assignments to set on the new link.</param>
        public EntityReferencesTargets(DslModeling::Store store, DslModeling::RoleAssignment[] roleAssignments, DslModeling::PropertyAssignment[] propertyAssignments)
            : base(store != null ? store.DefaultPartition : null, roleAssignments, propertyAssignments)
        {
            //utilis lors du chargemenr
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="partition">Partition where new link is to be created.</param>
        /// <param name="roleAssignments">List of relationship role assignments.</param>
        public EntityReferencesTargets(DslModeling::Partition partition, params DslModeling::RoleAssignment[] roleAssignments)
            : base(partition, roleAssignments, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="partition">Partition where new link is to be created.</param>
        /// <param name="roleAssignments">List of relationship role assignments.</param>
        /// <param name="propertyAssignments">List of properties assignments to set on the new link.</param>
        public EntityReferencesTargets(DslModeling::Partition partition, DslModeling::RoleAssignment[] roleAssignments, DslModeling::PropertyAssignment[] propertyAssignments)
            : base(partition, roleAssignments, propertyAssignments)
        {
            
        }

        internal sealed partial class parentFieldPropertyHandler
        {
            protected override void OnValueChanged(EntityReferencesTargets element, string oldValue, string newValue)
            {
                base.OnValueChanged(element, oldValue, newValue);
            }
        }
    }
}
