using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Modeling;

namespace Evaluant.EussDSL
{
    public class InheritanceDetector
    {
        /// <summary>
        /// Detects the inheritance state concerning an Entity, removing obsolete inheritance 
        /// relations and adding the valid new ones.
        /// </summary>
        /// <param name="element"></param>
        public static void DetectAndShow(Entity element)
        {
            // mode : type d'hritage :
            // 0 : table per class hierarchy
            // 1 : table per subclass
            // 2 : table per concrete class

            // liste des formes d'entites concernees par l'ventuel nouvel hritage
            List<EntityShape> les = new List<EntityShape>();
            // la forme d'entit initiatrice de l'action (cad la forme associe au parametre "element")
            EntityShape esConcerned = null;
            foreach (ModelElement me in element.Store.ElementDirectory.AllElements)
            {
                if (me is EntityShape)
                {
                    Entity e = (Entity)(((EntityShape)(me)).ModelElement);
                    // pour qu'il y ait heritage, il faut que "e" et "element" ait la mme table et les memes identifiers
                    if (e.table == element.table) // meme table
                    {
                        bool sameIdentifiers = true;
                        if (e.Identifiers.Count == element.Identifiers.Count)
                        {
                            for (int i = 0; i < element.Identifiers.Count; i++)
                            {
                                if (element.Identifiers[i].field != e.Identifiers[i].field)
                                {
                                    sameIdentifiers = false;
                                }
                            }
                            if (sameIdentifiers)
                            {
                                les.Add((EntityShape)me);
                            }
                        }
                    }
                    if (((EntityShape)(me)).ModelElement == element)
                    {
                        esConcerned = (EntityShape)me;
                    }
                }
            }

            // suppression prealable des liens d'heritage qui existent
            foreach (EntityShape es in les)
            {
                for (int i = 0; i < es.Link.Count; i++)
                {
                    if (es.Link[i] is InheritanceConnector)
                    {
                        es.Link.RemoveAt(i);
                        i--;
                    }
                }
            }

            // on cherche l'entite parente de l'heritage, c'est a dire l'entite pour laquelle le type est le meme que la table
            int parentIndex = -1;
            for (int i = 0; i < les.Count; i++)
            {
                if (((Entity)(les[i].ModelElement)).type.EndsWith(element.table)
                    || ((Entity)(les[i].ModelElement)).type.EndsWith(":" + element.table))
                {
                    parentIndex = i;
                }
            }

            if (parentIndex != -1)
            {

                // ------------------------------------------
                // ---  dtermination du mode d'hritage  ---
                // ------------------------------------------

                // pour avoir mode = 0, il faut que les entits concernes aient tous le mme type de generator
                // et qu'il y ait toujours le meme discriminator, et qu'il soit renseign
                bool mode0 = true;
                for (int i = 0; i < les.Count; i++)
                {
                    Entity e_tmp = (Entity)(les[i].ModelElement);
                    if (i != 0)
                    {
                        for (int j = 0; j < e_tmp.Identifiers.Count; j++)
                        {
                            if (e_tmp.Identifiers[j].GeneratorName != ((Entity)les[0].ModelElement).Identifiers[j].GeneratorName)
                            {
                                mode0 = false;
                            }
                        }
                        if (e_tmp.discriminatorField != ((Entity)les[0].ModelElement).discriminatorField)
                        {
                            mode0 = false;
                        }
                    }
                    if(!string.IsNullOrEmpty( e_tmp.copyNode) || string.IsNullOrEmpty(e_tmp.discriminatorField))
                    {
                        mode0 = false;
                    }
                }

                // pour avoir mode = 1, il faut que toutes les entits concernes (sauf la racine) aient un generator "inherited", que "discriminator"
                // et "copyNode" ne soient pas renseigns
                bool mode1 = true;
                for (int i = 0; i < les.Count; i++)
                {
                    Entity e_tmp = (Entity)(les[i].ModelElement);
                    if (i != parentIndex)
                    {
                        for (int j = 0; j < e_tmp.Identifiers.Count; j++)
                        {
                            if (e_tmp.Identifiers[j].GeneratorName.ToLower() != "inherited")
                            {
                                mode1 = false;
                            }
                        }
                    }
                    if (!string.IsNullOrEmpty(e_tmp.discriminatorField) || !string.IsNullOrEmpty(e_tmp.copyNode))
                    {
                        mode1 = false;
                    }
                }

                // pour avoir mode = 2, il faut que toutes les entits concernes aient le mme generator, et que 
                // tous sauf la racine aient "copyNode" renseign, et "discriminator" non renseign
                bool mode2 = true;
                for (int i = 0; i < les.Count; i++)
                {
                    Entity e_tmp = (Entity)(les[i].ModelElement);
                    if (i != 0)
                    {
                        for (int j = 0; j < e_tmp.Identifiers.Count; j++)
                        {
                            if (e_tmp.Identifiers[j].GeneratorName != ((Entity)les[0].ModelElement).Identifiers[j].GeneratorName)
                            {
                                mode2 = false;
                            }
                        }
                    }
                    if (!string.IsNullOrEmpty(e_tmp.discriminatorField))
                    {
                        mode2 = false;
                    }
                    if (i != parentIndex && string.IsNullOrEmpty(e_tmp.copyNode))
                    {
                        mode2 = false;
                    }
                }

                if (mode0 || mode1 || mode2)
                {
                    for (int i = 0; i < les.Count; i++)
                    {
                        if (i != parentIndex)
                        {
                            EntityShape src = les[i];
                            EntityShape dest = les[parentIndex];

                            bool relationAlreadyExists = false;

                            for (int j = 0; j < src.Link.Count; j++)
                            {
                                if (src.Link[j] is InheritanceConnector)
                                {
                                    if (((InheritanceConnector)src.Link[j]).ToShape == dest)
                                    {
                                        relationAlreadyExists = true;
                                    }
                                }
                            }

                            if (!relationAlreadyExists)
                            {
                                InheritanceConnector icNew = new InheritanceConnector(element.Partition);
                                if (mode0)
                                {
                                    icNew.Type = "Table per class hierarchy";
                                }
                                if (mode1)
                                {
                                    icNew.Type = "Table per subclass";
                                }
                                if (mode2)
                                {
                                    icNew.Type = "Table per concrete class";
                                }
                                icNew.Connect(src, dest);
                                src.ParentShape.NestedChildShapes.Add(icNew);
                                icNew.Show();
                            }
                        }
                    }
                }
            }
            // else ... <-- TODO : que faire ici?

        }
    }
}
