using System;
using System.Collections.Generic;
using System.Text;

using DslModeling = global::Microsoft.VisualStudio.Modeling;
using DslDiagrams = global::Microsoft.VisualStudio.Modeling.Diagrams;

namespace Evaluant.EussDSL
{
    /// <summary>
    /// Redefinition of the serialization for the properties of the generator of an entity
    /// </summary>
    public partial class PropertySerializer
    {
        private void CustomConstructor()
        {
            DefaultConstructor();
        }

        private string CustomXmlTagName
        {
            get { return DefaultXmlTagName; }
        }

        private void CustomRead(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            //DefaultRead(serializationContext, element, reader);

            #region Check Parameters
            global::System.Diagnostics.Debug.Assert(serializationContext != null);
            if (serializationContext == null)
                throw new global::System.ArgumentNullException("serializationContext");
            global::System.Diagnostics.Debug.Assert(element != null);
            if (element == null)
                throw new global::System.ArgumentNullException("element");
            global::System.Diagnostics.Debug.Assert(reader != null);
            if (reader == null)
                throw new global::System.ArgumentNullException("reader");
            #endregion

            // Read properties serialized as XML attributes.

            //// version initiale
            //DefaultReadPropertiesFromAttributes(serializationContext, element, reader);

            Property instanceOfProperty = element as Property;
            global::System.Diagnostics.Debug.Assert(instanceOfProperty != null, "Expecting an instance of Property");

            // Name
            if (!serializationContext.Result.Failed)
            {
                string attribName = reader.GetAttribute("name");
                if (attribName != null)
                {
                    global::System.String valueOfName;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribName), out valueOfName))
                    {
                        instanceOfProperty.Name = valueOfName;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "name", typeof(global::System.String), attribName);
                    }
                }
            }
            // value
            if (!serializationContext.Result.Failed)
            {
                //string attribvalue = reader.GetAttribute("value");
                string attribvalue = reader.ReadElementContentAsString(); //reader.GetAttribute("value");
                if (attribvalue != null)
                {
                    global::System.String valueOfvalue;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribvalue), out valueOfvalue))
                    {
                        instanceOfProperty.value = valueOfvalue;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "value", typeof(global::System.String), attribvalue);
                    }
                }
            }

            // Read nested XML elements.
            if (!serializationContext.Result.Failed)
            {
                if (!reader.IsEmptyElement)
                {
                    // Read to the start of the first child element.
                    DslModeling::SerializationUtilities.SkipToFirstChild(reader);

                    // Read nested XML elements, they can be either properties serialized as XML elements, or child 
                    // model elements.
                    while (!serializationContext.Result.Failed && !reader.EOF && reader.NodeType == global::System.Xml.XmlNodeType.Element)
                    {
                        DefaultReadElements(serializationContext, element, reader);
                        if (!serializationContext.Result.Failed && !reader.EOF && reader.NodeType == global::System.Xml.XmlNodeType.Element)
                        {
                            // Encountered one unknown XML element, skip it and keep reading.
                            EussDSLSerializationBehaviorSerializationMessages.UnexpectedXmlElement(serializationContext, reader);
                            DslModeling::SerializationUtilities.Skip(reader);
                        }
                    }
                }
            }

            // Advance the reader to the next element (open tag of the next sibling, end tag of the parent, or EOF)
            DslModeling::SerializationUtilities.Skip(reader);
        }

        private static void CustomReadPropertiesFromAttributes(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            DefaultReadPropertiesFromAttributes(serializationContext, element, reader);
        }

        private static void CustomReadElements(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            DefaultReadElements(serializationContext, element, reader);
        }

        private DslModeling::ModelElement CustomTryCreateInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::Partition partition)
        {
            return DefaultTryCreateInstance(serializationContext, reader, partition);
        }

        private DslModeling::ModelElement CustomCreateInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::Partition partition)
        {
            return DefaultCreateInstance(serializationContext, reader, partition);
        }

        private DslModeling::Moniker CustomTryCreateMonikerInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::ModelElement sourceRolePlayer, global::System.Guid relDomainClassId, DslModeling::Partition partition)
        {
            return DefaultTryCreateMonikerInstance(serializationContext, reader, sourceRolePlayer, relDomainClassId, partition);
        }

        private DslModeling::Moniker CustomCreateMonikerInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::ModelElement sourceRolePlayer, global::System.Guid relDomainClassId, DslModeling::Partition partition)
        {
            return DefaultCreateMonikerInstance(serializationContext, reader, sourceRolePlayer, relDomainClassId, partition);
        }

        private void CustomWrite(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer, DslModeling::RootElementSettings rootElementSettings)
        {
            //DefaultWrite(serializationContext, element, writer, rootElementSettings);

            #region Check Parameters
            global::System.Diagnostics.Debug.Assert(serializationContext != null);
            if (serializationContext == null)
                throw new global::System.ArgumentNullException("serializationContext");
            global::System.Diagnostics.Debug.Assert(element != null);
            if (element == null)
                throw new global::System.ArgumentNullException("element");
            global::System.Diagnostics.Debug.Assert(writer != null);
            if (writer == null)
                throw new global::System.ArgumentNullException("writer");
            #endregion


            // Write start of element, including schema target namespace if specified.
            if (rootElementSettings != null && !string.IsNullOrEmpty(rootElementSettings.SchemaTargetNamespace))
                writer.WriteStartElement(this.XmlTagName, rootElementSettings.SchemaTargetNamespace);
            else
                writer.WriteStartElement(this.XmlTagName);

            // Write version info (in the format 1.2.3.4), if necessary
            if (rootElementSettings != null && rootElementSettings.Version != null)
                writer.WriteAttributeString("dslVersion", rootElementSettings.Version.ToString(4));

            // au lieu d'ecrire <Property name="xxx" value="yyy" />
            // on va ecrire <Property name="xxx">yyy</Property>

            //// version par dfaut
            //DefaultWritePropertiesAsAttributes(serializationContext, element, writer);

            Property instanceOfProperty = element as Property;
            global::System.Diagnostics.Debug.Assert(instanceOfProperty != null, "Expecting an instance of Property");

            // Name
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfProperty.Name;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("name", propValue);
                }
            }

            // value
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfProperty.value;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteString(propValue);
                        //writer.WriteAttributeString("value", propValue); // version initiale
                }
            }


            if (!serializationContext.Result.Failed)
            {
                // Write 1) properties serialized as nested XML elements and 2) child model elements into XML.
                DefaultWriteElements(serializationContext, element, writer);
            }

            writer.WriteEndElement();
        }

        private static void CustomWritePropertiesAsAttributes(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer)
        {
            DefaultWritePropertiesAsAttributes(serializationContext, element, writer);
        }

        private static void CustomWriteElements(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer)
        {
            DefaultWriteElements(serializationContext, element, writer);
        }

        private string CustomCalculateQualifiedName(DslModeling::DomainXmlSerializerDirectory directory, DslModeling::ModelElement element)
        {
            return DefaultCalculateQualifiedName(directory, element);
        }

        private string CustomGetMonikerQualifier(DslModeling::DomainXmlSerializerDirectory directory, DslModeling::ModelElement element)
        {
            return DefaultGetMonikerQualifier(directory, element);
        }
    }
}
