using System;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss;


namespace NUnit.Mapping.Interfaces
{
	/// <summary>
	/// Description rsume de Proxy.
	/// </summary>
	[TestClass]
	public abstract class Fixture
	{
        protected ObjectContext oc;
        protected Evaluant.Uss.SqlMapper.SqlMapperProvider provider = new Evaluant.Uss.SqlMapper.SqlMapperProvider();

        public Fixture()
        {
            
            provider.ConnectionString = "Server=.;Database=uss;UID=sa;PWD=";
            provider.Dialect = "Evaluant.Uss.SqlMapper.MsSqlDialect";
            provider.Driver = "Evaluant.Uss.SqlMapper.MsSqlDriver";
            provider.TablePrefix = "INT_";
            provider.RegisterMetaData(MetaDataFactory.FromAssembly(typeof(INameable).Assembly));

            Configure();
            provider.InitializeConfiguration();

            oc = new ObjectService(provider).CreateObjectContext();
        }

        public virtual void Configure()
        {
            provider.MappingFileName = @"..\..\..\NUnit\Mapping\Interfaces\mapping_TPH.xml";
        }

        [TestMethod]
        public void ShouldSerializeInheritedEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";

            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();

        }

        [TestMethod]
        public void ShouldLoadInheritedEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.Load<INameable>(typeof(INameable)).Count);
            Assert.AreEqual(1, oc.Load<Human>(typeof(Human)).Count);
            Assert.AreEqual(1, oc.Load<Dog>(typeof(Dog)).Count);

            INameable nameable = oc.LoadSingle<INameable>(typeof(INameable), "[Name = 'Rox']");
            Assert.AreEqual("Rox", nameable.Name);
        }

        [TestMethod]
        public void ShouldLoadReferencedInterfaces()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";

            Zoo zoo = new Zoo();
            zoo.Items.Add(d);
            zoo.Items.Add(h);

            oc.BeginTransaction();
            oc.Serialize(zoo);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.Load<INameable>(typeof(INameable)).Count);
            Assert.AreEqual(1, oc.Load<Human>(typeof(Human)).Count);
            Assert.AreEqual(1, oc.Load<Dog>(typeof(Dog)).Count);

            INameable nameable = oc.LoadSingle<INameable>(typeof(INameable), "[Name = 'Rox']");
            Assert.AreEqual("Rox", nameable.Name);

            Assert.AreEqual(2, oc.Load<INameable>(typeof(Zoo), ".Items").Count);

        }

        [TestMethod]
        public void ShouldLoadReferencesFromInterfaces()
        {
            oc.InitializeRepository();

            Dog dog = new Dog();
            dog.Kind = "Fox";
            dog.Name = "Rox";

            Toy bone = new Toy();
            bone.Name = "Bone";

            dog.Toys.Add(bone);

            Human human = new Human();
            human.Name = "Erik";
            human.LastName = "Smith";

            Toy cards = new Toy();
            cards.Name = "Cards";

            human.Toys.Add(cards);

            Zoo zoo = new Zoo();
            zoo.Items.Add(dog);
            zoo.Items.Add(human);

            oc.BeginTransaction();
            oc.Serialize(zoo);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.Load<Toy>(typeof(Toy)).Count);
            Assert.AreEqual(2, oc.Load<Toy>(typeof(INameable), ".Toys").Count);

            Assert.AreEqual("Bone", oc.LoadSingle<Toy>(typeof(INameable), "[Name = 'Rox'].Toys").Name);

            Assert.AreEqual(2, oc.Load<INameable>(typeof(INameable)).Count);
            Assert.AreEqual(1, oc.Load<Human>(typeof(Human)).Count);
            Assert.AreEqual(1, oc.Load<Dog>(typeof(Dog)).Count);

            INameable nameable = oc.LoadSingle<INameable>(typeof(INameable), "[Name = 'Rox']");
            Assert.AreEqual("Rox", nameable.Name);

            Assert.AreEqual(2, oc.Load<INameable>(typeof(Zoo), ".Items").Count);

        }
    }
}
