using System;
using System.Collections;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss;

namespace TestUSS
{
	/// <summary>
	/// Description rsume de Performance.
	/// </summary>
	[TestClass]
	public class Performance
	{
		public Performance()
		{
			WriteHeaders();
		}

		[TestMethod]
		public void Entities()
		{
			// 156,25 ms
			// 3396 ko

			long memory1 = Environment.WorkingSet / 1024;
			Counter c = new Counter();

			c.Start();
			EntitySet entitySet = CreateEntities(10000);

			foreach(Entity e in entitySet)
			{
				string name = e.GetString("Name");
				int age = e.GetInt32("Age");
			}

			long memory2 = Environment.WorkingSet / 1024;

			Console.WriteLine(c.Elapsed().TotalMilliseconds.ToString() + " ms");
			Console.WriteLine((memory2 - memory1).ToString() + " ko");
		}

		[TestMethod]
		public void GenericXML()
		{
			TestEngine("Xml");
		}

		[TestMethod]
		public void GenericSQL()
		{
			TestEngine("Sql");
		}

		[TestMethod]
		public void MemoryXml()
		{
			TestEngine("MemoryXml");
		}

		[TestMethod]
		public void CachedXml()
		{
			TestEngine("CachedXml");
		}

		[TestMethod]
		public void MemorySql()
		{
			TestEngine("MemorySql");
		}

		private EntitySet CreateEntities(int count)
		{
			EntitySet es = new EntitySet();

			Entity last = null;

			for(int i=0; i< count; i++)
			{
				Entity e = new Entity("Person");
				e.AddValue("Name", "Person N " + i);
				e.AddValue("Age", i);
				e.AddValue("Active", true);
				e.AddValue("Birth", DateTime.Now);
				e.AddValue("Colors", new string[] { "Blue", "White", "Red" });

				if(last != null)
					last.AddValue("Related", e);
				else
					es.Add(e);

				last = e;
			}

			return es;
		}

		private void WriteHeaders()
		{
			Console.Write("Engine" + "\t");
			Console.Write("Load" + "\t");
			Console.Write("Initialize" + "\t");
			Console.Write("Cr. 100" + "\t");
			Console.Write("R. 100" + "\t");
			Console.Write("Cr. 1000" + "\t");
			Console.Write("R. 1100" + "\t");
			Console.WriteLine();
		}

		private void TestEngine(string name)
		{
			Console.Write(name + "\t");
			Counter counter = new Counter();

			// Load engine
			counter.Start();
			IPersistenceEngine pe = XmlConfigLoader.LoadXmlConfig("engines.config", name).CreatePersistenceEngine();
			Console.Write(counter.Elapsed().TotalSeconds + "s\t");

			// Initialize repository
			counter.Start();
			pe.InitializeRepository();
			Console.Write(counter.Elapsed().TotalSeconds + "s\t");

			// Create 100 entities
			EntitySet es;
			counter.Start();
			es = CreateEntities(100);
			Transaction t = new Transaction();
			t.Serialize(es);
			t.Commit(pe);
			Console.Write(counter.Elapsed().TotalMilliseconds + "ms\t");

			// Read 100 entities
			counter.Start();
			pe.Load("Person");
			Console.Write(counter.Elapsed().TotalMilliseconds + "ms\t");

			// Create 1000 entities
			counter.Start();
			es = CreateEntities(1000);
			t = new Transaction();
			t.Serialize(es);
			t.Commit(pe);
			Console.Write(counter.Elapsed().TotalMilliseconds + "ms\t");

			// Read 1100 entities
			counter.Start();
			pe.Load("Person");
			Console.Write(counter.Elapsed().TotalMilliseconds + "ms\t");

			Console.WriteLine();

		}

		public void Memory()
		{
			long memory1 = 0, memory2 = 0;

			Counter counter = new Counter();
			counter.Start();
			memory1 = Environment.WorkingSet / 1024;
			EntitySet es = CreateEntities(100000);
			memory2 = Environment.WorkingSet / 1024;
			Console.WriteLine("{0} Ko reserved for {1} entities (with 3 attributes), created in {2} seconds", memory2 - memory1, 100000, counter.Elapsed().TotalSeconds);

		}
	}
}
