using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using Evaluant.Uss;
using Evaluant.Uss.Collections;
using Evaluant.Uss.Commands;

namespace TestUSS.Collections
{
	/// <summary>
	/// Description rsume de TestHashedArray.
	/// </summary>
	[TestClass]
	public class TestHashedList
	{
		HashedList list = new HashedList();

		public TestHashedList()
		{
		}

		[TestInitialize]
		public void Init()
		{
			list = new HashedList();
		}

		[TestMethod]
		public void Add()
		{
			try
			{
				list.Add(null);
				Assert.Fail();
			}
			catch{}
		}

		[TestMethod]
		public void Clear()
		{
			// Clear
			list.Clear();
			Assert.IsFalse(list.Contains("string"));
			Assert.IsTrue(list.Count == 0);
		}

		[TestMethod]
		public void Contains()
		{
            list.Clear();
            Assert.IsFalse(list.Contains("item"));

			//Contains
			try
			{
				// We can't add null reference
				list.Contains(null);
				Assert.Fail();
			}
			catch{}

			list.Add("string");
            
			Assert.IsTrue(list.Contains("string"));
			Assert.IsTrue(list.Count == 1);
			Assert.IsFalse(list.Contains("test"));
		}

		[TestMethod]
		public void Remove()
		{
			// Remove
            list.Clear();
			list.Add("string");
			Assert.IsTrue(list.Contains("string"));
			list.Remove("string");

			Assert.IsFalse(list.Contains("string"));
			Assert.IsTrue(list.Count == 0);
			Assert.IsFalse(list.Contains("test"));

            list.Clear();
            
            list.Add(0);
            list.Add(1);

            list.Remove(0);
            list.Remove(1);

            Assert.AreEqual(0, list.Count);
            Assert.IsFalse(list.Contains(0));
		}

		[TestMethod]
		public void IEnumerable()
		{

			// IEnumerable
			for(int i=0; i<1000; i++)
			{
				list.Add(i);
			}

			Assert.IsTrue(list.Contains(0));
			Assert.IsTrue(list.Contains(999));
			Assert.IsTrue(list.Count == 1000);
			Assert.IsFalse(list.Contains(1000));
		}    
		   
		[TestMethod]
		public void Index()
		{
			try
			{
				object o = list[0];
				Assert.Fail();
			}
			catch{}

			list.Add("string");
			list.Add("test");

			Assert.AreEqual("string", list[0]);
			Assert.AreEqual("test", list[1]);
		}

        [TestMethod] 
        public void IndexOf()
        {
            list.Clear();
            for(int i=0; i<10; i++)
                list.Add(i);

            int cpt = 0;
            foreach(int i in list)
            {
                Assert.AreEqual(cpt, list[cpt]);
                Assert.AreEqual(cpt, i);
                Assert.AreEqual(cpt, list.IndexOf(i));
                cpt++;
            }
            
        }

		[TestMethod]
		public void Serializable()
		{
			string filename = "stream.bin";

			for(int i=0; i<100; i++)
				list.Add(i);

			FileStream fs = new FileStream(filename, FileMode.Create);
			BinaryFormatter bf = new BinaryFormatter();
			bf.Serialize(fs, list);
			fs.Close();

			fs = new FileStream(filename, FileMode.Open);
			list = (HashedList) bf.Deserialize(fs);
			fs.Close();

			File.Delete(filename);

			Assert.AreEqual(100, list.Count);
			Assert.AreEqual(0, list[0]);
		}

		[TestMethod]
		public void enumerator()
		{
			HashedList list = new HashedList();
			list.Add(new Person("a", "a", 1));
			list.Add(new Person("b", "b", 2));
			list.Add(new Person("c", "c", 3));
			list.Add(new Person("d", "d", 4));
			list.Add(new Person("e", "e", 5));
            
			Assert.AreEqual(1, ((Person)list[0]).Age);
			Assert.AreEqual(5, ((Person)list[4]).Age);

			int i=1;
			foreach(Person p in list)
			{
				Assert.AreEqual(i++, p.Age);
			}

			Assert.AreEqual(6, i);
		}

		[TestMethod]
		public void Insert()
		{
			HashedList list = new HashedList();
			list.Add(new Person("b", "b", 2));
			list.Add(new Person("d", "d", 4));
			list.Add(new Person("e", "e", 5));
			
			// insert in first position
			list.Insert(0, new Person("a", "a", 1));

			Assert.AreEqual(4, list.Count);
			Assert.AreEqual(1, ((Person)list[0]).Age);
			Assert.AreEqual(2, ((Person)list[1]).Age);
			Assert.AreEqual(4, ((Person)list[2]).Age);
			Assert.AreEqual(5, ((Person)list[3]).Age);

			// insert in middle
			list.Insert(2, new Person("c", "c", 3));

			Assert.AreEqual(5, list.Count);
			Assert.AreEqual(1, ((Person)list[0]).Age);
			Assert.AreEqual(2, ((Person)list[1]).Age);
			Assert.AreEqual(3, ((Person)list[2]).Age);
			Assert.AreEqual(4, ((Person)list[3]).Age);
			Assert.AreEqual(5, ((Person)list[4]).Age);

			// insert in last position
			list.Insert(5, new Person("f", "f", 6));

			Assert.AreEqual(6, list.Count);
			Assert.AreEqual(1, ((Person)list[0]).Age);
			Assert.AreEqual(2, ((Person)list[1]).Age);
			Assert.AreEqual(3, ((Person)list[2]).Age);
			Assert.AreEqual(4, ((Person)list[3]).Age);
			Assert.AreEqual(5, ((Person)list[4]).Age);
			Assert.AreEqual(6, ((Person)list[5]).Age);
		}

		[TestMethod]
		public void CopyTo()
		{
			HashedList hl = new HashedList();
            
			hl.Add("a");
			hl.Add("b");
			hl.Add("c");
			hl.Add("d");

			string[] array = new string[4];
			hl.CopyTo(array, 0);

			Assert.AreEqual("a", array[0]);
			Assert.AreEqual("b", array[1]);
			Assert.AreEqual("c", array[2]);
			Assert.AreEqual("d", array[3]);

			hl.Clear();

			hl.Add("a");
			hl.Add("b");
			hl.Add("c");

			array = new string[4];
			hl.CopyTo(array, 0);

			Assert.AreEqual("a", array[0]);
			Assert.AreEqual("b", array[1]);
			Assert.AreEqual("c", array[2]);
		}
	}
}
