using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss;
using System.ComponentModel;

namespace TestUSS
{
	/// <summary>
	/// Tests for common services
	/// </summary>
	[TestClass]
	public class TestStorageServices
	{

		[TestMethod]
		public void SimpleEntity()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "Thomas");
			p1.AddValue("Lastname", "Gil");
			p1.AddValue("Age", 27);

			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Sbastien");
			p2.AddValue("Lastname", "Ros");
			p2.AddValue("Age", 27);
		}

		[TestMethod]
		public void EntityWithReference()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "Thomas");
			p1.AddValue("Lastname", "Gil");
			p1.AddValue("Age", 27);

			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Sbastien");
			p2.AddValue("Lastname", "Ros");
			p2.AddValue("Age", 27);

			p1.AddValue("Partners", p2);

			Assert.AreEqual(p2, p1.GetEntitySet("Partners")[0]);
		}

		[TestMethod]
		public void EntryIndexer()
		{
			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Sbastien");
			p2.AddValue("Lastname", "Ros");
			p2.AddValue("Age", 27);

			Assert.AreEqual(p2["Firstname"], "Sbastien");
			Assert.AreEqual(p2["Lastname"], "Ros");
			Assert.AreEqual(p2["Age"], 27);
			Assert.IsNull(p2["random"]);
		}

		[TestMethod]
		public void Sort()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Name", "p1");
			p1.AddValue("Age", 3);

			Entity p2 = new Entity("Person");
			p2.AddValue("Name", "p2");
			p2.AddValue("Age", 1);

			Entity p3 = new Entity("Person");
			p3.AddValue("Name", "p3");
			p3.AddValue("Age", 2);

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			EntitySet ee = new EntitySet();
			ee.Add(p1);
			ee.Add(p2);
			ee.Add(p3);
			
			ee.Sort("Name");
			Assert.AreEqual(p1, ee[0]);
			Assert.AreEqual(p2, ee[1]);
			Assert.AreEqual(p3, ee[2]);

			ee.Sort("Age");
			Assert.AreEqual(p2, ee[0]);
			Assert.AreEqual(p3, ee[1]);
			Assert.AreEqual(p1, ee[2]);

		}

		[TestMethod]
		public void EditableObject()
		{
			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Sbastien");
			p2.AddValue("Lastname", "Ros");
			p2.AddValue("Age", 27);

			((IEditableObject)p2).BeginEdit();
            p2["Firstname"] = "toto";
			p2["random"] = "titi";

			Assert.AreEqual(p2["Firstname"], "toto");
			Assert.AreEqual(p2["random"], "titi");

			((IEditableObject)p2).CancelEdit();

			Assert.AreEqual(p2["Firstname"], "Sbastien");
			Assert.IsNull(p2["random"]);
		
		}

		[TestMethod]
		public void State()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Name", "p1");

			p1["Name"] = "p4";
			p1["Name"] = "p5";

			Assert.AreEqual(Evaluant.Uss.State.New, p1.State);
			foreach(EntityEntry ee in p1)
				Assert.AreEqual(Evaluant.Uss.State.New, ee.State);
		}
	}
}
