using System;
using NUnit.Framework;
using System.Collections;
using Descriptors = Evaluant.Uss.ObjectContext.Descriptors;

namespace NUnit.DynamicProxy
{
	/// <summary>
	/// Description rsume de ReflectionDescriptor.
	/// </summary>
	[TestFixture]
	public class PersistentDescriptor
	{
		public PersistentDescriptor()
		{
		}

		[Test]
		public void ReflectionDescriptor()
		{
			Descriptors.ReflectionDescriptor descriptor = new Evaluant.Uss.ObjectContext.Descriptors.ReflectionDescriptor();
			
			Descriptors.PropertyDescriptor[] descriptors = descriptor.GetPersistentProperties(typeof(Person));
			Hashtable entries = ConvertToDictionnary(descriptors);

			Assert.IsTrue(entries.Contains("Name"));
			Assert.IsTrue(entries.Contains("Age"));
			Assert.IsTrue(entries.Contains("Birth"));
			Assert.IsTrue(entries.Contains("Partners"));
			Assert.IsTrue(entries.Contains("Address"));
			descriptor.GetIdDescriptor(typeof(Person)).PropertyName = "PersonId";

			// Name
			Assert.IsTrue(entries.Contains("Name"));
			Assert.AreEqual("_Name", ((Descriptors.PropertyDescriptor)entries["Name"]).FieldName);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Name"]).IsEntity);
			Assert.AreEqual(typeof(string), ((Descriptors.PropertyDescriptor)entries["Name"]).Type);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Name"]).IsList);
			Assert.AreEqual("Name", ((Descriptors.PropertyDescriptor)entries["Name"]).PropertyName);

			// Age
			Assert.IsTrue(entries.Contains("Age"));
			Assert.AreEqual("m_Age", ((Descriptors.PropertyDescriptor)entries["Age"]).FieldName);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Age"]).IsEntity);
			Assert.AreEqual(typeof(int), ((Descriptors.PropertyDescriptor)entries["Age"]).Type);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Age"]).IsList);
			Assert.AreEqual("Age", ((Descriptors.PropertyDescriptor)entries["Age"]).PropertyName);

			// DateTime
			Assert.IsTrue(entries.Contains("Birth"));
			Assert.AreEqual("birth", ((Descriptors.PropertyDescriptor)entries["Birth"]).FieldName);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Birth"]).IsEntity);
			Assert.AreEqual(typeof(DateTime), ((Descriptors.PropertyDescriptor)entries["Birth"]).Type);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Birth"]).IsList);
			Assert.AreEqual("Birth", ((Descriptors.PropertyDescriptor)entries["Birth"]).PropertyName);

			// Partners
			Assert.IsTrue(entries.Contains("Partners"));
			Assert.AreEqual("partners", ((Descriptors.PropertyDescriptor)entries["Partners"]).FieldName);
			Assert.AreEqual(true, ((Descriptors.PropertyDescriptor)entries["Partners"]).IsEntity);
			Assert.AreEqual(typeof(IList), ((Descriptors.PropertyDescriptor)entries["Partners"]).Type);
			Assert.AreEqual(true, ((Descriptors.PropertyDescriptor)entries["Partners"]).IsList);
			Assert.AreEqual("Partners", ((Descriptors.PropertyDescriptor)entries["Partners"]).PropertyName);

			// Address
			Assert.IsTrue(entries.Contains("Address"));
			Assert.AreEqual("m_address", ((Descriptors.PropertyDescriptor)entries["Address"]).FieldName);
			Assert.AreEqual(true, ((Descriptors.PropertyDescriptor)entries["Address"]).IsEntity);
			Assert.AreEqual(typeof(Address), ((Descriptors.PropertyDescriptor)entries["Address"]).Type);
			Assert.AreEqual(false, ((Descriptors.PropertyDescriptor)entries["Address"]).IsList);
			Assert.AreEqual("Address", ((Descriptors.PropertyDescriptor)entries["Address"]).PropertyName);
			descriptor.GetIdDescriptor(typeof(Address)).PropertyName = "Id";

			// Employee's Properties

			entries = ConvertToDictionnary(descriptor.GetPersistentProperties(typeof(Employee)));

			Assert.IsTrue(entries.Contains("Name"));
			Assert.IsTrue(entries.Contains("Age"));
			Assert.IsTrue(entries.Contains("Birth"));
			Assert.IsTrue(entries.Contains("Partners"));
			Assert.IsTrue(entries.Contains("Address"));
			descriptor.GetIdDescriptor(typeof(Employee)).PropertyName = "PersonId";

			Assert.IsTrue(entries.Contains("Salary"));


		}

		private Hashtable ConvertToDictionnary(Descriptors.PropertyDescriptor[] descriptors)
		{
			Hashtable index = new Hashtable();
			foreach(Descriptors.PropertyDescriptor propd in descriptors)
			{
				index.Add(propd.PropertyName, propd);
			}

			return index;
		}
	}
}
