using System;
using System.Collections;

using NUnit.Framework;
using MyNamespace;

using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;

namespace NUnit.PersistenceManager.Performance
{
	/// <summary>
	/// Description rsume de Performance.
	/// </summary>
	public class Performance
	{
		ObjectService _Factory;
		Evaluant.Uss.ObjectContext.ObjectContext _Manager;

		public Performance()
		{
		}

		private void LoadEngine(string engine)
		{
			Counter c = new Counter();
			c.Start();

			_Factory = new ObjectService("engines.config", engine);
			_Factory.AddAssembly("NUnit");

			_Manager = _Factory.CreateObjectContext();
			Console.WriteLine("{0} engine loaded in {1} seconds", engine, c.Elapsed().TotalSeconds);
		}

		private void Create(int count)
		{
			_Manager.InitializeRepository();

			PersonCollection people = new PersonCollection();

			for(int i=1; i <= count; i++)
			{
				Person p = new Person();
				p.FirstName = "Person " + i.ToString();
				p.Age = i;
				people.Add(p);
			}

			Counter c = new Counter();
			c.Start();

			_Manager.BeginTransaction();
			_Manager.Serialize(people);
			_Manager.CommitTransaction();

			Console.WriteLine("Serialized {0} objects in {1} seconds", count, c.Elapsed().TotalSeconds);
		}

		private void Read()
		{
			_Manager.Clear();

			Counter c = new Counter();
			c.Start();

			IList people = _Manager.Load(typeof(Person));

			Console.WriteLine("{0} objects read in {1} seconds", people.Count, c.Elapsed().TotalSeconds);
		}

		private void Run(string engine)
		{
			int count = 1000;
			LoadEngine(engine);
			Create(count);
			Read();
		}
		
		[Test]
		public void Statistics()
		{
//			Run("Xml");
//			Run("Memory");
//			Run("Prevalence");
			Run("AccessGeneric");
		}
	}
}
