using System;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss;
using NUnit.OptimizedInferredRelationships.Domain;

namespace NUnit.OptimizedInferredRelationships
{
	/// <summary>
	/// Description rsume de Proxy.
	/// </summary>
	[TestClass]
	public class Fixture
	{
        protected ObjectContext oc;
        Evaluant.Uss.SqlMapper.SqlMapperProvider provider;

        public Fixture()
        {
            provider = new Evaluant.Uss.SqlMapper.SqlMapperProvider();
            
            provider.ConnectionString = @"Data Source=.;Initial Catalog=uss;Integrated Security=True";
            provider.Dialect = "Evaluant.Uss.SqlMapper.MsSqlDialect";
            provider.Driver = "Evaluant.Uss.SqlMapper.MsSqlDriver";
            provider.MappingFileName = @"..\..\..\NUnit\PersistenceManager\OptimizedInferredRelationships\mapping_1.xml";
            provider.TablePrefix = "OIR_";
            provider.RegisterMetaData(MetaDataFactory.FromAssembly(typeof(Person).Assembly));

            provider.InitializeConfiguration();
        }

        [TestInitialize]
        public void SetUp()
        {
            oc = new ObjectService(provider).CreateObjectContext();
            oc.InitializeRepository();
        }

        [TestMethod]
        public void ShouldSerializeEntities()
        {
            Person p1 = new Person("p1");
            Person p2 = new Person("p2");
            
            p1.Address = new Address("a1");
            p2.Address = new Address("a2");

            p1.Vehicule = new Vehicule();
            p2.Vehicule = new Vehicule();

            p1.Vehicule.Passengers.Add(new Passenger("pas11"));
            p1.Vehicule.Passengers.Add(new Passenger("pas12"));
            p2.Vehicule.Passengers.Add(new Passenger("pas21"));
            p2.Vehicule.Passengers.Add(new Passenger("pas21"));

            oc.BeginTransaction();
            oc.Serialize(p1);
            oc.Serialize(p2);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.Load<Person>().Count);
            Assert.AreEqual(2, oc.Load<Address>().Count);
            Assert.AreEqual(2, oc.Load<Vehicule>().Count);
            Assert.AreEqual(4, oc.Load<Passenger>().Count);
        }

        [TestMethod]
        public void ShouldLoadOneLevelWithLazyLoading()
        {
            ShouldSerializeEntities();

            Query q = new Query(typeof(Person));
            //q.InferredRelationships.Add("Address");

            List<Person> ps = oc.Load<Person>(q);

            // To ensure it is not called
            oc.PersistenceEngine = null;

            try
            {
                Assert.AreEqual(2, ps.Count);
                Assert.IsNotNull(ps[0].Address);
                Assert.IsNotNull(ps[1].Address);
            }
            catch(NullReferenceException)
            {
                return;
            }

            Assert.Fail();
        }

        [TestMethod]
        public void ShouldLoadOneLevelWithoutLazyLoading()
        {
            ShouldSerializeEntities();

            oc.Clear();

            Query q = new Query(typeof(Person));
            q.InferredRelationships.Add("Address");

            List<Person> ps = oc.Load<Person>(q);

            // To ensure it is not called
            oc.PersistenceEngine = null;

            Assert.AreEqual(2, ps.Count);
            Assert.IsNotNull(ps[0].Address);
            Assert.IsNotNull(ps[1].Address);
        }

        [TestMethod]
        public void ShouldLoadOneLevelWithCacheAndNoLazyLoading()
        {
            // A bug can occur when the related entities are already in the first level cache. 

            ShouldSerializeEntities();

            Query q = new Query(typeof(Person));
            q.InferredRelationships.Add("Address");

            List<Person> ps = oc.Load<Person>(q);

            // To ensure it is not called
            oc.PersistenceEngine = null;

            Assert.AreEqual(2, ps.Count);
            Assert.IsNotNull(ps[0].Address);
            Assert.IsNotNull(ps[1].Address);
        }

        [TestMethod]
        public void ShouldLazyLoadMany()
        {
            ShouldSerializeEntities();

            Query q = new Query(typeof(Person));
            q.InferredRelationships.Add("Address");
            q.InferredRelationships.Add("Vehicule");

            List<Person> ps = oc.Load<Person>(q);

            // To ensure it is not called
            oc.PersistenceEngine = null;

            Assert.AreEqual(2, ps.Count);
            Assert.IsNotNull(ps[0].Address);
            Assert.IsNotNull(ps[1].Address);
            Assert.IsNotNull(ps[0].Vehicule);
            Assert.IsNotNull(ps[1].Vehicule);
        }

        [TestMethod]
        public void PotPourri()
        {
            object o = null;
            bool b = false;

            if (o != null && b)
            {
                Console.WriteLine();
            }
        }
    }
}
