using System;
using System.Collections;
using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;
using OneToMany;

using NUnit.Framework;

namespace UseCases.OneToManyModel
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		private ObjectService pmf;
		private ObjectContext pm;

		[SetUp]
		public void SetUp()
		{
			pmf = new ObjectService("UseCases.config", "OneToManyModel");
			pmf.AddAssembly(typeof(Customer).Assembly);
			pm = pmf.CreateObjectContext();

			Customer c = new Customer();
			c.Name = "Wang";
			c.Firstname = "Yang";
			c.Address = "Hauptstr. 29";
			c.City = "Berlin";

			Order o1 = new Order();
			o1.ProductName = "Coffe";
			o1.Amount = 11.5;

			Order o2 = new Order();
			o2.ProductName = "Chocolade";
			o2.Amount = 53.4;

			c.Orders.Add(o1);
			c.Orders.Add(o2);

			pm.InitializeRepository();

			pm.BeginTransaction();
			pm.Serialize(c);
			pm.CommitTransaction();
		}

		[Test]
		public void TestOrders()
		{
			pm.Clear();

			IList orders = pm.Load(typeof(Order));
			Assert.AreEqual(2, orders.Count);

			pm.Clear();

			IList order1List = pm.Load(typeof(Order), "[ProductName='Coffe']");
			Assert.AreEqual(1, order1List.Count);
			Order order1 = (Order)order1List[0];

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(Customer), order1.Parents.GetCustomer().GetType());
//			Assert.AreEqual("Wang", order1.Parents.GetCustomer().Name);

			pm.Clear();

			IList order2List = pm.Load(typeof(Order), "[ProductName='Chocolade']");
			Assert.AreEqual(1, order2List.Count);
			Order order2 = (Order)order2List[0];

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(Customer), order2.Parents.GetCustomer().GetType());
//			Assert.AreEqual("Wang", order2.Parents.GetCustomer().Name);
		}

		[Test]
		public void TestCustomers()
		{
			pm.Clear();

			IList customers = pm.Load(typeof(Customer));
			Assert.AreEqual(1, customers.Count);
			Customer customer = (Customer)customers[0];
			Assert.AreEqual("Wang", customer.Name);
			Assert.AreEqual(2, customer.Orders.Count);
		}
	}
}
