using System;
using System.IO;
using System.Diagnostics;
using Microsoft.Win32;

using Mono.Cecil;
using Mono.Cecil.Binary;
using Mono.Cecil.Metadata;

namespace Pdb.Tools
{
	/// <summary>
	/// Description rsume de PdbCheck.
	/// </summary>
	public class PdbCheck
	{
        //private static string EUSS_INSTALLATION_DIR_KEY = "SOFTWARE\\Evaluant\\EUSS";

		public static void Check(string assemblyName, string pdbName)
		{
			AssemblyDefinition ass = AssemblyFactory.GetAssembly(assemblyName);

			//	Get the age of the assembly
			uint age = ass.MainModule.Image.DebugHeader.Age;

			//	Get the signature of the assembly
			Guid signature = ass.MainModule.Image.DebugHeader.Signature;

			BinaryReader reader = new BinaryReader(new FileStream(pdbName, FileMode.Open));
			
			//	Go to the page size position
			reader.BaseStream.Position += 32;

			//	Get the page size of the pdb file
			byte[] pageSize = reader.ReadBytes(4);
			Array.Reverse(pageSize);

			int psize = 0;
			for(int i = 0; i < 4; i++)
				psize += pageSize[i] * (int) Math.Pow(2, i * 4);

			//int psize = reader.ReadInt32();

			//	Go to the page count definition
			reader.BaseStream.Position += 4;

			//	Get the number of pages
			byte[] pageCount = reader.ReadBytes(4);
			Array.Reverse(pageSize);

			int pCount = 0;
			for(int i = 0; i < 4; i++)
				pCount += pageCount[i] * (int) Math.Pow(2, i * 4);

			//int pCount = reader.ReadInt32();

			//	Go to the page 1
			reader.BaseStream.Position += psize - 44;

			//	Define the page index of the page which contains the pdb
			//	Signature and age definitions
			byte[] nbToFind = new byte[]{0x94, 0x2E, 0x31, 0x01};

			//	Get the page which contains the age and signature definitions
			int tmpPage = 0;
			for(int page = 1; page < pCount - 1; page++)
			{
				byte[] nb = reader.ReadBytes(4);
				if(nb[0] == nbToFind[0] && nb[1] == nbToFind[1] && nb[2] == nbToFind[2] && nb[3] == nbToFind[3] )
				{
					tmpPage = page;
					break;
				}
				//	Go to the next page
				reader.BaseStream.Position += psize - 4;
			}

			reader.Close();

			//	Write the age and signature in the generated pdb
			BinaryWriter writer = new BinaryWriter(new FileStream(pdbName, FileMode.Open));
			writer.BaseStream.Position = tmpPage * psize + 8;
			writer.Write(age);
			writer.Write(signature.ToByteArray());
			writer.Close();
		}
	}
}
