using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;

using layerGen = Evaluant.Uss.PersistenceLayerGen;

namespace Evaluant.Uss.PersistenceLayerGenerator
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class Program
	{
		[STAThread]
		public static int Main(string[] args)
		{
            try
            {
                ArgParser parser = new ArgParser(args);

                string progLang = string.Empty;
                string outputPath = string.Empty;
                string modelFileName = string.Empty;

                if (parser.Exists("?"))
                {
                    Help();
                    return 0;
                }

                if (!parser.Exists("model"))
                {
                    Help();
                    return 1;
                }

                if (parser.Exists("lang"))
                    progLang = parser["lang"].Value;

                layerGen.PersistenceLayerGen.ProgrammingLangEnum programmingLang = GetProgLangFromString(progLang);

                if (parser.Exists("model"))
                {
                    modelFileName = parser["model"].Value;
                }
                if (!File.Exists(modelFileName))
                    throw new FileNotFoundException(String.Format("Cannot find the file \"{0}\"", modelFileName));

                string path = Path.GetDirectoryName(modelFileName);
                if (path == string.Empty)
                    path = Environment.CurrentDirectory;

                path += "\\";

                //if (parser.Exists("layerPath"))
                //{
                //    outputPath = parser["layerPath"].Value;
                //    if (Path.GetDirectoryName(outputPath) == string.Empty)
                //        outputPath = String.Concat(path, "\\", outputPath);
                //}
                //else
                //{
                //    outputPath = String.Concat(path, "model", (programmingLang == PersistenceLayerGenerator.ProgrammingLangEnum.CSharp) ? ".cs" : ".vb");
                //}
                outputPath = String.Concat(path, "model", (programmingLang == layerGen.PersistenceLayerGen.ProgrammingLangEnum.CSharp) ? ".cs" : ".vb");

                layerGen.PersistenceLayerGen modelGen = new layerGen.PersistenceLayerGen(modelFileName, outputPath, programmingLang);
                modelGen.GenerateFile();

                Console.WriteLine(String.Format("\"{0}\" successfully created", outputPath));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine("Generation aborted !");
                return -1;
            }
			return 0;
		}

        public static layerGen.PersistenceLayerGen.ProgrammingLangEnum GetProgLangFromString(string progLang)
        {
            try
            {
                layerGen.PersistenceLayerGen.ProgrammingLangEnum progL = 
                    (layerGen.PersistenceLayerGen.ProgrammingLangEnum)Enum.Parse(typeof(layerGen.PersistenceLayerGen.ProgrammingLangEnum), progLang, true);
                return progL;
            }
            catch (Exception ex)
            {
                return layerGen.PersistenceLayerGen.ProgrammingLangEnum.CSharp;
            }
        }

		static public void Help()
		{
			Console.WriteLine(@"

Help for domain.exe

Generates the persistence source code layer corresponding to the domain model.
The source code is generated in the same folder as the model.

domain /model:model_filename [/lang:programming_language] 

	model_filename
		Mandatory. The filename of the model. 
        It can be an Euss XML model file or an XMI

    programming_language
        (Optional)The programming language used to generate 
        the persistence layer (C# by default)
        Allowed values: 'csharp' or 'vbnet'
");

		}
	}
}
