using System;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;

using Evaluant.Uss.ObjectContext;

using System.IO;

using Evaluant.Uss;
using Evaluant.Uss.Weave.PersistenceModel;

using Mono.Cecil;
using Mono.Cecil.Cil;

using System.Diagnostics;

namespace PersistenceManagerWeaver
{
	/// <summary>
	/// Description rsume de EUSSWeaver.
	/// </summary>
	public class EUSSWeaver
	{     
		//Contains all FieldReferences which corresponds to persistent fields
		Hashtable _DeclaredFields;
		//Contains the Property name which corresponds to the FieldReference
		Hashtable _DeclaredProperties;
		Model persistentClasses;
		private Pdb.Model.PdbInfos _pdbModel;

		public Pdb.Model.PdbInfos PdbModel
		{
			get { return _pdbModel; }
			set { _pdbModel = value; }
		}

		#region Constructors

		public EUSSWeaver(Model model, string assemblyFileName, string newAssemblyFileName)
		{
			persistentClasses = model;

			_assemblyFileName = assemblyFileName;
			_newAssemblyFileName = newAssemblyFileName;

			if (newAssemblyFileName != String.Empty)
				_saveInNewAssembly = true;
			else
				_saveInNewAssembly = false;

			InitPrimitiveTypesOpCodes();
		}

		private Hashtable _PrimitiveTypesOpCodes;

		private void InitPrimitiveTypesOpCodes()
		{
			_PrimitiveTypesOpCodes = new Hashtable();
			_PrimitiveTypesOpCodes[typeof(sbyte).FullName] = OpCodes.Ldind_I1;
			_PrimitiveTypesOpCodes[typeof(byte).FullName] = OpCodes.Ldind_U1;
			_PrimitiveTypesOpCodes[typeof(char).FullName] = OpCodes.Ldind_U2;
			_PrimitiveTypesOpCodes[typeof(short).FullName] = OpCodes.Ldind_I2;
			_PrimitiveTypesOpCodes[typeof(ushort).FullName] = OpCodes.Ldind_U2;
			_PrimitiveTypesOpCodes[typeof(int).FullName] = OpCodes.Ldind_I4;
			_PrimitiveTypesOpCodes[typeof(uint).FullName] = OpCodes.Ldind_U4;
			_PrimitiveTypesOpCodes[typeof(long).FullName] = OpCodes.Ldind_I8;
			_PrimitiveTypesOpCodes[typeof(ulong).FullName] = OpCodes.Ldind_I8;
			_PrimitiveTypesOpCodes[typeof(bool).FullName] = OpCodes.Ldind_I1;
			_PrimitiveTypesOpCodes[typeof(double).FullName] = OpCodes.Ldind_R8;
			_PrimitiveTypesOpCodes[typeof(float).FullName] = OpCodes.Ldind_R4;
		}

		public EUSSWeaver(Model persistentClasses, string assemblyFileName)
			: this(persistentClasses, assemblyFileName, String.Empty)
		{

		}
		#endregion

		#region Documents management

		private string _assemblyFileName;
		private string _newAssemblyFileName;
		private bool _saveInNewAssembly;

		#endregion

		#region Common Field references

		FieldReference _entityFieldReference;
		FieldReference _persistenceManagerFieldReference;

		public void InitializeFieldReferences(AssemblyDefinition ass, TypeDefinition typeDefinition)
		{	
			Hashtable classHierarchy = GetTypeHierarchy(ass, typeDefinition);

			_entityFieldReference = null;
			_persistenceManagerFieldReference = null;
			for(int i = classHierarchy.Count - 1; i >=0; i--)
			{
				TypeDefinition currentType = (TypeDefinition)classHierarchy[i];
				foreach (FieldDefinition field in currentType.Fields)
				{
					if (field.Name == "_entity")
						_entityFieldReference = field;
					if (field.Name == "_persistenceManager")
						_persistenceManagerFieldReference = field;
				}
			}
		}

		#endregion

		#region Common MethodBase

#if EUSS11
        Type iPersistableCollectionType;
        Type iListType;
        Type iEnumerable;
#else
        Type iPersistableCollectionType;
        Type iPersistableCollectionGenericType;
        Type iListType;
        Type iListGenericType;
        Type iEnumerable;
        Type iEnumerableGeneric;
#endif

		System.Reflection.MethodBase addValueMethodBase;
		System.Reflection.MethodBase getValueMethodBase;
		System.Reflection.MethodBase get_IdMethodBase;
		System.Reflection.MethodBase set_IdMethodBase;
		System.Reflection.MethodBase deleteElementMethodBase;
		System.Reflection.MethodBase iListGetItemMethodBase;
		System.Reflection.MethodBase getEntityMethodBase;
		System.Reflection.MethodBase entitySetItemMethodBase;
		System.Reflection.MethodBase findEntryMethodBase;
		System.Reflection.MethodBase iPersistableCollectionDefaultMethodBase;
		System.Reflection.MethodBase iPersistableCollectionValuedMethodBase;
		System.Reflection.MethodBase loadReferenceMethodBase;
		System.Reflection.MethodBase trackObjectIPersistableCollectionMethodBase;
		System.Reflection.MethodBase trackObjectIPersistableMethodBase;
		System.Reflection.MethodBase getCountMethodBase;
        System.Reflection.MethodBase getTypeFromHandleMethodBase;

#if EUSS11
#else
        System.Reflection.MethodBase iPersistableCollectionGenericDefaultMethodBase;
        System.Reflection.MethodBase iPersistableCollectionGenericValuedMethodBase;
#endif

		public void InitializeMethodsBase()
		{
#if EUSS11
			iPersistableCollectionType = typeof(Entity).Assembly.GetType("Evaluant.Uss.ObjectContext.IPersistableCollection");
			iListType = typeof(int).Assembly.GetType("System.Collections.IList");
            iListType = typeof(int).Assembly.GetType("System.Collections.IList");
            iEnumerable = typeof(int).Assembly.GetType("System.Collections.IEnumerable");
#else
            iPersistableCollectionType = typeof(Entity).Assembly.GetType("Evaluant.Uss.ObjectContext.IPersistableCollection");
			iPersistableCollectionGenericType = typeof(Entity).Assembly.GetType("Evaluant.Uss.ObjectContext.IPersistableCollection`1");
			iListGenericType = typeof(int).Assembly.GetType("System.Collections.Generic.IList`1");
            iListType = typeof(int).Assembly.GetType("System.Collections.IList");
            iEnumerableGeneric = typeof(int).Assembly.GetType("System.Collections.Generic.IEnumerable`1");
            iEnumerable = typeof(int).Assembly.GetType("System.Collections.IEnumerable");
#endif

			addValueMethodBase = typeof(Entity).GetMethod("AddValue", new Type[]{typeof(string), typeof(object)});

			 if (addValueMethodBase == null)
				throw new Exception("Entity.AddValue(string, object) method is not loaded");

			get_IdMethodBase = typeof(Entity).GetProperty("Id").GetGetMethod();
			set_IdMethodBase = typeof(Entity).GetProperty("Id").GetSetMethod();

			if (get_IdMethodBase == null)
				throw new Exception("Entity.get_Id() method is not loaded");

			if (set_IdMethodBase == null)
				throw new Exception("Entity.set_Id(string) method is not loaded");

			deleteElementMethodBase = typeof(Entity).GetMethod("DeleteElement", new Type[]{typeof(string)});

			if (deleteElementMethodBase == null)
				throw new Exception("Entity.DeleteElement(string) method is not loaded");

			getValueMethodBase = typeof(Entity).GetMethod("GetValue", new Type[]{typeof(string)});

			if (getValueMethodBase == null)
				throw new Exception("Entity.GetValue(string) method is not loaded");

			entitySetItemMethodBase = typeof(Entity).GetMethod("set_Item", new Type[]{typeof(string), typeof(int)});

			if (entitySetItemMethodBase == null)
				throw new Exception("Entity.set_Item(string) method is not loaded");

			findEntryMethodBase = typeof(Entity).GetMethod("FindEntry", new Type[]{typeof(string)});

			if (findEntryMethodBase == null)
				throw new Exception("Entity.FindEntry(string) method is not loaded");

			loadReferenceMethodBase = typeof(ObjectContext).GetMethod("LoadReference", new Type[]{typeof(Entity), typeof(string)});

			if (loadReferenceMethodBase == null)
				throw new Exception("ObjectContext.LoadReference(Entity, string) method is not loaded");

			//trackObjectIPersistableCollectionMethodBase = typeof(ObjectContext).GetMethod("TrackObject", new Type[]{iPersistableCollectionType});

            foreach (System.Reflection.MethodBase m in typeof(ObjectContext).GetMethods())
            {
                if (m.Name == "TrackObject")
                {
                    System.Reflection.ParameterInfo p = m.GetParameters()[0];
#if EUSS11
                    if (p.ParameterType.Name == iPersistableCollectionType.Name)
                    {
                        trackObjectIPersistableCollectionMethodBase = m;
                    }
#else
                    if (p.ParameterType.Name == iPersistableCollectionGenericType.Name)
                    {
                        trackObjectIPersistableCollectionMethodBase = m;
                    }
                    if (p.ParameterType.Name == iPersistableCollectionType.Name)
                    {
                        trackObjectIPersistableCollectionMethodBase = m;
                    }
#endif
                }
            }

			if (trackObjectIPersistableCollectionMethodBase == null)
				throw new Exception("ObjectContext.TrackObject(IPersistableCollection) method is not loaded");

			trackObjectIPersistableMethodBase = typeof(ObjectContext).GetMethod("TrackObject", new Type[]{typeof(IPersistable)});

			if (trackObjectIPersistableMethodBase == null)
				throw new Exception("ObjectContext.TrackObject(IPersistable) method is not loaded");

			getEntityMethodBase = typeof(IPersistable).GetProperty("Entity").GetGetMethod();

			if (getEntityMethodBase == null)
				throw new Exception("IPersistable.get_Entity() method is not loaded");


            iPersistableCollectionDefaultMethodBase = iPersistableCollectionType.GetConstructor(new Type[] { });

            if (iPersistableCollectionDefaultMethodBase == null)
                throw new Exception("IPersistableCollection default constructor is not loaded");
#if EUSS11
#else
            iPersistableCollectionGenericDefaultMethodBase = iPersistableCollectionGenericType.GetConstructor(new Type[] { });

            if (iPersistableCollectionGenericDefaultMethodBase == null)
                throw new Exception("IPersistableCollection default constructor is not loaded");
#endif

            iPersistableCollectionValuedMethodBase = iPersistableCollectionType.GetConstructor(new Type[] { typeof(ObjectContext), typeof(IPersistable), typeof(string), typeof(IEnumerable) });

            if (iPersistableCollectionValuedMethodBase == null)
                throw new Exception("IPersistableCollection valued constructor is not loaded");
#if EUSS11
#else
            foreach (System.Reflection.MethodBase m in iPersistableCollectionGenericType.GetConstructors())
            {
                if (m.GetParameters().Length > 1)
                {
                    System.Reflection.ParameterInfo p = m.GetParameters()[3];
                    if (p.ParameterType.Name == iEnumerableGeneric.Name)
                    {
                        iPersistableCollectionGenericValuedMethodBase = m;
                    }
                }
            }

            if (iPersistableCollectionGenericValuedMethodBase == null)
                throw new Exception("IPersistableCollectionGeneric valued constructor is not loaded");
#endif

			getCountMethodBase = typeof(ICollection).GetProperty("Count").GetGetMethod();

			if (getCountMethodBase == null)
				throw new Exception("IList.get_Count() method is not loaded");

			iListGetItemMethodBase = typeof(IList).GetMethod("get_Item", new Type[]{typeof(int)});

			if (iListGetItemMethodBase == null)
				throw new Exception("IList.get_Item() method is not loaded");

            getTypeFromHandleMethodBase = typeof(Type).GetMethod("GetTypeFromHandle");
            if (getTypeFromHandleMethodBase == null)
                throw new Exception("Type.GetTypeFromHandle() method is not loaded");
		}

		#endregion

		#region IPersistable

		public void DuplicateIPersistable(TypeDefinition typeDefinition, AssemblyDefinition assembly)
		{
			AssemblyDefinition mixins = AssemblyFactory.GetAssembly(this.GetType().Assembly.Location);
			TypeDefinition mixin = mixins.MainModule.Types["PersistenceManagerWeaver.Mixin"];

            foreach (MethodDefinition constructor in mixin.Constructors)
            {
                if (constructor.Parameters.Count != 0)
                {
                    MethodDefinition m = assembly.MainModule.Inject(constructor);
                    typeDefinition.Constructors.Add(m);
                }
            }

            typeDefinition.Methods.Add(assembly.MainModule.Inject(mixin.Methods.GetMethod("Evaluant.Uss.ObjectContext.IPersistable.TrackChildren", new TypeReference[]{})));

            typeDefinition.Methods.Add(assembly.MainModule.Inject(mixin.Methods.GetMethod("Initialize", new TypeReference[] { })));
		}

		public void DuplicateIPersistableBaseMembers(TypeDefinition typeDefinition, AssemblyDefinition assembly)
		{
			AssemblyDefinition mixins = AssemblyFactory.GetAssembly(this.GetType().Assembly.Location);
			TypeDefinition mixin = mixins.MainModule.Types["PersistenceManagerWeaver.Mixin"];

			foreach (TypeReference interf in mixin.Interfaces)
			{
				TypeReference itf = assembly.MainModule.Import(interf);
				typeDefinition.Interfaces.Add(itf);
			}

			foreach (FieldDefinition field in mixin.Fields)
			{
				FieldDefinition f = new FieldDefinition(field.Name, assembly.MainModule.Import(field.FieldType), field.Attributes);
				typeDefinition.Fields.Add(f);
			}

			foreach (MethodDefinition method in mixin.Methods)
			{			
				if(method.Name != "Evaluant.Uss.ObjectContext.IPersistable.TrackChildren" && method.Name != "Evaluant.Uss.ObjectContext.IPersistable.DeleteChildren" && method.Name != "Initialize")
				{
                    MethodDefinition m = assembly.MainModule.Inject(method);
                    typeDefinition.Methods.Add(m);
				}
			}

			foreach (PropertyDefinition property in mixin.Properties)
			{
				PropertyDefinition pd = new PropertyDefinition(property.Name, assembly.MainModule.Import(property.PropertyType), property.Attributes);
				pd.GetMethod = typeDefinition.Methods.GetMethod(property.GetMethod.Name)[0];
				pd.SetMethod = typeDefinition.Methods.GetMethod(property.SetMethod.Name)[0];
				typeDefinition.Properties.Add(pd);
			}
		}

		#endregion

		#region Application

		private Hashtable GetTypeHierarchy(AssemblyDefinition ass, TypeDefinition typeDefinition)
		{
			int index = 0;
			Hashtable hierarchy = new Hashtable();
			TypeDefinition tBase = ass.MainModule.Types[typeDefinition.BaseType.FullName];
			hierarchy.Add(index, typeDefinition);

			index = 1;

			while(tBase != null)
			{
				hierarchy.Add(index, tBase);
				tBase = ass.MainModule.Types[tBase.BaseType.FullName];
				index++;
			}
			return hierarchy;
		}

		public void ReplaceMixinByTypeDefinition(ICollection methods, Hashtable fields)
		{
			foreach (MethodDefinition method in methods)
			{
				InstructionCollection instructions = method.Body.Instructions;
				for (int i = 0; i < instructions.Count; i++)
				{
					Instruction ins = instructions[i];

					if (ins.OpCode.Equals(OpCodes.Stfld))
					{
						if (fields[((FieldReference)ins.Operand).Name] != null)
						{
							ins = method.Body.CilWorker.Create(OpCodes.Stfld, (FieldReference)fields[((FieldReference)ins.Operand).Name]);

							method.Body.CilWorker.InsertBefore(instructions[i + 1], ins);
							method.Body.CilWorker.Delete(instructions[i]);

							i += 2;
						}
					}
					if (ins.OpCode.Equals(OpCodes.Ldfld))
					{
						if (fields[((FieldReference)ins.Operand).Name] != null)
						{
							ins = method.Body.CilWorker.Create(OpCodes.Ldfld, (FieldReference)fields[((FieldReference)ins.Operand).Name]);

							method.Body.CilWorker.InsertBefore(instructions[i + 1], ins);
							method.Body.CilWorker.Delete(instructions[i]);

							i += 2;
						}
					}
				}
			}
		}

		public void InsertInitializeMethodInCtor(AssemblyDefinition assembly, TypeDefinition typeDefinition)
		{
			Hashtable typeHierarchy = GetTypeHierarchy(assembly, typeDefinition);

			MethodReference initializeMethodReference = null;
			for(int i = typeHierarchy.Count - 1; i >=0; i--)
			{
				TypeDefinition currentType = (TypeDefinition)typeHierarchy[i];
				
				MethodDefinition initialize = currentType.Methods.GetMethod("Initialize", new TypeReference[]{});
				
				if(initialize == null)
					continue;

				InstructionCollection instructions = initialize.Body.Instructions;
				for (int index = 0; index < instructions.Count; index++)
				{
					Instruction ins = instructions[index];

					if (ins.OpCode.Equals(OpCodes.Stfld))
					{
						if (((FieldReference)ins.Operand).Name.Equals(_entityFieldReference.Name))
						{
							ins = initialize.Body.CilWorker.Create(OpCodes.Stfld, _entityFieldReference);

							initialize.Body.CilWorker.InsertBefore(instructions[index + 1], ins);
							initialize.Body.CilWorker.Delete(instructions[index]);

							index += 2;
						}
					}
				}
				initializeMethodReference = initialize;
			}

			//Insert Initialize method in constructors
			foreach (MethodDefinition ctor in typeDefinition.Constructors)
			{
				InstructionCollection instructions = ctor.Body.Instructions;
				Instruction ins = instructions[0];

				Instruction insertThis = ctor.Body.CilWorker.Create(OpCodes.Ldarg_0);
				ctor.Body.CilWorker.InsertAfter(ins, insertThis);
				Instruction insertInitialize = ctor.Body.CilWorker.Create(OpCodes.Call, initializeMethodReference);
				ctor.Body.CilWorker.InsertAfter(insertThis, insertInitialize);
			}
		}

		public void ModifyFields(ICollection methods, AssemblyDefinition assembly/*, Pdb.Model.PdbInfos pdb*/)
		{
			foreach (MethodDefinition methodDef in methods)
			{
				InstructionCollection instructions = methodDef.Body.Instructions;
				for (int i = 0; i < instructions.Count; i++)
				{
					Instruction ins = instructions[i];

					//AfterSet
					if (ins.OpCode.Equals(OpCodes.Stfld))
					{
						if (_DeclaredFields[((FieldReference)ins.Operand).Name] != null)
						{
							//Pdb.Model.Method m = pdb.Methods.GetMethod(methodDef.Name);

							FieldReference field = (FieldReference)_DeclaredFields[((FieldReference)ins.Operand).Name];
							Property prop = (Property)_DeclaredProperties[field.Name];
							string storageName = String.Empty;
							if(prop.Name != String.Empty)
								storageName = prop.Name;

							this.AfterSetInstructions(ref i, ins, methodDef, instructions, assembly, field, storageName);

//							foreach(Pdb.Model.Entry e in m.SequencePoints.Entries)
//							{
//								e.Offset += nbIns;
//							}
						}
					}
					//BeforeGet
					if (ins.OpCode.Equals(OpCodes.Ldfld))
					{
						if (_DeclaredFields[((FieldReference)ins.Operand).Name] != null)
						{
							FieldReference field = (FieldReference)_DeclaredFields[((FieldReference)ins.Operand).Name];
							Property prop = (Property)_DeclaredProperties[field.Name];
							string storageName = String.Empty;
							if(prop.Name != String.Empty)
								storageName = prop.Name;

							this.BeforeGetInstructions(ref i, ins, methodDef, assembly, field, storageName);
						}
					}
				}
			}
		}

		public void ReplaceIListByIPersistableCollection(FieldReference field, TypeDefinition typeDefinition, AssemblyDefinition assembly)
		{
			foreach (FieldDefinition fieldDef in typeDefinition.Fields)
			{
				if (fieldDef.Name == field.Name)
				{
#if EUSS11
					fieldDef.FieldType = assembly.MainModule.Import(iPersistableCollectionType);
#else
                    GenericInstanceType gt = fieldDef.FieldType as GenericInstanceType;
                    if (gt != null && gt.GenericArguments.Count > 0)
                    {
                        TypeReference tr = assembly.MainModule.Import(iPersistableCollectionGenericType);
                        GenericInstanceType ip = new GenericInstanceType(tr);
                        ip.GenericArguments.Add(gt.GenericArguments[0]);
                        fieldDef.FieldType = ip;
                    }
                    else
                        fieldDef.FieldType = assembly.MainModule.Import(iPersistableCollectionType);

                    //if (fieldDef.FieldType.GenericParameters.Count > 0)
                    //{
                    //    //TypeReference genericArg = assembly.MainModule.Import(fieldDef.FieldType.GenericArguments[0]);
                    //    fieldDef.FieldType = assembly.MainModule.Import(iPersistableCollectionGenericType);
                    //    //fieldDef.FieldType.GenericArguments.Add(genericArg);
                    //}
                    //else
                    //    fieldDef.FieldType = assembly.MainModule.Import(iPersistableCollectionType);
#endif
				}
			}
		}

		public void InsertIsLoadedFieldInstructions(TypeDefinition typeDefinition, FieldReference child, AssemblyDefinition assembly, string propertyName)
		{
			TypeReference boolReference = assembly.MainModule.Import(Type.GetType("System.Boolean"));
			typeDefinition.Fields.Add(new FieldDefinition("_Is" + propertyName + "Loaded", boolReference, FieldAttributes.Private));
		}

		public void ModifyChildrenFields(ICollection methods, TypeDefinition typeDefinition, AssemblyDefinition assembly, FieldReference childFieldReference, string multiplicity, string fieldName, string propertyName)
		{
			foreach (MethodDefinition methodDef in methods)
			{
				InstructionCollection instructions = methodDef.Body.Instructions;
				for (int i = 0; i < instructions.Count; i++)
				{
					Instruction ins = instructions[i];


					FieldReference fieldReference = null;
					foreach (FieldDefinition field in typeDefinition.Fields)
					{
						if (field.Name == "_Is" + propertyName + "Loaded")
							fieldReference = field;
					}

					if (ins.OpCode.Equals(OpCodes.Stfld))
					{
						if (((FieldReference)ins.Operand).Name.Equals(childFieldReference.Name))
						{
							if (multiplicity == "1")
							{
								this.SetOneChildInstruction(ref i, instructions[i-1], methodDef, fieldReference, assembly, childFieldReference, propertyName);
							}
							if (multiplicity == "*")
							{
								this.SetManyChildrenInstruction(ref i, instructions[i], methodDef, assembly, instructions, childFieldReference, propertyName);
							}
						}
					}

					if (ins.OpCode.Equals(OpCodes.Ldfld))
					{
						if (((FieldReference)ins.Operand).Name.Equals(childFieldReference.Name))
						{
							if (multiplicity == "*")
							{
                                this.GetManyChildrenInstruction(ref i, instructions[i - 1], methodDef, assembly, childFieldReference, propertyName);
							}
							if (multiplicity == "1")
							{
								this.GetOneChildInstruction(ref i, ins, methodDef, fieldReference, assembly, childFieldReference, propertyName);
							}
						}
					}
				}
			}
		}

		private void ProcessBasePersistentClasses(AssemblyDefinition ass, TypeDefinition typeDef, Model persistentClasses)
		{
			Hashtable typeHierarchy = GetTypeHierarchy(ass, typeDef);

			if(typeHierarchy.Count == 1)
			{
				Class currentClass = persistentClasses.Classes.GetClassByName(typeDef.FullName);
				if(currentClass != null)
				{
					DuplicateIPersistableBaseMembers( typeDef, ass);
				}
			}
			else
			{
				for(int i = typeHierarchy.Count - 1; i >= 0; i--)
				{
					Class currentClass = persistentClasses.Classes.GetClassByName(((TypeDefinition)typeHierarchy[i]).FullName);
					if(currentClass != null)
					{
						if(typeDef.BaseType.FullName == typeof(IPersistable).FullName)
						{
							DuplicateIPersistableBaseMembers( (TypeDefinition)typeHierarchy[i], ass);
						}
					}
				}
			}
		}

		public Hashtable GetFieldsFromType(AssemblyDefinition ass, TypeDefinition typeDef)
		{
			Hashtable classHierarchy = GetTypeHierarchy(ass, typeDef);
			Hashtable fields = new Hashtable();
			for(int i = classHierarchy.Count - 1; i >=0; i--)
			{
				TypeDefinition currentType = (TypeDefinition)classHierarchy[i];
				foreach (FieldDefinition field in currentType.Fields)
				{
					fields.Add(field.Name, field);
				}
			}
			return fields;
		}

		public int Weave()
		{
			if(persistentClasses.Classes == null)
				return 2;

			this.InitializeMethodsBase();

			AssemblyDefinition assembly = AssemblyFactory.GetAssembly(_assemblyFileName);

			AssemblyDefinition mixins = AssemblyFactory.GetAssembly(this.GetType().Assembly.Location);
			MethodReference persistableAssembly = null;
			
			foreach (CustomAttribute ca in mixins.CustomAttributes)
			{
				if (ca.Constructor.DeclaringType.FullName == typeof(PersistableAssemblyAttribute).FullName)
				{
					persistableAssembly = ca.Constructor;
					break;
				}
			}

			if (persistableAssembly == null)
				throw new Exception("PersistableAssemblyAttribute constructor is not loaded");

			//If assembly is already weaved => skip
			foreach (CustomAttribute ca in assembly.CustomAttributes)
			{
				if (ca.Constructor.DeclaringType.FullName == typeof(PersistableAssemblyAttribute).FullName)
					return 1;
			}

			//else => add Persistable Attribute
			assembly.CustomAttributes.Add(new CustomAttribute(assembly.MainModule.Import(persistableAssembly)));

			//foreach (TypeDefinition typeDefinition in assembly.MainModule.Types)
			foreach(Class currentClass in persistentClasses.Classes)
			{
				TypeDefinition typeDefinition = assembly.MainModule.Types[currentClass.Name];
				if(typeDefinition == null)
					throw new UniversalStorageException("Cannot find the class " + currentClass.Name +  " in your assembly");
				//Class currentClass = persistentClasses.Classes.GetClassByName(typeDefinition.FullName);
//				if (currentClass == null)
//					continue;
 
				_DeclaredFields = new Hashtable();
				_DeclaredProperties = new Hashtable();
				foreach (FieldDefinition field in typeDefinition.Fields)
				{
					//Gets the persistent properties => storage name = property name
					Property prop  = currentClass.Properties.GetPropertyByField(field.Name);
					if(prop == null)
						continue;
					
					_DeclaredFields.Add(field.Name, field);
					_DeclaredProperties.Add(prop.Field, prop);
					
				}

				DuplicateIPersistable(typeDefinition, assembly);
				this.ProcessBasePersistentClasses(assembly, typeDefinition, persistentClasses);

				this.InitializeFieldReferences(assembly, typeDefinition);

				Hashtable fields = GetFieldsFromType(assembly, typeDefinition);

				this.ReplaceMixinByTypeDefinition(typeDefinition.Constructors, fields);
				this.ReplaceMixinByTypeDefinition(typeDefinition.Methods, fields);

				this.InsertInitializeMethodInCtor(assembly, typeDefinition);

				foreach (Relationship relationship in currentClass.Relationships)
				{
					FieldReference childFieldReference = typeDefinition.Fields.GetField(relationship.Field);

					if (childFieldReference == null)
						throw new Exception("The relationship is not defined in the Parent");
                            
					if (relationship.Multiplicity == "*")
						this.ReplaceIListByIPersistableCollection(childFieldReference, typeDefinition, assembly);

					if (relationship.Multiplicity == "1")
						this.InsertIsLoadedFieldInstructions(typeDefinition, childFieldReference, assembly, relationship.Property);

					this.ModifyChildrenFields(typeDefinition.Methods, typeDefinition, assembly, childFieldReference, relationship.Multiplicity, relationship.Field, relationship.Property);
					this.ModifyChildrenFields(typeDefinition.Constructors, typeDefinition, assembly, childFieldReference, relationship.Multiplicity, relationship.Field, relationship.Property);

					//Modify TrackChildren method

					MethodDefinition trackChildren = typeDefinition.Methods.GetMethod("Evaluant.Uss.ObjectContext.IPersistable.TrackChildren", new TypeReference[]{});

					if(trackChildren != null)
					{
						Instruction end = trackChildren.Body.Instructions[trackChildren.Body.Instructions.Count - 1];
#if !DEBUG
						Instruction insBrs = trackChildren.Body.CilWorker.Create(OpCodes.Br_S, end);
						trackChildren.Body.CilWorker.InsertBefore(end, insBrs);
#endif
						Instruction start = trackChildren.Body.Instructions[3];
						if (relationship.Multiplicity == "*")
							this.TrackChildrenInstructionMany(start, end, trackChildren, assembly, childFieldReference);
						if (relationship.Multiplicity.Equals("1"))
							this.TrackChildrenInstructionOne(start, end, trackChildren, assembly, childFieldReference);
					}
				}

				//Add after set and before get id instructions
				if (currentClass.Id != null)
				{
					foreach (MethodDefinition methodDef in typeDefinition.Methods)
					{
						InstructionCollection instructions = methodDef.Body.Instructions;

						FieldDefinition idField = typeDefinition.Fields.GetField(currentClass.Id.Field);
								
						if (idField != null)
						{
							for (int i = 0; i < instructions.Count; i++)
							{
								Instruction ins = instructions[i];

								//AfterSetId
								if (ins.OpCode.Equals(OpCodes.Stfld))
								{
									if (((FieldReference)ins.Operand).Name == currentClass.Id.Field)
									{
										this.AfterSetIdInstructions(ref i, ins, methodDef, instructions, assembly, idField);
									}
								}
								//BeforeGetId
								if (ins.OpCode.Equals(OpCodes.Ldfld))
								{
									if (((FieldReference)ins.Operand).Name == currentClass.Id.Field)
									{
										this.BeforeGetIdInstructions(ref i, ins, methodDef, assembly, idField);
									}
								}
							}
						}
					}
				}

				this.ModifyFields(typeDefinition.Methods, assembly/*, pdbBase*/);
				this.ModifyFields(typeDefinition.Constructors, assembly/*, pdbBase*/);

				assembly.MainModule.Import(typeDefinition);
			}

			if (_saveInNewAssembly == true)
				AssemblyFactory.SaveAssembly(assembly, _newAssemblyFileName);
			else
				AssemblyFactory.SaveAssembly(assembly, _assemblyFileName);

			return 0;
		}
		#endregion

		#region Instructions

		public void AfterSetInstructions(ref int index, Instruction start, MethodDefinition methodDef, InstructionCollection instructions, AssemblyDefinition assembly, FieldReference field, string storageName)
		{
			int startindex = index;

            /*
             *	this._Name = value;
             *  //If field type is enum
             *  this._entity["EnumName"]= Enum.GetName(typeof(EnumType), this._EnumValue);
             * 
             *  //Else If field type is not enum
             *	this._entity["_Name"] = this._Name;
            */

            TypeDefinition fieldtype = field.FieldType as TypeDefinition;
            bool isEnum = false;
            if (fieldtype != null)
                isEnum = fieldtype.IsEnum;
            int iloffset = 22;
            if (isEnum)
            {
                //this._entity["EnumName"]= Enum.GetName(typeof(EnumType), this._EnumValue);
                Instruction i1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(start, i1);

                Instruction i2 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
                methodDef.Body.CilWorker.InsertAfter(i1, i2);

                Instruction i3 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, storageName);
                methodDef.Body.CilWorker.InsertAfter(i2, i3);

                Instruction i4 = methodDef.Body.CilWorker.Create(OpCodes.Ldtoken, field.FieldType);
                methodDef.Body.CilWorker.InsertAfter(i3, i4);

                Instruction i5 = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(getTypeFromHandleMethodBase));
                methodDef.Body.CilWorker.InsertAfter(i4, i5);

                Instruction i6 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(i5, i6);

                Instruction i7 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, field);
                methodDef.Body.CilWorker.InsertAfter(i6, i7);

                Instruction i8 = methodDef.Body.CilWorker.Create(OpCodes.Box, field.FieldType);
                methodDef.Body.CilWorker.InsertAfter(i7, i8);

                System.Reflection.MethodBase mb = typeof(Enum).GetMethod("GetName");
                Instruction i9 = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(mb));
                methodDef.Body.CilWorker.InsertAfter(i8, i9);

                Instruction i10 = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(entitySetItemMethodBase));
                methodDef.Body.CilWorker.InsertAfter(i9, i10);

                index += 10;
            }
            else
            {
                //	this._entity["Name"] = this._Name;

                Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(start, insertThis1);

                Instruction insertEntityField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
                methodDef.Body.CilWorker.InsertAfter(insertThis1, insertEntityField);

                Instruction insertBusinessFieldName = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, storageName);
                methodDef.Body.CilWorker.InsertAfter(insertEntityField, insertBusinessFieldName);

                Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(insertBusinessFieldName, insertThis2);

                Instruction insertBusinessField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, field);
                methodDef.Body.CilWorker.InsertAfter(insertThis2, insertBusinessField);

                Instruction insertSetItem = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(entitySetItemMethodBase));
                methodDef.Body.CilWorker.InsertAfter(insertBusinessField, insertSetItem);

                iloffset = 22;

                index += 6;

                //Type fieldType = Evaluant.Uss.Common.Utils.GetType(field.FieldType.FullName);

                if (field.FieldType.IsValueType)
                {
                    Instruction insertBox = methodDef.Body.CilWorker.Create(OpCodes.Box, field.FieldType);
                    methodDef.Body.CilWorker.InsertBefore(insertSetItem, insertBox);
                    index += 1;

                    iloffset += 5;
                }
            }

			if(_pdbModel != null)
			{
				Pdb.Model.Method method = _pdbModel.Methods.GetMethodByFullName(methodDef.ToString());
				if(method != null)
				{
					int ilStart = 0;
					for(int i = startindex; i >= 0; i--)
					{
						if(((Instruction)instructions[i]).OpCode == OpCodes.Ldarg_0)
						{
							ilStart = i + startindex;
							break;
						}
					}

					int debugLine = method.SequencePoints.GetIndexOf(((Instruction)instructions[ilStart - startindex]).Offset);

					Pdb.Model.Entry e = method.SequencePoints[debugLine + 1];
					e.Offset = e.Offset + iloffset;
				}
			}
		}

		public void BeforeGetInstructions(ref int index, Instruction start, MethodDefinition methodDef, AssemblyDefinition assembly, FieldReference field, string storageName)
		{
			int current = index;

            /*
             *	if (this._entity.FindEntry("Name") != null)
             *	{
             *      // If field type is enum
             *		 this._name = (EnumType) Enum.Parse(typeof(EnumType), (string)this._entity.GetValue("Name"));
             * 
             *      // Else if field type is not enum
             *      this._Name = (string) this._entity.GetValue("Name");
             *	}
             *	return this._Name;
             */

            //	if (this._entity.FindEntry("Name") != null)
			//	{

			Instruction insertEntityField1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
			methodDef.Body.CilWorker.InsertBefore(start, insertEntityField1);

			Instruction insertBusinessFieldName = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, storageName);
			methodDef.Body.CilWorker.InsertAfter(insertEntityField1, insertBusinessFieldName);

			Instruction insertFindEntry = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(findEntryMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertBusinessFieldName, insertFindEntry);

			//Define for jump
			Instruction insertThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);

			Instruction insertBrFalse = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertThis3);
			methodDef.Body.CilWorker.InsertAfter(insertFindEntry, insertBrFalse);

            TypeDefinition fieldtype = field.FieldType as TypeDefinition;
            bool isEnum = false;
            if (fieldtype != null)
                isEnum = fieldtype.IsEnum;

            if (isEnum)
            {
                //this._name = (EnumType) Enum.Parse(typeof(EnumType), (string)this._entity.GetValue("Name"));
                
                Instruction i1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(insertBrFalse, i1);

                Instruction i2 = methodDef.Body.CilWorker.Create(OpCodes.Ldtoken, field.FieldType);
                methodDef.Body.CilWorker.InsertAfter(i1, i2);

                Instruction i3 = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(getTypeFromHandleMethodBase));
                methodDef.Body.CilWorker.InsertAfter(i2, i3);

                Instruction i4 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(i3, i4);

                Instruction i5 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
                methodDef.Body.CilWorker.InsertAfter(i4, i5);

                Instruction i6 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, storageName);
                methodDef.Body.CilWorker.InsertAfter(i5, i6);

                Instruction i7 = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(getValueMethodBase));
                methodDef.Body.CilWorker.InsertAfter(i6, i7);

                System.Reflection.MethodBase mb = typeof(Enum).GetMethod("Parse", new Type[] { typeof(Type), typeof(string) });
                Instruction i8 = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(mb));
                methodDef.Body.CilWorker.InsertAfter(i7, i8);

                Instruction i9 = methodDef.Body.CilWorker.Create(OpCodes.Unbox_Any, field.FieldType);
                methodDef.Body.CilWorker.InsertAfter(i8, i9);

                Instruction i10 = methodDef.Body.CilWorker.Create(OpCodes.Stfld, field);
                methodDef.Body.CilWorker.InsertAfter(i9, i10);

                methodDef.Body.CilWorker.InsertAfter(i10, insertThis3);

                index += 11;
            }
            else
            {
                // this._Name = (string) this._entity.GetValue("Name");

                Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(insertBrFalse, insertThis1);

                Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
                methodDef.Body.CilWorker.InsertAfter(insertThis1, insertThis2);

                Instruction insertEntityField2 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
                methodDef.Body.CilWorker.InsertAfter(insertThis2, insertEntityField2);

                Instruction insertBusinessFieldName2 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, storageName);
                methodDef.Body.CilWorker.InsertAfter(insertEntityField2, insertBusinessFieldName2);

                Instruction insertGetValue = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(getValueMethodBase));
                methodDef.Body.CilWorker.InsertAfter(insertBusinessFieldName2, insertGetValue);

                Instruction insertSet = methodDef.Body.CilWorker.Create(OpCodes.Stfld, field);
                methodDef.Body.CilWorker.InsertAfter(insertGetValue, insertSet);

                //If value type
                //Type fieldType = Evaluant.Uss.Common.Utils.GetType(field.FieldType.FullName);

                //					System.Reflection.Assembly.Load((Type.GetType(field.FieldType.FullName).Assembly.FullName));
                if (field.FieldType.IsValueType)
                {
                    Instruction insertBox = methodDef.Body.CilWorker.Create(OpCodes.Unbox, field.FieldType);
                    methodDef.Body.CilWorker.InsertBefore(insertSet, insertBox);

                    if (_PrimitiveTypesOpCodes[field.FieldType.FullName] != null)
                    {
                        Instruction ldPrim = methodDef.Body.CilWorker.Create((OpCode)_PrimitiveTypesOpCodes[field.FieldType.FullName]);
                        methodDef.Body.CilWorker.InsertAfter(insertBox, ldPrim);
                    }
                    else
                    {
                        Instruction ldObj = methodDef.Body.CilWorker.Create(OpCodes.Ldobj, field.FieldType);
                        methodDef.Body.CilWorker.InsertAfter(insertBox, ldObj);
                    }

                    index += 2;
                }
                else
                {
                    Instruction insertCastClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, field.FieldType);
                    methodDef.Body.CilWorker.InsertBefore(insertSet, insertCastClass);

                    index += 1;
                }

                methodDef.Body.CilWorker.InsertAfter(insertSet, insertThis3);

                index += 7;
            }

            index += 4;

			//	}
					
			//	return this._Name;

//			Pdb.Model.Method method = _pdbModel.Methods.GetMethod(methodDef.ToString());
//			if(method != null)
//			{
//				foreach(Pdb.Model.Entry e in method.SequencePoints.Entries)
//				{
//					if(e.StartRow == 23)
//					{
//						e.Offset = 0x96;
//					}
//				}
//			}
		}

		public int AfterSetIdInstructions(ref int index, Instruction start, MethodDefinition methodDef, InstructionCollection instructions, AssemblyDefinition assembly, FieldDefinition idField)
		{
			/*
			 *	this._entity.set_Id((string) this._PersonId);
			 */

			//	this._entity.set_Id((string) this._PersonId);

			Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(start, insertThis1);

			Instruction insertEntityField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis1, insertEntityField);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertEntityField, insertThis2);

			Instruction insertIdField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, idField);
			methodDef.Body.CilWorker.InsertAfter(insertThis2, insertIdField);

			Instruction insertSetId = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(set_IdMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertIdField, insertSetId);

			index += 5;

			int iloffset = 17;

			return iloffset;
		}

		public int BeforeGetIdInstructions(ref int index, Instruction start, MethodDefinition methodDef, AssemblyDefinition assembly, FieldDefinition idField)
		{
			/*
			 *	this._PersonId = (int) this._entity.get_Id();
			 */

			//	this._PersonId = (int) this._entity.get_Id();

			Instruction insertThis = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(start, insertThis);

			Instruction insertEntityField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis, insertEntityField);

			Instruction insertGetId = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(get_IdMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertEntityField, insertGetId);

			Instruction insertStIdField = methodDef.Body.CilWorker.Create(OpCodes.Stfld, idField);
			methodDef.Body.CilWorker.InsertAfter(insertGetId, insertStIdField);

			Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertStIdField, insertThis1);

			index += 5;

			int iloffset = 17;

			return iloffset;
		}

		public void TrackChildrenInstructionOne(Instruction start, Instruction end, MethodDefinition methodDef, AssemblyDefinition assembly, FieldReference childFieldReference)
		{
			Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(start, insertThis1);

			Instruction insertLdChildField1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis1, insertLdChildField1);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildField1, insertThis2);

			Instruction insertLdPersistenceManager1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis2, insertLdPersistenceManager1);

			Instruction insertThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdPersistenceManager1, insertThis3);

			Instruction insertLdChildField2 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis3, insertLdChildField2);

			Instruction insertTrackObjectMethod = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(trackObjectIPersistableMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertLdChildField2, insertTrackObjectMethod);

			Instruction insertBrTrue = methodDef.Body.CilWorker.Create(OpCodes.Brtrue_S, insertThis1);
			methodDef.Body.CilWorker.Delete(methodDef.Body.Instructions[2]);
			methodDef.Body.CilWorker.Insert(methodDef.Body.Instructions[2], insertBrTrue);

			Instruction insertBrFalseChild = null;
			if (insertTrackObjectMethod.Next == null)
				insertBrFalseChild = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, end);
			else
				insertBrFalseChild = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertTrackObjectMethod.Next);

			methodDef.Body.CilWorker.InsertAfter(insertLdChildField1, insertBrFalseChild);
		}

		public void TrackChildrenInstructionMany(Instruction start, Instruction end, MethodDefinition methodDef, AssemblyDefinition assembly, FieldReference childFieldReference)
		{
			Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(start, insertThis1);

			Instruction insertLdChildField1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis1, insertLdChildField1);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildField1, insertThis2);

			Instruction insertLdPersistenceManager1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis2, insertLdPersistenceManager1);

			Instruction insertThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdPersistenceManager1, insertThis3);

			Instruction insertLdChildField2 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis3, insertLdChildField2);

            Instruction insertCastClass = null;
#if EUSS11
			insertCastClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, assembly.MainModule.Import(iPersistableCollectionType));
#else
            if (childFieldReference.FieldType.GenericParameters.Count > 0)
                insertCastClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, assembly.MainModule.Import(iPersistableCollectionGenericType));
            else
                insertCastClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, assembly.MainModule.Import(iPersistableCollectionType));
#endif

			methodDef.Body.CilWorker.InsertAfter(insertLdChildField2, insertCastClass);

			Instruction insertTrackObjectMethod = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(trackObjectIPersistableCollectionMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertCastClass, insertTrackObjectMethod);

			Instruction insertBrTrue = methodDef.Body.CilWorker.Create(OpCodes.Brtrue_S, insertThis1);
			methodDef.Body.CilWorker.Delete(methodDef.Body.Instructions[2]);
			methodDef.Body.CilWorker.Insert(methodDef.Body.Instructions[2], insertBrTrue);

			Instruction insertBrFalseChild = null;
			if (insertTrackObjectMethod.Next == null)
				insertBrFalseChild = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, end);
			else
				insertBrFalseChild = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertTrackObjectMethod.Next);

			methodDef.Body.CilWorker.InsertAfter(insertLdChildField1, insertBrFalseChild);
		}

		
		public int SetManyChildrenInstruction(ref int index, Instruction start2, MethodDefinition methodDef, AssemblyDefinition assembly, InstructionCollection instructions, FieldReference child, string propertyName)
		{
			/*
			 * 
			 *	if (this._Partners == null)
			 *	{
             *		this._Partners = new IPersistableCollection(this._persistenceManager, this, "Partners", null);
			 *	}
			 *	if (list != null)
			 *	{
             *		for (int i = 0; i < list.Count; i++)
             *		{
			 *			this._Partners.Add((IPersistable)list[i]);
             *		}
			 *	}
			 * 
			 */

			VariableDefinition collection1 = null;
            
#if EUSS11
            collection1 = new VariableDefinition(assembly.MainModule.Import(iPersistableCollectionType));
#else
            if (child.FieldType.GenericParameters.Count > 0)
                collection1 = new VariableDefinition(assembly.MainModule.Import(iPersistableCollectionGenericType));
            else
                collection1 = new VariableDefinition(assembly.MainModule.Import(iPersistableCollectionType));
#endif
			methodDef.Body.Variables.Add(collection1);

			TypeReference int32Reference = assembly.MainModule.Import(Type.GetType("System.Int32"));
			VariableDefinition num1 = new VariableDefinition(int32Reference);

			methodDef.Body.Variables.Add(num1);

			MethodReference addIPersistableCollectionMethod = null;
			
#if EUSS11
            foreach (System.Reflection.MethodBase method in iPersistableCollectionType.GetMethods())
				if (method.Name == "Add")
					addIPersistableCollectionMethod = assembly.MainModule.Import(method);
#else
            if (child.FieldType.GenericParameters.Count > 0)
            {
                foreach (System.Reflection.MethodBase method in iPersistableCollectionGenericType.GetMethods())
                    if (method.Name == "Add")
                        addIPersistableCollectionMethod = assembly.MainModule.Import(method);
            }
            else
            {
                foreach (System.Reflection.MethodBase method in iPersistableCollectionType.GetMethods())
                    if (method.Name == "Add")
                        addIPersistableCollectionMethod = assembly.MainModule.Import(method);
            }
#endif

			Instruction start = instructions[index - 1];
			Instruction end = instructions[index];

			Instruction arg = start;
			Instruction newObj = start;

			if (start.OpCode == OpCodes.Ldnull)
				throw new Exception("Cannot set a persistable items collection with Null");

			VariableDefinition list1 = new VariableDefinition("list1", 0, methodDef, assembly.MainModule.Import(iListType));

			bool ctor = false;
			if (start.OpCode == OpCodes.Newobj)
			{
				methodDef.Body.Variables.Add(list1);

//				Instruction insertNewObj = methodDef.Body.CilWorker.Create(OpCodes.Newobj, assembly.MainModule.Import((MethodReference)newObj.Operand));
//				methodDef.Body.CilWorker.InsertAfter(start, insertNewObj);

				Instruction insertStLoc3 = methodDef.Body.CilWorker.Create(OpCodes.Stloc, list1);
				methodDef.Body.CilWorker.InsertAfter(start, insertStLoc3);

//				Instruction insertLdLoc32 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc, list1);
//				methodDef.Body.CilWorker.InsertAfter(insertStLoc3, insertLdLoc32);

				end = end.Next;
				methodDef.Body.CilWorker.Delete(start.Previous);
				methodDef.Body.CilWorker.Delete(end.Previous);

				start = methodDef.Body.CilWorker.Create(OpCodes.Ldloc_S, list1);
				//start = insertLdLoc32;
				//arg = insertLdLoc32;

				ctor = true;

				index++;
			}

			Instruction argumentRef = start;

			if (start.OpCode == OpCodes.Ldarg_1 || start.OpCode == OpCodes.Ldarg_2 || start.OpCode == OpCodes.Ldarg_3)
			{
				argumentRef = start;

				methodDef.Body.CilWorker.Delete(instructions[index]);
				methodDef.Body.CilWorker.Delete(instructions[index - 1]);
				methodDef.Body.CilWorker.Delete(instructions[index - 2]);

				//start = instructions[0];
//				start = instructions[index - 2];
//				arg = start;
				end = instructions[0];

				//index -= 3;
			}

			//Initialize the list before insert new values
			//if (this._orders == null)
			//{
			//	this._orders = new IPersistableCollection(base._persistenceManager, this, "_orders", null);
			//}

			Instruction insertThis = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(end, insertThis);

			Instruction insertLdChildField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, child);
			methodDef.Body.CilWorker.InsertAfter(insertThis, insertLdChildField);

			Instruction argument = null;
			if(argumentRef.Operand == null)
				argument = methodDef.Body.CilWorker.Create(argumentRef.OpCode);
			else
				argument = methodDef.Body.CilWorker.Create(argumentRef.OpCode, (VariableDefinition)argumentRef.Operand);

			Instruction insertBrTrueChild = methodDef.Body.CilWorker.Create(OpCodes.Brtrue_S, argument);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildField, insertBrTrueChild);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertBrTrueChild, insertThis2);

			Instruction insertLdThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertThis2, insertLdThis3);

			Instruction insertLdPM = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertLdThis3, insertLdPM);

			Instruction insertLdThis4 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdPM, insertLdThis4);

			Instruction insertLdChildFieldName1 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, propertyName);
			methodDef.Body.CilWorker.InsertAfter(insertLdThis4, insertLdChildFieldName1);

			Instruction insertNull = methodDef.Body.CilWorker.Create(OpCodes.Ldnull);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildFieldName1, insertNull);

			Instruction insertIPersistableCollectionValuedCtor1 = methodDef.Body.CilWorker.Create(OpCodes.Newobj, assembly.MainModule.Import(iPersistableCollectionValuedMethodBase));
			methodDef.Body.CilWorker.InsertAfter(insertNull, insertIPersistableCollectionValuedCtor1);

			Instruction insertStChildField1 = methodDef.Body.CilWorker.Create(OpCodes.Stfld, child);
			methodDef.Body.CilWorker.InsertAfter(insertIPersistableCollectionValuedCtor1, insertStChildField1);

			//Argument is the list which is given by user (value for the set_Method of a property
			//_Field is the persistent field for a given class
			//if(Argument != null)
			//{
			//	for (int num1 = 0; num1 < Argument.Count; num1++)
			//	{
			//		_Field.Add((IPersistable) Argument[num1]);
			//	}
			//}

			methodDef.Body.CilWorker.InsertAfter(insertStChildField1, argument);

			Instruction brFalse = methodDef.Body.CilWorker.Create(OpCodes.Brfalse, end);
			methodDef.Body.CilWorker.InsertAfter(argument, brFalse);

			Instruction ldcI40 = methodDef.Body.CilWorker.Create(OpCodes.Ldc_I4_0);
			methodDef.Body.CilWorker.InsertAfter(brFalse, ldcI40);

			Instruction stNum1 = methodDef.Body.CilWorker.Create(OpCodes.Stloc_S, num1);
			methodDef.Body.CilWorker.InsertAfter(ldcI40, stNum1);
			
			Instruction ldNum1_3 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc_S, num1);

			Instruction brs = methodDef.Body.CilWorker.Create(OpCodes.Br_S, ldNum1_3);
			methodDef.Body.CilWorker.InsertAfter(stNum1, brs);

			Instruction lda0 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(brs, lda0);

			Instruction ldChild = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, child);
			methodDef.Body.CilWorker.InsertAfter(lda0, ldChild);

			Instruction argument_1;
			if(argumentRef.Operand == null)
				argument_1 = methodDef.Body.CilWorker.Create(argumentRef.OpCode);
			else
				argument_1 = methodDef.Body.CilWorker.Create(argumentRef.OpCode, (VariableDefinition)argumentRef.Operand);
			
			methodDef.Body.CilWorker.InsertAfter(ldChild, argument_1);

			Instruction ldNum1 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc_S, num1);
			methodDef.Body.CilWorker.InsertAfter(argument_1, ldNum1);

			Instruction getItem = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(iListGetItemMethodBase));
			methodDef.Body.CilWorker.InsertAfter(ldNum1, getItem);

			Instruction castClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, assembly.MainModule.Import(assembly.MainModule.Import(typeof(IPersistable))));
			methodDef.Body.CilWorker.InsertAfter(getItem, castClass);

			Instruction addIPersistable = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(addIPersistableCollectionMethod));
			methodDef.Body.CilWorker.InsertAfter(castClass, addIPersistable);

#if EUSS11
            Instruction pop = methodDef.Body.CilWorker.Create(OpCodes.Pop);
#else
            Instruction pop = null;
            if(iPersistableCollectionType == typeof(IPersistableCollection))
                pop = methodDef.Body.CilWorker.Create(OpCodes.Pop);
            else
                pop = methodDef.Body.CilWorker.Create(OpCodes.Nop);
#endif
            methodDef.Body.CilWorker.InsertAfter(addIPersistable, pop);

			Instruction ldNum1_2 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc_S, num1);
            methodDef.Body.CilWorker.InsertAfter(pop, ldNum1_2);

			Instruction ldcI41 = methodDef.Body.CilWorker.Create(OpCodes.Ldc_I4_1);
			methodDef.Body.CilWorker.InsertAfter(ldNum1_2, ldcI41);

			Instruction add = methodDef.Body.CilWorker.Create(OpCodes.Add);
			methodDef.Body.CilWorker.InsertAfter(ldcI41, add);

			Instruction stNum1_2 = methodDef.Body.CilWorker.Create(OpCodes.Stloc_S, num1);
			methodDef.Body.CilWorker.InsertAfter(add, stNum1_2);

			methodDef.Body.CilWorker.InsertAfter(stNum1_2, ldNum1_3);

			Instruction argument_2;
			if(arg.Operand == null)
				argument_2 = methodDef.Body.CilWorker.Create(argumentRef.OpCode);
			else
				argument_2 = methodDef.Body.CilWorker.Create(argumentRef.OpCode, (VariableDefinition)argumentRef.Operand);
			
			methodDef.Body.CilWorker.InsertAfter(ldNum1_3, argument_2);

			Instruction getCount = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(getCountMethodBase));
			methodDef.Body.CilWorker.InsertAfter(argument_2, getCount);

			Instruction blts = methodDef.Body.CilWorker.Create(OpCodes.Blt_S, lda0);
			methodDef.Body.CilWorker.InsertAfter(getCount, blts);

			index += 32;

			int nbIns = 29;

			if (ctor)
			{
//				Instruction insertNewObj = methodDef.Body.CilWorker.Create(OpCodes.Newobj, assembly.MainModule.Import((MethodReference)newObj.Operand));
//				methodDef.Body.CilWorker.InsertBefore(start, insertNewObj);
//
//				Instruction insertStLoc3 = methodDef.Body.CilWorker.Create(OpCodes.Stloc, list1);
//				methodDef.Body.CilWorker.InsertAfter(insertNewObj, insertStLoc3);
//
//				index += 3;
//				nbIns += 3;
			}

			//index += 32;

			for (int i = 0; i < methodDef.Body.Instructions.Count; i++)
			{
				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Brfalse_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Brfalse;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Brtrue_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Brtrue;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Ble_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Ble;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Br_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Br;
			}

			return nbIns;
		}

		public int SetOneChildInstruction(ref int index, Instruction start, MethodDefinition methodDef, FieldReference isChildLoaded, AssemblyDefinition assembly, FieldReference child, string propertyName)
		{
			/*
			 * 
			 *  if (this.m_address != null)
			 *	{
             *		this._entity.DeleteElement("Address");
			 *	}
			 *	if (value != null)
			 *	{
             *		this._entity.AddValue("Address", ((IPersistable) value).Entity);
			 *	}
			 *	this.m_address = value;
			 * 
			 */

			Instruction ins = start;

			//	if (this.m_address != null)
			//	{

				Instruction insertLdChildField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, child);
				methodDef.Body.CilWorker.InsertBefore(start, insertLdChildField);

				//Define for jump
				Instruction insertArg1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_1);

				Instruction insertBrFalse = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertArg1);
				methodDef.Body.CilWorker.InsertAfter(insertLdChildField, insertBrFalse);
			
					//	this._entity.DeleteElement("Address");

					Instruction insertThis = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
					methodDef.Body.CilWorker.InsertAfter(insertBrFalse, insertThis);

					Instruction insertEntityField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
					methodDef.Body.CilWorker.InsertAfter(insertThis, insertEntityField);

					Instruction insertChildName = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, propertyName);
					methodDef.Body.CilWorker.InsertAfter(insertEntityField, insertChildName);

					Instruction insertDeleteElement = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(deleteElementMethodBase));
					methodDef.Body.CilWorker.InsertAfter(insertChildName, insertDeleteElement);

			//	}

			//	if (value != null)
			//	{

				methodDef.Body.CilWorker.InsertAfter(insertDeleteElement, insertArg1);

				//Define for jump
				Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);

				Instruction insertBrFalse1 = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertThis2);
				methodDef.Body.CilWorker.InsertAfter(insertArg1, insertBrFalse1);
			
					//	this._entity.AddValue("Address", ((IPersistable) value).Entity);

					Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
					methodDef.Body.CilWorker.InsertAfter(insertBrFalse1, insertThis1);

					Instruction insertEntityField1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
					methodDef.Body.CilWorker.InsertAfter(insertThis1, insertEntityField1);

					Instruction insertChildName1 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, propertyName);
					methodDef.Body.CilWorker.InsertAfter(insertEntityField1, insertChildName1);

					Instruction insertArg11 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_1);
					methodDef.Body.CilWorker.InsertAfter(insertChildName1, insertArg11);

					Instruction insertGetEntity = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(getEntityMethodBase));
					methodDef.Body.CilWorker.InsertAfter(insertArg11, insertGetEntity);

					Instruction insertAddValue = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(addValueMethodBase));
					methodDef.Body.CilWorker.InsertAfter(insertGetEntity, insertAddValue);

					methodDef.Body.CilWorker.InsertBefore(ins, insertThis2);

			//	}

			index += 15;

			for (int i = 0; i < methodDef.Body.Instructions.Count; i++)
			{
				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Brfalse_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Brfalse;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Brtrue_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Brtrue;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Ble_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Ble;

				if (methodDef.Body.Instructions[i].OpCode.Equals(OpCodes.Br_S))
					methodDef.Body.Instructions[i].OpCode = OpCodes.Br;
			}

			int iloffset = 61;

			return iloffset;
		}

		public int GetOneChildInstruction(ref int index, Instruction start, MethodDefinition methodDef, FieldReference isChildLoaded, AssemblyDefinition assembly, FieldReference child, string propertyName)
		{
			MethodReference getCountPC = null;

			foreach (System.Reflection.MethodBase method in iPersistableCollectionType.GetMethods())
				if (method.Name == "get_Count")
					getCountPC = assembly.MainModule.Import(method);

			VariableDefinition collection1 = new VariableDefinition(assembly.MainModule.Import(assembly.MainModule.Import(iPersistableCollectionType)));
			methodDef.Body.Variables.Add(collection1);

			Instruction insertThis1 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(start, insertThis1);

			Instruction insertStChildField = methodDef.Body.CilWorker.Create(OpCodes.Stfld, isChildLoaded);
			methodDef.Body.CilWorker.InsertBefore(insertThis1, insertStChildField);

			Instruction insertLdc = methodDef.Body.CilWorker.Create(OpCodes.Ldc_I4_1);
			methodDef.Body.CilWorker.InsertBefore(insertStChildField, insertLdc);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(insertLdc, insertThis2);

			Instruction insertStChildField1 = methodDef.Body.CilWorker.Create(OpCodes.Stfld, child);
			methodDef.Body.CilWorker.InsertBefore(insertThis2, insertStChildField1);

			Instruction insertCastClass = methodDef.Body.CilWorker.Create(OpCodes.Castclass, assembly.MainModule.Import(child.FieldType));
			methodDef.Body.CilWorker.InsertBefore(insertStChildField1, insertCastClass);

			Instruction insertGetItem = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(iListGetItemMethodBase));
			methodDef.Body.CilWorker.InsertBefore(insertCastClass, insertGetItem);

			Instruction insertLdc1 = methodDef.Body.CilWorker.Create(OpCodes.Ldc_I4_0);
			methodDef.Body.CilWorker.InsertBefore(insertGetItem, insertLdc1);

			Instruction insertLdloc1 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc, collection1);
			methodDef.Body.CilWorker.InsertBefore(insertLdc1, insertLdloc1);

			Instruction insertThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(insertLdloc1, insertThis3);

			Instruction insertBle = methodDef.Body.CilWorker.Create(OpCodes.Ble_S, insertThis2);
			methodDef.Body.CilWorker.InsertBefore(insertThis3, insertBle);

			Instruction insertLdc2 = methodDef.Body.CilWorker.Create(OpCodes.Ldc_I4_0);
			methodDef.Body.CilWorker.InsertBefore(insertBle, insertLdc2);

			Instruction getCountMethod = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(getCountPC));
			methodDef.Body.CilWorker.InsertBefore(insertLdc2, getCountMethod);

			Instruction insertLdloc2 = methodDef.Body.CilWorker.Create(OpCodes.Ldloc, collection1);
			methodDef.Body.CilWorker.InsertBefore(getCountMethod, insertLdloc2);

			Instruction insertStloc2 = methodDef.Body.CilWorker.Create(OpCodes.Stloc, collection1);
			methodDef.Body.CilWorker.InsertBefore(insertLdloc2, insertStloc2);

			Instruction insertLoadReference = methodDef.Body.CilWorker.Create(OpCodes.Callvirt, assembly.MainModule.Import(loadReferenceMethodBase));
			methodDef.Body.CilWorker.InsertBefore(insertStloc2, insertLoadReference);

			Instruction insertChildName2 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, propertyName);
			methodDef.Body.CilWorker.InsertBefore(insertLoadReference, insertChildName2);

			Instruction insertEntityField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _entityFieldReference);
			methodDef.Body.CilWorker.InsertBefore(insertChildName2, insertEntityField);

			Instruction insertThis4 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(insertEntityField, insertThis4);

			Instruction insertPMField1 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertBefore(insertThis4, insertPMField1);

			Instruction insertThis5 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(insertPMField1, insertThis5);

			Instruction insertBrFalsePM = methodDef.Body.CilWorker.Create(OpCodes.Brfalse_S, insertThis2);
			methodDef.Body.CilWorker.InsertBefore(insertThis5, insertBrFalsePM);

			Instruction insertPMField2 = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertBefore(insertBrFalsePM, insertPMField2);

			Instruction insertThis6 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(insertPMField2, insertThis6);

			Instruction insertBrTrueChild = methodDef.Body.CilWorker.Create(OpCodes.Brtrue_S, insertThis2);
			methodDef.Body.CilWorker.InsertBefore(insertThis6, insertBrTrueChild);

			//			Instruction insertGetIsLoadedFields = methodDef.Body.CilWorker.Create(OpCodes.Call, assembly.MainModule.Import(getIsLoadedFieldMethodReference));
			//			methodDef.Body.CilWorker.InsertBefore(insertBrTrueChild, insertGetIsLoadedFields);

			Instruction insertChildField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, isChildLoaded);
			methodDef.Body.CilWorker.InsertBefore(insertBrTrueChild, insertChildField);

			index += 29;

			int iloffset = 146;

			return iloffset;
		}

		public int GetManyChildrenInstruction(ref int index, Instruction start, MethodDefinition methodDef, AssemblyDefinition assembly, FieldReference childFieldReference, string propertyName)
		{
			//Code inserted before [Start instruction] :
			//if (_Field == null)
			//{
			//	_Field = new IPersistableCollection(base._persistenceManager, this, _FieldName, null);
			//}

			index += 11;

			Instruction insertThis = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertBefore(start, insertThis);

			Instruction insertLdChildField = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertThis, insertLdChildField);

			Instruction insertBrTrueChild = methodDef.Body.CilWorker.Create(OpCodes.Brtrue_S, start);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildField, insertBrTrueChild);

			Instruction insertThis2 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertBrTrueChild, insertThis2);

			Instruction insertLdThis3 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertThis2, insertLdThis3);

			Instruction insertLdPM = methodDef.Body.CilWorker.Create(OpCodes.Ldfld, _persistenceManagerFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertLdThis3, insertLdPM);

			Instruction insertLdThis4 = methodDef.Body.CilWorker.Create(OpCodes.Ldarg_0);
			methodDef.Body.CilWorker.InsertAfter(insertLdPM, insertLdThis4);

			Instruction insertLdChildFieldName1 = methodDef.Body.CilWorker.Create(OpCodes.Ldstr, propertyName);
			methodDef.Body.CilWorker.InsertAfter(insertLdThis4, insertLdChildFieldName1);

			Instruction insertNull = methodDef.Body.CilWorker.Create(OpCodes.Ldnull);
			methodDef.Body.CilWorker.InsertAfter(insertLdChildFieldName1, insertNull);

            MethodReference ctor = null;
            GenericInstanceType gt = childFieldReference.FieldType as GenericInstanceType;
            if (gt != null)
            {
#if EUSS11
#else
				ctor = assembly.MainModule.Import(iPersistableCollectionGenericValuedMethodBase);
#endif
            }
            else
                ctor = assembly.MainModule.Import(iPersistableCollectionValuedMethodBase);

			Instruction insertIPersistableCollectionValuedCtor1 = methodDef.Body.CilWorker.Create(OpCodes.Newobj, ctor);
			methodDef.Body.CilWorker.InsertAfter(insertNull, insertIPersistableCollectionValuedCtor1);

			Instruction insertStChildField1 = methodDef.Body.CilWorker.Create(OpCodes.Stfld, childFieldReference);
			methodDef.Body.CilWorker.InsertAfter(insertIPersistableCollectionValuedCtor1, insertStChildField1);

			//index += 11;

			int iloffset = 91;

			return iloffset;
		}

		#endregion
	}
}