using System;
using System.IO;
using System.Xml;
using System.Xml.Serialization;
using System.Diagnostics;

using Evaluant.Uss.SqlMapper;

namespace Evaluant.Uss.ScriptGenerator
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class Program
	{
		[STAThread]
		public static int Main(string[] args)
		{
            try
            {
                ArgParser parser = new ArgParser(args);

                string mappingFileName = string.Empty;
                string scriptFileName = string.Empty;
                string dialect = string.Empty;

                if (parser.Exists("?"))
                {
                    Help();
                    return 0;
                }

                if (!parser.Exists("mapping"))
                {
                    Help();
                    return 1;
                }

                if (!parser.Exists("dialect"))
                {
                    Help();
                    return 1;
                }

                if (parser.Exists("dialect"))
                    dialect = parser["dialect"].Value;
 
                mappingFileName = parser["mapping"].Value;

                string mappingPath = Path.GetDirectoryName(mappingFileName);
                if (mappingPath == String.Empty)
                    mappingPath = Environment.CurrentDirectory;

                if (parser.Exists("script"))
                {
                    scriptFileName = parser["script"].Value;

                    if(Path.GetDirectoryName(scriptFileName) == String.Empty)
                        scriptFileName = String.Concat(mappingPath, "\\", scriptFileName);
                }
                else
                {
                    scriptFileName = String.Concat(mappingPath, "\\", "script.sql");
                }

                if (!File.Exists(mappingFileName))
                    throw new FileNotFoundException(String.Format("Cannot find the file \"{0}\"", mappingFileName));

                XmlTextReader reader = new XmlTextReader(mappingFileName);
                XmlSerializer serializer = new XmlSerializer(typeof(Mapping));

                Mapping mapping = (Mapping)serializer.Deserialize(reader);
                
                ScriptGenerator generator = new ScriptGenerator(mapping, scriptFileName, dialect);

                generator.GenerateScript();

                Console.WriteLine(String.Format("\"{0}\" script successfully created", scriptFileName));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                Console.WriteLine("Generation aborted !");
                return -1;
            }
			return 0;
		}

		static public void Help()
		{
			Console.WriteLine(@"

Help for scriptddl.exe

Generates the DDL script from a mapping file

scriptddl.exe /mapping:mapping_filename /dialect:dialect_type [/script:script_filename]

	mapping_filename
		The file containing the mapping

    dialect_type
        The name of the dialect to use

	script_filename
		(Optional) The file containing the generated script. If not defined the script is 
        generated on the basis of the mapping file name

example:
scriptddl.exe /mapping:""c:\model.eum.xml"" /dialect:Evaluant.Uss.SqlMapper.MsSqlDialect /script:""c:\model.sql""
");

		}
	}
}
