using System;
using System.Collections;
using UniversalStorageServices;
using UniversalStorageServices.PersistenceManager;
using Example;



namespace ManyToManyModel
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class Program
	{
		[STAThread]
		public static void Main()
		{
			Console.WriteLine();
			Console.WriteLine("===== Many-to-Many association example =====");

			PersistenceManagerFactory pmf = 
				new PersistenceManagerFactory("..\\..\\engines.config");
			PersistenceManager pm = pmf.CreatePersistenceManager();

			Company c1 = new Company();
			c1.Name = "Hanari Carnes";
			c1.Chairman = "Martn Sommer";
			c1.Address = "Rua do Pao, 67";
			c1.City = "Rio de Janeiro";

			Company c2 = new Company();
			c2.Name = "Que Delcia";
			c2.Chairman = "Bernardo Batista";
			c2.Address = "Rua da Panificadora, 12";
			c2.City = "Lisboa";

			Shareholder s1 = new Shareholder();
			s1.Name = "King Robert";
			s1.SharesNumber = 10;

			Shareholder s2 = new Shareholder();
			s2.Name = "Buchanan Steven";
			s2.SharesNumber = 78;

			c1.Shareholders.Add(s1);
			c1.Shareholders.Add(s2);

			c2.Shareholders.Add(s1);
			c2.Shareholders.Add(s2);

			pm.InitializeRepository();

			pm.BeginTransaction();
			pm.Serialize(c1);
			pm.Serialize(c2);
			pm.CommitTransaction();

			IList pc = pm.Load(typeof(Company));

			foreach(Company c in pc)
			{
				Console.WriteLine(c.Name + " owns these shareholders:");
				foreach(Shareholder s in c.Shareholders)
				{
					Console.WriteLine(s.Name + " (" + s.SharesNumber + " shares)");
				}
			}

			IList shareholders = pm.Load(typeof(Shareholder));

			foreach(Shareholder s in shareholders)
			{
				Console.WriteLine(s.Name + " owns these companies:");
				foreach(Company c in s.Parents.GetCompanies())
				{
					Console.WriteLine(c.Name);
				}
			}
		}
	}
}
