
using System;
using System.Reflection;
using System.Collections;
using UniversalStorageServices;
using UniversalStorageServices.PersistenceManager;

		

namespace Example
{
		
	public interface ICompany
	{
		string Id { get; set; }
		System.String Name { get; set; }
		System.String Chairman { get; set; }
		System.String Address { get; set; }
		System.String City { get; set; }
		
		IShareholder[]  Shareholders { get; }

	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Company'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='CompanyCollection'/>
	[Serializable()]
	public class  CompanyCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/>.
		/// </para>
		/// </summary>
		public CompanyCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/> based on another <see cref='CompanyCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='CompanyCollection'/> from which the contents are copied
		/// </param>
		public CompanyCollection( CompanyCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/> containing any array of <see cref='Company'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Company'/> objects with which to intialize the collection
		/// </param>
		public CompanyCollection( Company[] val)
		{
			this.AddRange(val);
		}
		
		public CompanyCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Company'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Company this[int index] 
		{
			get 
			{
				return ((Company)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Company'/> with the specified value to the 
		/// <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.AddRange'/>
		public int Add( Company val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='CompanyCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='CompanyCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.Add'/>
		public void AddRange( CompanyCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Company[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='CompanyCollection'/> contains the specified <see cref='Company'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Company'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.IndexOf'/>
		public bool Contains( Company val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='CompanyCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='CompanyCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='CompanyCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Company[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Company'/> in 
		/// the <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Company'/> of <paramref name='val'/> in the 
		/// <see cref='CompanyCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.Contains'/>
		public int IndexOf( Company val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Company'/> into the <see cref='CompanyCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Company'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='CompanyCollection.Add'/>
		public void Insert(int index,  Company val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  CompanyEnumerator GetEnumerator()
		{
			return new  CompanyEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Company'/> from the 
		/// <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to remove from the <see cref='CompanyCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Company val)
		{
			List.Remove(val);
		}
		
		public class  CompanyEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  CompanyEnumerator( CompanyCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Company Current 
			{
				get 
				{
					return (( Company)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Company :  IPersistable, ICompany 
	{		
	
		protected Entity _Entity;
		
		[NonSerialized]
		protected PersistenceManager _PersistenceManager;
		
		Entity IPersistable.Entity
		{
			get
			{
				return _Entity;
			}
			set
			{
				_Entity = value;
			}
		}
		
		PersistenceManager IPersistable.PersistenceManager
		{
			get
			{
				return _PersistenceManager;
			}
			set
			{
				_PersistenceManager = value;
			}
		}
		
		public string Id
		{
			get
			{
				return _Entity.Id;
			}
			set
			{
				_Entity.Id = value;
			}
		}
		
		~Company()
		{
			if(_PersistenceManager != null)
				_PersistenceManager.RemoveFromCache(this);
		}
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
			if(_Shareholders != null)
				_PersistenceManager.TrackObject(_Shareholders);
		
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public System.String Name
		{
			get 
			{ 
				if(_Entity.GetValue("Name") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Name"]; 
			}
		
			set 
			{ 
				this._Entity["Name"] = value; 
			}
		}
		
		public System.String Chairman
		{
			get 
			{ 
				if(_Entity.GetValue("Chairman") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Chairman"]; 
			}
		
			set 
			{ 
				this._Entity["Chairman"] = value; 
			}
		}
		
		public System.String Address
		{
			get 
			{ 
				if(_Entity.GetValue("Address") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Address"]; 
			}
		
			set 
			{ 
				this._Entity["Address"] = value; 
			}
		}
		
		public System.String City
		{
			get 
			{ 
				if(_Entity.GetValue("City") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["City"]; 
			}
		
			set 
			{ 
				this._Entity["City"] = value; 
			}
		}
		
		protected ShareholderCollection _Shareholders;
	
		public ShareholderCollection  Shareholders
		{
			get
			{
				if (_Shareholders == null)
				{
					if(_PersistenceManager == null) // Standalone collection
						_Shareholders = new ShareholderCollection(_Entity, "Shareholders", new IPersistableCollection());
					else
					{
						IPersistableCollection refs = _PersistenceManager.LoadReference(_Entity, "Shareholders") ;
						_Shareholders = new ShareholderCollection(_Entity, "Shareholders", refs) ;
					}
				}
				return _Shareholders;
			}	

			set
			{
				_Shareholders = value;
			}
		}	

		IShareholder[] ICompany.Shareholders
		{	                
			get 
			{ 
				IPersistable[] result = new IPersistable[Shareholders.Count];
				Shareholders.CopyTo(result, 0);
				return (IShareholder[])result; 
			}
		}
	
		public Company ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Company (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
		}
	}
	
	public interface IShareholder
	{
		string Id { get; set; }
		System.String Name { get; set; }
		System.Int32 SharesNumber { get; set; }
		
	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Shareholder'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='ShareholderCollection'/>
	[Serializable()]
	public class  ShareholderCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='ShareholderCollection'/>.
		/// </para>
		/// </summary>
		public ShareholderCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='ShareholderCollection'/> based on another <see cref='ShareholderCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='ShareholderCollection'/> from which the contents are copied
		/// </param>
		public ShareholderCollection( ShareholderCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='ShareholderCollection'/> containing any array of <see cref='Shareholder'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Shareholder'/> objects with which to intialize the collection
		/// </param>
		public ShareholderCollection( Shareholder[] val)
		{
			this.AddRange(val);
		}
		
		public ShareholderCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Shareholder'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Shareholder this[int index] 
		{
			get 
			{
				return ((Shareholder)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Shareholder'/> with the specified value to the 
		/// <see cref='ShareholderCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Shareholder'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='ShareholderCollection.AddRange'/>
		public int Add( Shareholder val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='ShareholderCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='ShareholderCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='ShareholderCollection.Add'/>
		public void AddRange( ShareholderCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Shareholder[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='ShareholderCollection'/> contains the specified <see cref='Shareholder'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Shareholder'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Shareholder'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='ShareholderCollection.IndexOf'/>
		public bool Contains( Shareholder val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='ShareholderCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='ShareholderCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='ShareholderCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Shareholder[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Shareholder'/> in 
		/// the <see cref='ShareholderCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Shareholder'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Shareholder'/> of <paramref name='val'/> in the 
		/// <see cref='ShareholderCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='ShareholderCollection.Contains'/>
		public int IndexOf( Shareholder val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Shareholder'/> into the <see cref='ShareholderCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Shareholder'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='ShareholderCollection.Add'/>
		public void Insert(int index,  Shareholder val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='ShareholderCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  ShareholderEnumerator GetEnumerator()
		{
			return new  ShareholderEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Shareholder'/> from the 
		/// <see cref='ShareholderCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Shareholder'/> to remove from the <see cref='ShareholderCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Shareholder val)
		{
			List.Remove(val);
		}
		
		public class  ShareholderEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  ShareholderEnumerator( ShareholderCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Shareholder Current 
			{
				get 
				{
					return (( Shareholder)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Shareholder :  IPersistable, IShareholder 
	{		
	
		protected Entity _Entity;
		
		[NonSerialized]
		protected PersistenceManager _PersistenceManager;
		
		Entity IPersistable.Entity
		{
			get
			{
				return _Entity;
			}
			set
			{
				_Entity = value;
			}
		}
		
		PersistenceManager IPersistable.PersistenceManager
		{
			get
			{
				return _PersistenceManager;
			}
			set
			{
				_PersistenceManager = value;
			}
		}
		
		public string Id
		{
			get
			{
				return _Entity.Id;
			}
			set
			{
				_Entity.Id = value;
			}
		}
		
		~Shareholder()
		{
			if(_PersistenceManager != null)
				_PersistenceManager.RemoveFromCache(this);
		}
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public System.String Name
		{
			get 
			{ 
				if(_Entity.GetValue("Name") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Name"]; 
			}
		
			set 
			{ 
				this._Entity["Name"] = value; 
			}
		}
		
		public System.Int32 SharesNumber
		{
			get 
			{ 
				if(_Entity.GetValue("SharesNumber") == null)
					return (new System.Int32[1])[0];
				else
					return (System.Int32)this._Entity["SharesNumber"]; 
			}
		
			set 
			{ 
				this._Entity["SharesNumber"] = value; 
			}
		}
		
		public Shareholder ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Shareholder (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
			public Company[] GetCompanies()
			{
				IPersistableCollection items = _Owner.PersistenceManager.Load(typeof(Company), String.Format("[ count(Shareholders[Id = '{0}']) > 0]", _Owner.Id));
				Company[] result = new Company[items.Count];
				items.CopyTo(result, 0);

				return result;
			}
	
		}
	}

}
