
using System;
using System.Reflection;
using System.Collections;
using UniversalStorageServices;
using UniversalStorageServices.PersistenceManager;

		

namespace ObjectModel
{
	
	public enum  Offices 
	{ 
		Accountant,
		Worker,
		Technician
	}
	
	public interface IPerson
	{
		string Id { get; set; }
		System.String Name { get; set; }
		System.String Firstname { get; set; }
		System.DateTime Birthday { get; set; }
		
		IPerson[]  Partners { get; }

	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Person'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='PersonCollection'/>
	[Serializable()]
	public class  PersonCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='PersonCollection'/>.
		/// </para>
		/// </summary>
		public PersonCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='PersonCollection'/> based on another <see cref='PersonCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='PersonCollection'/> from which the contents are copied
		/// </param>
		public PersonCollection( PersonCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='PersonCollection'/> containing any array of <see cref='Person'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Person'/> objects with which to intialize the collection
		/// </param>
		public PersonCollection( Person[] val)
		{
			this.AddRange(val);
		}
		
		public PersonCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Person'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Person this[int index] 
		{
			get 
			{
				return ((Person)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Person'/> with the specified value to the 
		/// <see cref='PersonCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Person'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='PersonCollection.AddRange'/>
		public int Add( Person val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='PersonCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='PersonCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='PersonCollection.Add'/>
		public void AddRange( PersonCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Person[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='PersonCollection'/> contains the specified <see cref='Person'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Person'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Person'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='PersonCollection.IndexOf'/>
		public bool Contains( Person val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='PersonCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='PersonCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='PersonCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Person[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Person'/> in 
		/// the <see cref='PersonCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Person'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Person'/> of <paramref name='val'/> in the 
		/// <see cref='PersonCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='PersonCollection.Contains'/>
		public int IndexOf( Person val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Person'/> into the <see cref='PersonCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Person'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='PersonCollection.Add'/>
		public void Insert(int index,  Person val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='PersonCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  PersonEnumerator GetEnumerator()
		{
			return new  PersonEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Person'/> from the 
		/// <see cref='PersonCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Person'/> to remove from the <see cref='PersonCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Person val)
		{
			List.Remove(val);
		}
		
		public class  PersonEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  PersonEnumerator( PersonCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Person Current 
			{
				get 
				{
					return (( Person)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Person :  IPersistable, IPerson 
	{		
	
		protected Entity _Entity;
		
		[NonSerialized]
		protected PersistenceManager _PersistenceManager;
		
		Entity IPersistable.Entity
		{
			get
			{
				return _Entity;
			}
			set
			{
				_Entity = value;
			}
		}
		
		PersistenceManager IPersistable.PersistenceManager
		{
			get
			{
				return _PersistenceManager;
			}
			set
			{
				_PersistenceManager = value;
			}
		}
		
		public string Id
		{
			get
			{
				return _Entity.Id;
			}
			set
			{
				_Entity.Id = value;
			}
		}
		
		~Person()
		{
			if(_PersistenceManager != null)
				_PersistenceManager.RemoveFromCache(this);
		}
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
			if(_Partners != null)
				_PersistenceManager.TrackObject(_Partners);
		
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public System.String Name
		{
			get 
			{ 
				if(_Entity.GetValue("Name") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Name"]; 
			}
		
			set 
			{ 
				this._Entity["Name"] = value; 
			}
		}
		
		public System.String Firstname
		{
			get 
			{ 
				if(_Entity.GetValue("Firstname") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Firstname"]; 
			}
		
			set 
			{ 
				this._Entity["Firstname"] = value; 
			}
		}
		
		public System.DateTime Birthday
		{
			get 
			{ 
				if(_Entity.GetValue("Birthday") == null)
					return (new System.DateTime[1])[0];
				else
					return (System.DateTime)this._Entity["Birthday"]; 
			}
		
			set 
			{ 
				this._Entity["Birthday"] = value; 
			}
		}
		
		protected PersonCollection _Partners;
	
		public PersonCollection  Partners
		{
			get
			{
				if (_Partners == null)
				{
					if(_PersistenceManager == null) // Standalone collection
						_Partners = new PersonCollection(_Entity, "Partners", new IPersistableCollection());
					else
					{
						IPersistableCollection refs = _PersistenceManager.LoadReference(_Entity, "Partners") ;
						_Partners = new PersonCollection(_Entity, "Partners", refs) ;
					}
				}
				return _Partners;
			}	

			set
			{
				_Partners = value;
			}
		}	

		IPerson[] IPerson.Partners
		{	                
			get 
			{ 
				IPersistable[] result = new IPersistable[Partners.Count];
				Partners.CopyTo(result, 0);
				return (IPerson[])result; 
			}
		}
	
		public Person ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Person (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
			public Person GetPerson()
			{
				IPersistableCollection items = _Owner.PersistenceManager.Load(typeof(Person), String.Format("[count(Partners[Id = '{0}']) > 0]", _Owner.Id));
			
				if (items.Count > 0)
					return (Person)items[0];
				else
					return null;
			}
	
		}
	}
	
	public interface IEmployee
	{
		string Id { get; set; }
		Offices Office { get; set; }
		
	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Employee'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='EmployeeCollection'/>
	[Serializable()]
	public class  EmployeeCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/>.
		/// </para>
		/// </summary>
		public EmployeeCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/> based on another <see cref='EmployeeCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='EmployeeCollection'/> from which the contents are copied
		/// </param>
		public EmployeeCollection( EmployeeCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/> containing any array of <see cref='Employee'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Employee'/> objects with which to intialize the collection
		/// </param>
		public EmployeeCollection( Employee[] val)
		{
			this.AddRange(val);
		}
		
		public EmployeeCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Employee'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Employee this[int index] 
		{
			get 
			{
				return ((Employee)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Employee'/> with the specified value to the 
		/// <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.AddRange'/>
		public int Add( Employee val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='EmployeeCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='EmployeeCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.Add'/>
		public void AddRange( EmployeeCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Employee[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='EmployeeCollection'/> contains the specified <see cref='Employee'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Employee'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.IndexOf'/>
		public bool Contains( Employee val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='EmployeeCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='EmployeeCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='EmployeeCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Employee[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Employee'/> in 
		/// the <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Employee'/> of <paramref name='val'/> in the 
		/// <see cref='EmployeeCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.Contains'/>
		public int IndexOf( Employee val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Employee'/> into the <see cref='EmployeeCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Employee'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='EmployeeCollection.Add'/>
		public void Insert(int index,  Employee val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  EmployeeEnumerator GetEnumerator()
		{
			return new  EmployeeEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Employee'/> from the 
		/// <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to remove from the <see cref='EmployeeCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Employee val)
		{
			List.Remove(val);
		}
		
		public class  EmployeeEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  EmployeeEnumerator( EmployeeCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Employee Current 
			{
				get 
				{
					return (( Employee)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Employee : Person, IPersistable, IEmployee 
	{		
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
			if(_Partners != null)
				_PersistenceManager.TrackObject(_Partners);
		
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public Offices Office
		{
			get
			{
				if(_Entity.GetValue("Office") == null)
					return (new Offices[1])[0];
				else
					return (Offices)this._Entity["Office"]; 
			}
		
			set 
			{ 
				this._Entity["Office"] = value; 
			}
		}
			
		public Employee ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Employee (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public new ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public new class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
			public Company GetCompany()
			{
				IPersistableCollection items = _Owner.PersistenceManager.Load(typeof(Company), String.Format("[count(Employees[Id = '{0}']) > 0]", _Owner.Id));
			
				if (items.Count > 0)
					return (Company)items[0];
				else
					return null;
			}
	
		}
	}
	
	public interface ICompany
	{
		string Id { get; set; }
		System.String Name { get; set; }
		System.String Chairman { get; set; }
		System.String Address { get; set; }
		System.String City { get; set; }
		
		IEmployee[]  Employees { get; }

	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Company'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='CompanyCollection'/>
	[Serializable()]
	public class  CompanyCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/>.
		/// </para>
		/// </summary>
		public CompanyCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/> based on another <see cref='CompanyCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='CompanyCollection'/> from which the contents are copied
		/// </param>
		public CompanyCollection( CompanyCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='CompanyCollection'/> containing any array of <see cref='Company'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Company'/> objects with which to intialize the collection
		/// </param>
		public CompanyCollection( Company[] val)
		{
			this.AddRange(val);
		}
		
		public CompanyCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Company'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Company this[int index] 
		{
			get 
			{
				return ((Company)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Company'/> with the specified value to the 
		/// <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.AddRange'/>
		public int Add( Company val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='CompanyCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='CompanyCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.Add'/>
		public void AddRange( CompanyCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Company[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='CompanyCollection'/> contains the specified <see cref='Company'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Company'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.IndexOf'/>
		public bool Contains( Company val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='CompanyCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='CompanyCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='CompanyCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Company[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Company'/> in 
		/// the <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Company'/> of <paramref name='val'/> in the 
		/// <see cref='CompanyCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='CompanyCollection.Contains'/>
		public int IndexOf( Company val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Company'/> into the <see cref='CompanyCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Company'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='CompanyCollection.Add'/>
		public void Insert(int index,  Company val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  CompanyEnumerator GetEnumerator()
		{
			return new  CompanyEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Company'/> from the 
		/// <see cref='CompanyCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Company'/> to remove from the <see cref='CompanyCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Company val)
		{
			List.Remove(val);
		}
		
		public class  CompanyEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  CompanyEnumerator( CompanyCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Company Current 
			{
				get 
				{
					return (( Company)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Company :  IPersistable, ICompany 
	{		
	
		protected Entity _Entity;
		
		[NonSerialized]
		protected PersistenceManager _PersistenceManager;
		
		Entity IPersistable.Entity
		{
			get
			{
				return _Entity;
			}
			set
			{
				_Entity = value;
			}
		}
		
		PersistenceManager IPersistable.PersistenceManager
		{
			get
			{
				return _PersistenceManager;
			}
			set
			{
				_PersistenceManager = value;
			}
		}
		
		public string Id
		{
			get
			{
				return _Entity.Id;
			}
			set
			{
				_Entity.Id = value;
			}
		}
		
		~Company()
		{
			if(_PersistenceManager != null)
				_PersistenceManager.RemoveFromCache(this);
		}
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
			if(_Employees != null)
				_PersistenceManager.TrackObject(_Employees);
		
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public System.String Name
		{
			get 
			{ 
				if(_Entity.GetValue("Name") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Name"]; 
			}
		
			set 
			{ 
				this._Entity["Name"] = value; 
			}
		}
		
		public System.String Chairman
		{
			get 
			{ 
				if(_Entity.GetValue("Chairman") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Chairman"]; 
			}
		
			set 
			{ 
				this._Entity["Chairman"] = value; 
			}
		}
		
		public System.String Address
		{
			get 
			{ 
				if(_Entity.GetValue("Address") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["Address"]; 
			}
		
			set 
			{ 
				this._Entity["Address"] = value; 
			}
		}
		
		public System.String City
		{
			get 
			{ 
				if(_Entity.GetValue("City") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["City"]; 
			}
		
			set 
			{ 
				this._Entity["City"] = value; 
			}
		}
		
		protected EmployeeCollection _Employees;
	
		public EmployeeCollection  Employees
		{
			get
			{
				if (_Employees == null)
				{
					if(_PersistenceManager == null) // Standalone collection
						_Employees = new EmployeeCollection(_Entity, "Employees", new IPersistableCollection());
					else
					{
						IPersistableCollection refs = _PersistenceManager.LoadReference(_Entity, "Employees") ;
						_Employees = new EmployeeCollection(_Entity, "Employees", refs) ;
					}
				}
				return _Employees;
			}	

			set
			{
				_Employees = value;
			}
		}	

		IEmployee[] ICompany.Employees
		{	                
			get 
			{ 
				IPersistable[] result = new IPersistable[Employees.Count];
				Employees.CopyTo(result, 0);
				return (IEmployee[])result; 
			}
		}
	
		public Company ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Company (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
		}
	}

}
