using System;
using UniversalStorageServices;


namespace ComplexTypeAttribute
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class BasicTypeAttribute
	{
		[STAThread]
		public static void Main()
		{
			IPersistenceEngineFactory pef = 
				XmlConfigLoader.LoadXmlConfig("..\\..\\engines.config");
			IPersistenceEngine pe = pef.CreatePersistenceEngine();

			Entity partner1 = new Entity("Partner");
			partner1.AddValue("Name", "Moos");
			partner1.AddValue("FirstName", "Robert");

			Entity partner2 = new Entity("Partner");
			partner2.AddValue("Name", "Kuamar");
			partner2.AddValue("FirstName", "Harry");

			Entity order1 = new Entity("Order");
			order1.AddValue("ProductName", "Coffee");
			order1.AddValue("Quantity", 1);

			Entity order2 = new Entity("Order");
			order2.AddValue("ProductName", "Chocolade");
			order2.AddValue("Quantity", 5);

			Entity customer = new Entity("Customer");
			customer.AddValue("Name", "Ottlieb");
			customer.AddValue("FirstName", "Sven");

			customer.AddValue("Orders", order1);
			customer.AddValue("Orders", order2);

			customer.AddValue("Partners", partner1);
			customer.AddValue("Partners", partner2);

			pe.InitializeRepository();

			Transaction t = new Transaction();
			t.Serialize(customer);
			pe.Commit(t);

			//Loading all references in the same time as the parent
			EntitySet es1 = pe.Load("Customer", "Orders,Partners");
			
			foreach(Entity currentCustomer in es1)
			{
				pe.LoadAttribute(currentCustomer);

				string name = currentCustomer.GetString("Name");
				string firstname = currentCustomer.GetString("FirstName");
				Console.WriteLine(name + " " + firstname + " orders:");

				foreach(Entity currentOrder in currentCustomer.GetEntitySet("Orders"))
				{
					pe.LoadAttribute(currentOrder);

					Console.WriteLine(currentOrder.GetString("ProductName"));
				}

				Console.WriteLine(name + " " + firstname + " has these partners:");

				foreach(Entity currentPartner in currentCustomer.GetEntitySet("Partners"))
				{
					pe.LoadAttribute(currentPartner);

					Console.WriteLine(currentPartner.GetString("Name"));
				}
			}

			//Just in time attributes loading (only the Name attribute)

			//Loading all references in the same time as the parent

			EntitySet es2 = pe.Load("Customer");
			
			foreach(Entity currentCustomer in es2)
			{
				pe.LoadAttribute(currentCustomer);

				string name = currentCustomer.GetString("Name");
				string firstname = currentCustomer.GetString("FirstName");

				pe.LoadReference(currentCustomer, "Orders", true);

				Console.WriteLine(name + " " + firstname + " orders:");

				foreach(Entity currentOrder in currentCustomer.GetEntitySet("Orders"))
				{
					Console.WriteLine(currentOrder.GetString("ProductName"));
				}

				Console.WriteLine(name + " " + firstname + " has these partners:");

				pe.LoadReference(currentCustomer, "Partners", true);

				foreach(Entity currentPartner in currentCustomer.GetEntitySet("Partners"))
				{
					Console.WriteLine(currentPartner.GetString("Name"));
				}
			}
		}
	}
}
