
using System;
using System.Reflection;
using System.Collections;
using UniversalStorageServices;
using UniversalStorageServices.PersistenceManager;

		

namespace Demo
{
		
	public interface IEmployee
	{
		string Id { get; set; }
		System.String LastName { get; set; }
		
	}
	
	
		/// <summary>
	/// <para>
	/// A collection that stores <see cref='Employee'/> objects.
	/// </para>
	/// </summary>
	/// <seealso cref='EmployeeCollection'/>
	[Serializable()]
	public class  EmployeeCollection : IPersistableCollection 
	{	
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/>.
		/// </para>
		/// </summary>
		public EmployeeCollection()
		{
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/> based on another <see cref='EmployeeCollection'/>.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='EmployeeCollection'/> from which the contents are copied
		/// </param>
		public EmployeeCollection( EmployeeCollection val)
		{
			this.AddRange(val);	
		}
		
		/// <summary>
		/// <para>
		/// Initializes a new instance of <see cref='EmployeeCollection'/> containing any array of <see cref='Employee'/> objects.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A array of <see cref='Employee'/> objects with which to intialize the collection
		/// </param>
		public EmployeeCollection( Employee[] val)
		{
			this.AddRange(val);
		}
		
		public EmployeeCollection (Entity parentEntity, string role, IPersistableCollection data): base(parentEntity, role, data)
		{
		}
		
		/// <summary>
		/// <para>Represents the entry at the specified index of the <see cref='Employee'/>.</para>
		/// </summary>
		/// <param name='index'><para>The zero-based index of the entry to locate in the collection.</para></param>
		/// <value>
		/// <para> The entry at the specified index of the collection.</para>
		/// </value>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is outside the valid range of indexes for the collection.</exception>
		public new Employee this[int index] 
		{
			get 
			{
				return ((Employee)base[index]);
			}
			set 
			{
				base[index] = value;
			}
		}
		
		/// <summary>
		/// <para>Adds a <see cref='Employee'/> with the specified value to the 
		/// <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to add.</param>
		/// <returns>
		/// <para>The index at which the new element was inserted.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.AddRange'/>
		public int Add( Employee val)
		{
			return base.Add(val);
		}
		
	
		
		/// <summary>
		/// <para>
		/// Adds the contents of another <see cref='EmployeeCollection'/> to the end of the collection.
		/// </para>
		/// </summary>
		/// <param name='val'>
		/// A <see cref='EmployeeCollection'/> containing the objects to add to the collection.
		/// </param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.Add'/>
		public void AddRange( EmployeeCollection val)
		{
			base.AddRange(val);
		}
		public void AddRange( Employee[] val)
		{
			base.AddRange(val);
		}
		
		/// <summary>
		/// <para>Gets a value indicating whether the 
		/// <see cref='EmployeeCollection'/> contains the specified <see cref='Employee'/>.</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to locate.</param>
		/// <returns>
		/// <para><see langword='true'/> if the <see cref='Employee'/> is contained in the collection; 
		/// otherwise, <see langword='false'/>.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.IndexOf'/>
		public bool Contains( Employee val)
		{
			return List.Contains(val);
		}
		
		/// <summary>
		/// <para>Copies the <see cref='EmployeeCollection'/> values to a one-dimensional <see cref='System.Array'/> instance at the 
		/// specified index.</para>
		/// </summary>
		/// <param name='array'><para>The one-dimensional <see cref='System.Array'/> that is the destination of the values copied from <see cref='EmployeeCollection'/> .</para></param>
		/// <param name='index'>The index in <paramref name='array'/> where copying begins.</param>
		/// <returns>
		/// <para>None.</para>
		/// </returns>
		/// <exception cref='System.ArgumentException'><para><paramref name='array'/> is multidimensional.</para> <para>-or-</para> <para>The number of elements in the <see cref='EmployeeCollection'/> is greater than the available space between <paramref name='index'/> and the end of <paramref name='array'/>.</para></exception>
		/// <exception cref='System.ArgumentNullException'><paramref name='array'/> is <see langword='null'/>. </exception>
		/// <exception cref='System.ArgumentOutOfRangeException'><paramref name='index'/> is less than <paramref name='array'/>'s lowbound. </exception>
		/// <seealso cref='System.Array'/>
		public void CopyTo( Employee[] array, int index)
		{
			List.CopyTo(array, index);
		}
		
		/// <summary>
		/// <para>Returns the index of a <see cref='Employee'/> in 
		/// the <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to locate.</param>
		/// <returns>
		/// <para>The index of the <see cref='Employee'/> of <paramref name='val'/> in the 
		/// <see cref='EmployeeCollection'/>, if found; otherwise, -1.</para>
		/// </returns>
		/// <seealso cref='EmployeeCollection.Contains'/>
		public int IndexOf( Employee val)
		{
			return List.IndexOf(val);
		}
		
		/// <summary>
		/// <para>Inserts a <see cref='Employee'/> into the <see cref='EmployeeCollection'/> at the specified index.</para>
		/// </summary>
		/// <param name='index'>The zero-based index where <paramref name='val'/> should be inserted.</param>
		/// <param name='val'>The <see cref='Employee'/> to insert.</param>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='EmployeeCollection.Add'/>
		public void Insert(int index,  Employee val)
		{
			List.Insert(index, val);
		}
		
		/// <summary>
		/// <para>Returns an enumerator that can iterate through 
		/// the <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <returns><para>None.</para></returns>
		/// <seealso cref='System.Collections.IEnumerator'/>
		public new  EmployeeEnumerator GetEnumerator()
		{
			return new  EmployeeEnumerator(this);
		}
		
		/// <summary>
		/// <para> Removes a specific <see cref='Employee'/> from the 
		/// <see cref='EmployeeCollection'/> .</para>
		/// </summary>
		/// <param name='val'>The <see cref='Employee'/> to remove from the <see cref='EmployeeCollection'/> .</param>
		/// <returns><para>None.</para></returns>
		/// <exception cref='System.ArgumentException'><paramref name='val'/> is not found in the Collection. </exception>
		public void Remove( Employee val)
		{
			List.Remove(val);
		}
		
		public class  EmployeeEnumerator : IEnumerator
		{
			IEnumerator baseEnumerator;
			IEnumerable temp;
			
			public  EmployeeEnumerator( EmployeeCollection mappings)
			{
				this.temp = mappings;
				this.baseEnumerator = temp.GetEnumerator();
			}
			
			public  Employee Current 
			{
				get 
				{
					return (( Employee)(baseEnumerator.Current));
				}
			}
			
			object IEnumerator.Current 
			{
				get 
				{
					return baseEnumerator.Current;
				}
			}
			
			public bool MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			bool IEnumerator.MoveNext()
			{
				return baseEnumerator.MoveNext();
			}
			
			public void Reset()
			{
				baseEnumerator.Reset();
			}
			
			void IEnumerator.Reset()
			{
				baseEnumerator.Reset();
			}
		}
	}

	
	[Serializable]
	public class Employee :  IPersistable, IEmployee 
	{		
	
		protected Entity _Entity;
		
		[NonSerialized]
		protected PersistenceManager _PersistenceManager;
		
		Entity IPersistable.Entity
		{
			get
			{
				return _Entity;
			}
			set
			{
				_Entity = value;
			}
		}
		
		PersistenceManager IPersistable.PersistenceManager
		{
			get
			{
				return _PersistenceManager;
			}
			set
			{
				_PersistenceManager = value;
			}
		}
		
		public string Id
		{
			get
			{
				return _Entity.Id;
			}
			set
			{
				_Entity.Id = value;
			}
		}
		
		~Employee()
		{
			if(_PersistenceManager != null)
				_PersistenceManager.RemoveFromCache(this);
		}
	
		void IPersistable.TrackChildren()
		{
			// if the object is not tracked there is nothing to do
			if(_PersistenceManager == null)
				return;
	
		}
		
		void IPersistable.DeleteChildren()
		{	
	
		}
	
		public System.String LastName
		{
			get 
			{ 
				if(_Entity.GetValue("LastName") == null)
					return (new System.String[1])[0];
				else
					return (System.String)this._Entity["LastName"]; 
			}
		
			set 
			{ 
				this._Entity["LastName"] = value; 
			}
		}
		
		public Employee ()
		{
			
			this._Entity = new Entity(PersistenceManager.ConvertNamespaceDomainToEuss(this.GetType().FullName));
		}

		public Employee (Entity e): this()
		{
			this._Entity = e;
		}

		private ParentRelationships _Parents;

		public ParentRelationships Parents
		{
			get
			{
				if (_Parents == null)
					if (_PersistenceManager == null)
						throw new ApplicationException("Can't call Parents untracked objects");
				else
					_Parents = new ParentRelationships(this);

				return _Parents;
			}
		}

		public class ParentRelationships
		{
			private IPersistable _Owner;
	
			public ParentRelationships(IPersistable o)
			{
				_Owner = o;
			}
		
		}
	}

}
