using System;
using System.IO;
using System.Xml;

using Evaluant.Uss.Xmi;

namespace Evaluant.Uss.DomainModelGenerator
{
	/// <summary>
	/// Description rsume de ScriptGenerator.
	/// </summary>
	public class DomainModelGenerator
	{
        #region Members
		
        private string _XmiFileName;
        private string _ModelFileName;

        #endregion

        #region Properties

		public string XmiFileName
		{
			get { return _XmiFileName; }
			set { _XmiFileName = value; }
		}

        public string ModelFileName
		{
			get { return _ModelFileName; }
			set { _ModelFileName = value; }
		}

        #endregion

		#region Ctor

		public DomainModelGenerator()
		{
		}

        public DomainModelGenerator(string xmiFileName): this(xmiFileName, string.Empty)
        {
        }

		public DomainModelGenerator(string xmiFileName, string modelFileName)
		{
            _XmiFileName = xmiFileName;
            _ModelFileName = modelFileName;
		}
		
		#endregion

        #region Generate

        public void GenerateDomainModel()
		{
			if(_XmiFileName == null || _XmiFileName == String.Empty)
				throw new DomainModelGeneratorException("_XmiFileName is not defined");

            if (_ModelFileName == null || _ModelFileName == String.Empty)
                throw new DomainModelGeneratorException("_ModelFileName is not defined");

            StreamReader reader = new StreamReader(_XmiFileName);
            string strXmi = reader.ReadToEnd();

            string strXmlModel = Converter.ConvertToSimple(strXmi, Converter.GetVersion(strXmi), Converter.GetExporter(strXmi));
            reader.Close();

            // Practical way of inserting the default namespace as no way was found to convert the default namespace name
            strXmlModel = strXmlModel.Replace("<Model", "<Model xmlns=\"http://euss.evaluant.com/schemas/GenerationModel.xsd\" ");

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(strXmlModel);
            xmlDoc.Save(_ModelFileName);
        }

        #endregion
    }
}
