using System;
using System.Collections;
using System.Threading;

using Evaluant.Uss.Common;
using Evaluant.Uss.MetaData;

namespace Evaluant.Uss.Memory
{
	/// <summary>
	/// Description rsume de MemoryProvider.
	/// </summary>
	public class MemoryProvider : PersistenceEngineFactoryImplementation
	{
		public MemoryProvider() : base()
		{
		}

		public MemoryProvider(IPersistenceProvider delegator) : base()
		{
			_Delegator = delegator;
		}

		public override void InitializeConfiguration()
		{
			_Entities = new Hashtable(10000);
            _RWL = new ReaderWriterLock();
			InitializeData();
		}

		public override IPersistenceEngine CreatePersistenceEngine()
		{
			if(_Delegator != null)
				return new MemoryEngine(_Delegator.CreatePersistenceEngine(), _Entities, _RWL, _Model, _Culture);
			else
				return new MemoryEngine(null, _Entities, _RWL, _Model, _Culture);
		}

		public override void RegisterMetaData(IMetaData[] metadata)
		{
			base.RegisterMetaData(metadata);

			if(_Delegator != null)
				_Delegator.RegisterMetaData(metadata);
		}

		private IPersistenceProvider _Delegator;
		public IPersistenceProvider Delegator
		{
			get { return _Delegator; }
			set { _Delegator = value; }
		}

		protected Hashtable _Entities;
        protected ReaderWriterLock _RWL;
        
		protected void InitializeData()
		{
			if(_Delegator == null)
				return;

			// Instantiates the Delegated persistence engine to load all its content
			IPersistenceEngine delegator = _Delegator.CreatePersistenceEngine();
        
			foreach(Models.Entity me in this._Model.Entities)
			{
				// Process only super types
				if(me.Inherit == null || me.Inherit == String.Empty)
				{
					foreach(Entity e in delegator.Load(me.Type))
						_Entities.Add(Utils.GetCacheKey(e.Type, e.Id), e);
				}
			}

			// Loads all references of each entity, and try to find the corresponding entity locally
			foreach(Entity entity in _Entities.Values)
			{
                Entity entityClone = entity.Clone();
                delegator.LoadReference(entityClone);

                foreach (EntityEntry ee in entityClone)
				{
					if(ee.IsEntity)
					{
						// Find this reference locally
						Entity localRef = (Entity)_Entities[Utils.GetCacheKey((Entity)ee.Value)];

						// Creates the reference
						if(localRef == null)
							throw new PersistenceEngineException("Integrity exception");

						entity.AddValue(ee.Name, localRef, typeof(Entity), ee.State);
					}
				}
			}
		}
	}
}
