using System;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

namespace Evaluant.Uss.Models
{
	/// <summary>
	/// Description rsume de Entity.
	/// </summary>
	/// 
	[Serializable]
	public class Entity
	{
		private string _Type;
        private string _Inherit;
        private string _Implement;
        private bool _IsInterface;
        private string[] _Interfaces;
        private AttributeCollection _Attributes;
		private ReferenceCollection _References;

		public Entity(string type, string inherit)
		{
			_Type = type;
			_Inherit = inherit;
            _Interfaces = new string[0];
			_Attributes = new AttributeCollection();
			_References = new ReferenceCollection();
		}

		public Entity(string type) : this(type, String.Empty)
		{
		}

		public Entity() : this(String.Empty, String.Empty)
		{
		}

        public string[] Interfaces
        {
            get { return _Interfaces; }
        }

		[XmlAttribute("type", DataType="string")]
		public string Type
		{
			get	{ return _Type;	}
			set { _Type = value; }
		}

        [XmlAttribute("inherit", DataType = "string")]
        public string Inherit
        {
            get { return _Inherit; }
            set { _Inherit = value; }
        }

        /// <summary>
        /// Gets or sets the implemented interfaces in the whole hierarchy, even of higher levels.
        /// </summary>
        /// <value>The interfaces.</value>
        [XmlAttribute("implement", DataType = "string")]
        public string Implement
        {
            get { return _Implement; }
            set 
            { 
                _Implement = value;
                _Interfaces = _Implement.Split(',');

                for (int i = 0; i < _Interfaces.Length; i++)
                {
                    _Interfaces[i] = _Interfaces[i].Trim();
                }
            }
        }


        [XmlAttribute("isInterface")]
        public bool IsInterface
        {
            get { return _IsInterface; }
            set { _IsInterface = value; }
        }

        [XmlElement("Attribute")]
		public AttributeCollection Attributes
		{
			get { return _Attributes; }
			set	{ _Attributes = value; }
		}

		[XmlElement("Reference")]
		public ReferenceCollection References
		{
			get { return _References; }
			set	{ _References = value; }
		}

        public override string ToString()
        {
            return _Type;
        }
	}
}
