using System;
using System.Data;
using System.Collections;
using Evaluant.OPath;

using Evaluant.Uss;
using Evaluant.Uss.Commands;
using Evaluant.Uss.Common;

using Evaluant.Uss.SqlMapper;
using Evaluant.Uss.SqlMapper.SqlObjectModel;
using Evaluant.Uss.SqlMapper.SqlObjectModel.LDD;
using SQLObject;

namespace Evaluant.Uss.Sql
{
    /// <summary>
    /// Description rsume de SqlPersistenceEngine.
    /// </summary>
    public class SqlPersistenceEngine : PersistenceEngineImplementation
    {
        #region Members

        protected IDbTransaction _Transaction;
        protected IDriver _Driver;
        protected DBDialect _Dialect;
        protected IDbConnection _Connection;
        protected ArrayList _CacheEntries;
        protected SqlMappingDataType _MappingType;

        protected SqlCommandProcessor _CommandProcessor;

        #endregion

        #region Constantes

        internal static readonly string BooleanValueColumn = "ValueBool";
        internal static readonly string IntegerValueColumn = "ValueInt";
        internal static readonly string StringValueColumn = "ValueString";
        internal static readonly string BinaryValueColumn = "ValueBin";
        internal static readonly string DateTimeValueColumn = "ValueDateTime";
        internal static readonly string FloatValueColumn = "ValueFloat";

        #endregion

        #region Ctor

        public SqlPersistenceEngine()
        {
        }

        public SqlPersistenceEngine(string connectionString, IDriver driver, DBDialect dialect, SqlMappingDataType mappingType, ArrayList cacheEntries, Evaluant.Uss.Models.Model model)
        {
            if (connectionString == null)
            {
                throw new ArgumentNullException("connectionString");
            }
            if (driver == null)
            {
                throw new ArgumentNullException("driver");
            }
            if (dialect == null)
            {
                throw new ArgumentNullException("dialect");
            }
            if (mappingType == null)
            {
                throw new ArgumentNullException("mappingType");
            }

            Driver = driver;
            Dialect = dialect;
            Connection = Driver.CreateConnection(connectionString);
            Model = model;
            _MappingType = mappingType;

            CacheEntries = cacheEntries;
            this.Initialize();
        }

        #endregion

        #region Properties

        public IDriver Driver
        {
            get { return _Driver; }
            set
            {
                _Driver = value;
                if (_Driver != null)
                    _Driver.Dialect = _Dialect;
            }
        }


        public DBDialect Dialect
        {
            get { return _Dialect; }
            set
            {
                _Dialect = value;
                if (_Driver != null)
                    _Driver.Dialect = value;
            }
        }


        public IDbConnection Connection
        {
            get { return _Connection; }
            set { _Connection = value; }
        }


        public ArrayList CacheEntries
        {
            get { return _CacheEntries; }
            set { _CacheEntries = value; }
        }


        public SqlMappingDataType MappingType
        {
            get { return _MappingType; }
            set { _MappingType = value; }
        }

        #endregion

        #region LoadWithId

        public override EntitySet LoadWithId(string type, string[] id, string[] attributes)
        {
            EntitySet resultset = new EntitySet();

            _Connection.Open();
            _Transaction = _Connection.BeginTransaction();

            try
            {
                for (int i = 0; i < id.Length; i++)
                    id[i] = Dialect.FormatValue(id[i], DbType.AnsiString);

                string idsList = String.Join(", ", id);
                string query = String.Empty;

                if (attributes == null)
                {
                    //    "SELECT [Entity].Type, [Entity].Id, NULL, NULL FROM [Entity] ",
                    //    "WHERE [Entity].[Id] in ('", idsList, "') ORDER BY [Entity].[Id]");

                    query = String.Format("SELECT {0}.{1}, {0}.{2}, NULL, NULL, NULL, NULL, NULL, NULL, NULL FROM {0} WHERE {0}.{2} in ( {3} ) ORDER BY {0}.{2}",
                                                    Dialect.FormatAttribute(EntityTableName),
                                                    Dialect.FormatAttribute("Type"),
                                                    Dialect.FormatAttribute("Id"),
                                                    idsList);
                }
                else
                {
                    //    "SELECT [Entity].Type, [Entity].Id, [Attribute].Name, [Attribute].Value FROM [Entity] INNER JOIN [Attribute] ON [Entity].[Id] = [Attribute].[FK_Entity] ",
                    //    "WHERE [Entity].[Id] in ('", idsList, "') ", criteria,
                    //    "ORDER BY [Entity].[Id]");

                    string criteria = string.Empty;
                    if (attributes.Length > 0)
                    {
                        for (int i = 0; i < attributes.Length; i++)
                            attributes[i] = Dialect.FormatValue(attributes[i], DbType.AnsiString);

                        criteria = String.Format(" AND {0}.{1} IN ( {2} )",
                                                    Dialect.FormatAttribute(AttributeTableName),
                                                    Dialect.FormatAttribute("Name"),
                                                    String.Join(", ", attributes));
                    }

                    JoinedTable join = new JoinedTable(new TableSource(null, EntityTableName), TypeJoinedTable.LeftOuter, new TableSource(null, AttributeTableName));
                    join.SearchConditions.Add(new BinaryLogicExpression(new Column(null, EntityTableName, "Id"), BinaryLogicOperator.Equals, new Column(null, AttributeTableName, "FK_Entity")));
                    FromClause from = new FromClause();
                    from.Add(join);

                    query = String.Format("SELECT {0}.{1}, {0}.{2}, {3}.{4}, {3}.{5}, {3}.{6}, {3}.{7}, {3}.{8}, {3}.{9}, {3}.{10}" +
                                                " {11} WHERE {0}.{2} in ( {12} ) {13} ORDER BY {0}.{2}",
                                                Dialect.FormatAttribute(EntityTableName),
                                                Dialect.FormatAttribute("Type"),
                                                Dialect.FormatAttribute("Id"),
                                                Dialect.FormatAttribute(AttributeTableName),
                                                Dialect.FormatAttribute("Name"),
                                                Dialect.FormatAttribute(BooleanValueColumn),
                                                Dialect.FormatAttribute(IntegerValueColumn),
                                                Dialect.FormatAttribute(StringValueColumn),
                                                Dialect.FormatAttribute(FloatValueColumn),
                                                Dialect.FormatAttribute(DateTimeValueColumn),
                                                Dialect.FormatAttribute(BinaryValueColumn),
                                                Dialect.RenderQueries(from)[0],
                                                idsList,
                                                criteria);
                }
                resultset = LoadSql(query);
                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                _Connection.Close();
            }

            return resultset;
        }

        #endregion

        #region Load


        /// <summary>
        /// Load an EntitySet containing Entity corresponding to the specified opath, allow sort and paging
        /// </summary>
        /// <param name="opath">opath query</param>
        /// <param name="attributes">entity attributes to load (null: no one, new string[0]: all)</param>
        /// <param name="orderby">attributes used for sorting (Attr1 [asc|desc][, ...])</param>
        /// <param name="first">the first Entity index to load (index base 1)</param>
        /// <param name="max">the number of Entity to load (0: unlimited)</param>
        /// <returns></returns>
        public override EntitySet Load(OPathQuery query, string[] attributes, string orderby, int first, int max)
        {
            if (first <= 0)
                throw new ArgumentException("first must be greater than 0");

            if (max < 0)
                throw new ArgumentException("max must be none negative");

            SqlTransformer transformer = new SqlTransformer(_Model, _MappingType, this);
            ISQLExpression exp = transformer.Transform(query);

            #region Sort / Page

            string[] sortFields;
            if (orderby != null && orderby != String.Empty)
            {
                sortFields = orderby.Split(',');
            }
            else
            {
                sortFields = new string[0];
            }

            string entityType = GetFinalType(query).Type;

            OrderByClauseColumn[] orderbyClauses = new OrderByClauseColumn[sortFields.Length];

            // create a query to pivot the table : each attribute name to sort becomes a column

            SelectStatement select = new SelectStatement(null);
            OrderByClause orderByClause = new OrderByClause(select);
            ((SelectStatement)exp).TableAlias = "eS";
            Table from = exp as Table;

            select.SelectList.Add(new SelectItem(null, "eS", "Type", String.Empty));
            select.SelectList.Add(new SelectItem(null, "eS", "Id", String.Empty));

            for (int i = 0; i < sortFields.Length; i++)
            {
                string[] values = sortFields[i].Trim().Split();

                if (values.Length > 2)
                    throw new ArgumentException("orderby");

                sortFields[i] = values[0].Trim();

                if (sortFields[i] != "Id")
                {
                    CaseExpression caseExp = new CaseExpression(null, new Column(null, String.Concat("a", i, sortFields[i]), "Name"), sortFields[i]);

                    string colName = SqlCommandProcessor.GetAttributeColumnNameFromDbType(SqlCommandProcessor.GetDbTypeFromType(_Model.GetAttribute(entityType, sortFields[i], true).Type, MappingType));
                    caseExp.TestExpressions.Add(new CaseTest(new Constant(sortFields[i], DbType.AnsiString), new Column(null, String.Concat("a", i, sortFields[i]), colName)));

                    select.SelectList.Add(caseExp);

                    from = new JoinedTable(from, TypeJoinedTable.LeftOuter, new TableSource(null, AttributeTableName, String.Concat("a", i, sortFields[i])));
                    ((JoinedTable)from).SearchConditions.Add(new BinaryLogicExpression(new Column(null, "eS", "Id"), BinaryLogicOperator.Equals, new Column(null, String.Concat("a", i, sortFields[i]), "FK_Entity")));
                    select.WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(null, String.Concat("a", i, sortFields[i]), "Name"), BinaryLogicOperator.Equals, new Constant(sortFields[i], DbType.AnsiString)));
                }

                bool isDesc = (values.Length == 1) ? false : (values[1].Trim().ToLower() == "desc");
                orderByClause.Add(orderbyClauses[i] = new OrderByClauseColumn(sortFields[i], isDesc));
            }

            select.FromClause.Add(from);

            if (first != 1 || max != 0)
            {
                OPath.Expressions.Function countFunction = new OPath.Expressions.Function(query.Path, Evaluant.OPath.Expressions.FunctionEnum.Count);
                OPath.Expressions.Call callCount = new OPath.Expressions.Call("eval", new Evaluant.OPath.Expressions.Collections.ConstraintCollection(new OPath.Expressions.Constraint[] { countFunction }));
                query.Expression = callCount;
                int maxEntity = Convert.ToInt32(LoadScalar(query));
                if (maxEntity < first)
                    return new EntitySet();		// paging out of range

                if (max == 0)
                    max = maxEntity;

                if (first + max > maxEntity)
                    max = maxEntity - first + 1;
            }

            exp = Dialect.Page(select, orderByClause, first, max == 0 ? -1 : max);

            #endregion

            // Modify the query to grab attributes if provided

            SelectStatement select0 = new SelectStatement(null);
            ((SelectStatement)exp).TableAlias = "e0";

            if (attributes == null)
            {
                //sqlQuery = String.Concat("SELECT e1.Type, e1.Id, NULL, NULL FROM (", sqlQuery, ") e1 ");

                select0.SelectList.Add(new Column(null, "e0", "Type", String.Empty));
                select0.SelectList.Add(new Column(null, "e0", "Id", String.Empty));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, "Name"));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.BooleanValueColumn));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.IntegerValueColumn));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.StringValueColumn));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.FloatValueColumn));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.DateTimeValueColumn));
                select0.SelectList.Add(new Constant(DBNull.Value, DbType.Object, SqlPersistenceEngine.BinaryValueColumn));

                select0.FromClause.Add(exp as Table);

            }
            else
            {
                select0.SelectList.Add(new Column(null, "e0", "Type", String.Empty));
                select0.SelectList.Add(new Column(null, "e0", "Id", String.Empty));
                select0.SelectList.Add(new Column(null, "a0", "Name", String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.BooleanValueColumn, String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.IntegerValueColumn, String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.StringValueColumn, String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.FloatValueColumn, String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.DateTimeValueColumn, String.Empty));
                select0.SelectList.Add(new Column(null, "a0", SqlPersistenceEngine.BinaryValueColumn, String.Empty));

                JoinedTable joinAttr = new JoinedTable(exp as Table, TypeJoinedTable.LeftOuter, new TableSource(null, AttributeTableName, "a0"));
                joinAttr.SearchConditions.Add(new BinaryLogicExpression(new Column(null, "e0", "Id"), BinaryLogicOperator.Equals, new Column(null, "a0", "FK_Entity")));
                select0.FromClause.Add(joinAttr);

                if (attributes.Length > 0)
                {
                    ExpressionCollection attrColl = new ExpressionCollection();
                    foreach (string att in attributes)
                        attrColl.Add(new Constant(att, DbType.AnsiString));

                    InPredicate inName = new InPredicate(new Column(null, "a0", "Name"), attrColl);
                    select0.WhereClause.SearchCondition.Add(inName);
                }

                foreach (OrderByClauseColumn oc in orderbyClauses)
                {
                    select0.OrderByClause.Add(new OrderByClauseColumn("e0", oc.ColumnName, oc.Desc));
                }
            }

            EntitySet result = null;

            _Connection.Open();
            _Transaction = _Connection.BeginTransaction();

            try
            {
                string[] queries = _Dialect.RenderQueries(select0);
                foreach (string q in queries)
                {
                    result = LoadSql(q);
                }

                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                _Connection.Close();
            }

            return result;
        }

        private EntitySet LoadSql(string sqlQuery)
        {
            // select type, id, attribute_name, 
            //        attribute_value_bool, attribute_value_int, attribute_value_string, attribute_value_float, attribute_value_datetime, attribute_value_binary 

            Hashtable entities = new Hashtable();
            EntitySet resultset = new EntitySet();

            IDbCommand cmd = _Driver.CreateCommand(sqlQuery, _Connection, _Transaction);
            using (IDataReader reader = cmd.ExecuteReader())
            {

                Entity entity = null;
                while (reader.Read())
                {
                    string newId = reader.GetString(1).Trim();

                    if (!entities.Contains(newId))
                    {
                        entity = new Entity(reader.GetString(0));
                        entity.Id = newId;
                        entity.State = State.UpToDate;
                        resultset.Add(entity);
                        entities.Add(newId, entity);
                    }
                    else
                        entity = entities[newId] as Entity;

                    if (!reader.IsDBNull(4))   // int
                        ImportAttribute(entity, reader.GetString(2), reader.GetValue(4));
                    else if (!reader.IsDBNull(5))   // string
                        ImportAttribute(entity, reader.GetString(2), reader.GetValue(5));
                    else if (!reader.IsDBNull(6))   // float
                        ImportAttribute(entity, reader.GetString(2), reader.GetValue(6));
                    else if (!reader.IsDBNull(7))   // datetime
                        ImportAttribute(entity, reader.GetString(2), reader.GetValue(7));
                    else if (!reader.IsDBNull(8))   // binary
                        ImportAttribute(entity, reader.GetString(2), reader.GetValue(8));
                    else if (!reader.IsDBNull(3))   // bool
                    {
                        if (!reader.IsDBNull(2))
                            ImportAttribute(entity, reader.GetString(2), reader.GetValue(3));   // at the end (with Access never null)
                    }

                    entity.State = State.UpToDate;
                }

                if (entity != null)
                    entity.State = State.UpToDate;

                return resultset;
            }
        }

        #endregion

        #region LoadScalar

        public override object LoadScalar(OPathQuery query)
        {
            if (query.QueryType == OPathQueryTypeEnum.Path)
            {
                query.QueryType = OPathQueryTypeEnum.Expression;
                query.Expression = new OPath.Expressions.Call("eval", new Evaluant.OPath.Expressions.Collections.ConstraintCollection(new OPath.Expressions.Constraint[] { query.Path }));
            }

            SqlTransformer transformer = new SqlTransformer(_Model, _MappingType, this);

            ISQLExpression exp = transformer.TransformScalar(query);

            _Connection.Open();
            _Transaction = _Connection.BeginTransaction();

            object result = null;
            try
            {
                string[] queries = _Dialect.RenderQueries(exp);
                IDbCommand command = Driver.CreateCommand(queries[queries.Length - 1], _Connection, _Transaction);

                object res = command.ExecuteScalar();
                if (res != DBNull.Value)
                    result = res;

                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                _Connection.Close();
            }

            // Ensure the result in an Int32 in case of a count()
            if (query.Expression.Operands[0] is OPath.Expressions.Function
                && ((OPath.Expressions.Function)query.Expression.Operands[0]).Type == Evaluant.OPath.Expressions.FunctionEnum.Count)
                result = Convert.ToInt32(result);

            return result;
        }

        #endregion

        #region LoadReference

        public override void LoadReference(IEnumerable entities, string[] references)
        {

            Hashtable entityIndex = new Hashtable();
            foreach (Entity entity in entities)
            {
                entity.RemoveReference(references);
                entityIndex.Add(entity.Id, entity);
            }

            if (entityIndex.Count == 0)
                return;

            string[] idsList = new string[entityIndex.Count];
            int i = 0;
            foreach (string id in entityIndex.Keys)
                idsList[i++] = id;

            //string sqlQuery = String.Concat(
            //    "SELECT ",
            //    "[Reference].[FK_Parent], ",	    // 0
            //    "[Reference].[Role], ",			// 1
            //    "[Entity].[Type], ",			    // 2
            //    "[Entity].[Id], ",				// 3
            //    "[Attribute].[Name], ",			// 4
            //    "[Attribute].[Value] ",			// 5
            //    "FROM [Entity] ",
            //    "LEFT JOIN [Reference] ON [Entity].[Id] = [Reference].[FK_Child] ",
            //    "LEFT JOIN [Attribute] ON [Entity].[Id] = [Attribute].[FK_Entity] ", // left join to select also referenced entities containing no attribute
            //    "WHERE ",
            //    "FK_Parent in ('", String.Join("', '", idsList), "') ",
            //    references.Length == 0 ? String.Empty : "AND [Reference].[Role] in ('", String.Join("', '", references), "') ",
            //    //				"AND [Attribute].[Name] in ('", String.Join("', '", attributes), "') ",
            //    "ORDER BY [Reference].[FK_Parent], [Entity].[Id]");

            SelectStatement select = new SelectStatement(null);
            select.SelectList.Add(new SelectItem(null, ReferenceTableName, "FK_Parent", String.Empty));              // 0
            select.SelectList.Add(new SelectItem(null, ReferenceTableName, "Role", String.Empty));                   // 1
            select.SelectList.Add(new SelectItem(null, EntityTableName, "Type", String.Empty));                      // 2
            select.SelectList.Add(new SelectItem(null, EntityTableName, "Id", String.Empty));                        // 3
            select.SelectList.Add(new SelectItem(null, AttributeTableName, "Name", String.Empty));                   // 4
            select.SelectList.Add(new SelectItem(null, AttributeTableName, BooleanValueColumn, String.Empty));       // 5
            select.SelectList.Add(new SelectItem(null, AttributeTableName, IntegerValueColumn, String.Empty));       // 6
            select.SelectList.Add(new SelectItem(null, AttributeTableName, StringValueColumn, String.Empty));        // 7
            select.SelectList.Add(new SelectItem(null, AttributeTableName, FloatValueColumn, String.Empty));         // 8    
            select.SelectList.Add(new SelectItem(null, AttributeTableName, DateTimeValueColumn, String.Empty));      // 9
            select.SelectList.Add(new SelectItem(null, AttributeTableName, BinaryValueColumn, String.Empty));        // 10

            JoinedTable joinRef = new JoinedTable(new TableSource(null, EntityTableName), TypeJoinedTable.Inner, new TableSource(null, ReferenceTableName));
            joinRef.SearchConditions.Add(new BinaryLogicExpression(new Column(null, EntityTableName, "Id"), BinaryLogicOperator.Equals, new Column(null, ReferenceTableName, "FK_Child")));

            JoinedTable joinAttr = new JoinedTable(joinRef, TypeJoinedTable.LeftOuter, new TableSource(null, AttributeTableName));
            joinAttr.SearchConditions.Add(new BinaryLogicExpression(new Column(null, EntityTableName, "Id"), BinaryLogicOperator.Equals, new Column(null, AttributeTableName, "FK_Entity")));

            select.FromClause.Add(joinAttr);

            ExpressionCollection parentList = new ExpressionCollection();
            foreach (string id in idsList)
                parentList.Add(new Constant(id, DbType.AnsiString));
            InPredicate parents = new InPredicate(new Column(null, ReferenceTableName, "FK_Parent"), parentList);
            select.WhereClause.SearchCondition.Add(parents);

            if (references.Length != 0)
            {
                ExpressionCollection refList = new ExpressionCollection();
                foreach (string r in references)
                    refList.Add(new Constant(r, DbType.AnsiString));

                InPredicate refs = new InPredicate(new Column(null, ReferenceTableName, "Role"), refList);
                select.WhereClause.SearchCondition.Add(refs);
            }

            select.OrderByClause.Add(new OrderByClauseColumn(ReferenceTableName, "FK_Parent", false));
            select.OrderByClause.Add(new OrderByClauseColumn(EntityTableName, "Id", false));

            _Connection.Open();
            _Transaction = _Connection.BeginTransaction();

            try
            {
                foreach (string sql in Dialect.RenderQueries(select, _Driver))
                {
                    IDbCommand command = Driver.CreateCommand(sql, _Connection, _Transaction);
                    using (IDataReader reader = command.ExecuteReader())
                    {

                        string parentId = String.Empty;
                        string id = String.Empty;
                        string referenceValue = String.Empty;
                        Entity parentEntity = null, reference = null;

                        while (reader.Read())
                        {
                            string newParentId = reader.GetString(0).Trim();
                            string newReferenceId = reader.GetString(3).Trim();

                            string newReferenceValue = reader.GetString(1).Trim();

                            // Go the next parent
                            if (parentId != newParentId)
                            {
                                if (parentEntity != null)
                                    parentEntity.State = State.UpToDate;

                                parentId = reader.GetString(0).Trim();
                                parentEntity = entityIndex[parentId] as Entity;
                                id = String.Empty;
                                referenceValue = String.Empty;
                            }

                            // Create a new reference
                            if (id != newReferenceId ||
                                referenceValue != newReferenceValue)
                            {
                                if (reference != null)
                                    reference.State = State.UpToDate;

                                reference = new Entity(reader.GetString(2));
                                id = newReferenceId;
                                referenceValue = reader.GetString(1);
                                reference.Id = id;
                                parentEntity.AddValue(reader.GetString(1), reference, State.UpToDate);
                            }

                            // Add the attribute
                            if (!reader.IsDBNull(6))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(6));
                            else if (!reader.IsDBNull(7))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(7));
                            else if (!reader.IsDBNull(8))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(8));
                            else if (!reader.IsDBNull(9))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(9));
                            else if (!reader.IsDBNull(10))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(10));
                            else if (!reader.IsDBNull(5))
                                ImportAttribute(reference, reader.GetString(4), reader.GetValue(5));
                        }

                        if (parentEntity != null)
                            parentEntity.State = State.UpToDate;

                        if (reference != null)
                            reference.State = State.UpToDate;

                        // mark the reference as loaded for each parentEntity
                        foreach (Entity e in entities)
                            foreach (string refName in references)
                                if (!e.InferredReferences.Contains(refName))
                                    e.InferredReferences.Add(refName);
                    }
                }

                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                _Connection.Close();
            }
        }

        #endregion

        #region Initialize

        public override void Initialize()
        {
            base.Initialize();
        }

        public override void InitializeRepository()
        {
            Hashtable commands = GetCommands();

            if (Connection.State != ConnectionState.Open)
                Connection.Open();

            _Transaction = _Connection.BeginTransaction();

            try
            {
                // First remove all constraints
                foreach (ISQLExpression query in commands.Values)
                {
                    if (query is AlterTableSQLCommand && ((AlterTableSQLCommand)query).ForeignKeys.Count > 0)
                    {
                        IDbCommand cmd = null;

                        try
                        {
                            AlterTableSQLCommand delete_constr = ((AlterTableSQLCommand)query).Clone() as AlterTableSQLCommand;
                            delete_constr.AlterType = AlterTypeEnum.Drop;
                            delete_constr.ColumnDefinitions.Clear();
                            delete_constr.PrimaryKey = null;

                            string[] queries = Dialect.RenderQueries(delete_constr, Driver);
                            foreach (string q in queries)
                            {
                                cmd = Driver.CreateCommand(q, Connection, _Transaction);
                                cmd.ExecuteNonQuery();
                            }
                        }
                        catch { }
                    }
                }

                // Create tables
                foreach (ISQLExpression query in commands.Values)
                {
                    if (query is CreateTableSQLCommand)
                    {
                        IDbCommand cmd = null;

                        // Remove table
                        DropTableSQLCommand delete_query = new DropTableSQLCommand(null, ((CreateTableSQLCommand)query).TableName);
                        foreach (string sql in Dialect.RenderQueries(delete_query, _Driver))
                        {
                            try
                            {
                                cmd = Driver.CreateCommand(sql, Connection, _Transaction);
                                cmd.ExecuteNonQuery();

                            }
                            catch
                            { }
                        }

                        // Create table
                        foreach (string sql in Dialect.RenderQueries(query, _Driver))
                        {
                            try
                            {
                                cmd = Driver.CreateCommand(sql, Connection, _Transaction);
                                cmd.ExecuteNonQuery();
                            }
                            catch
                            { }
                        }
                    }
                }

                // Add PrimaryKey constraints
                foreach (ISQLExpression query in commands.Values)
                {
                    if (query is AlterTableSQLCommand && ((AlterTableSQLCommand)query).PrimaryKey != null)
                    {
                        IDbCommand cmd = null;

                        AlterTableSQLCommand pk_query = ((AlterTableSQLCommand)query).Clone() as AlterTableSQLCommand;
                        pk_query.ColumnDefinitions.Clear();
                        pk_query.ForeignKeys.Clear();

                        foreach (string sql in Dialect.RenderQueries(pk_query, _Driver))
                        {
                            try
                            {
                                cmd = Driver.CreateCommand(sql, Connection, _Transaction);
                                cmd.ExecuteNonQuery();
                            }
                            catch { }
                        }
                    }
                }

                // Add ForeignKey constraints
                foreach (ISQLExpression query in commands.Values)
                {
                    if (query is AlterTableSQLCommand && ((AlterTableSQLCommand)query).ForeignKeys.Count > 0)
                    {
                        IDbCommand cmd = null;

                        AlterTableSQLCommand fk_query = ((AlterTableSQLCommand)query).Clone() as AlterTableSQLCommand;
                        fk_query.PrimaryKey = null;
                        fk_query.ColumnDefinitions.Clear();

                        foreach (string sql in Dialect.RenderQueries(fk_query, _Driver))
                        {
                            try
                            {
                                cmd = Driver.CreateCommand(sql, Connection, _Transaction);
                                cmd.ExecuteNonQuery();
                            }
                            catch { }
                        }
                    }
                }

                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                Connection.Close();
            }

        }

        public virtual Hashtable GetCommands()
        {
            Hashtable commands = new Hashtable();

            CreateTableSQLCommand table;
            AlterTableSQLCommand alter;
            string type = String.Empty;


            //CREATE TABLE [Entity]
            //([Id] char(36)  PRIMARY KEY NOT NULL,
            //[Type] varchar(255) );

            table = new CreateTableSQLCommand(null, EntityTableName);
            table.ColumnDefinitions.Add("Id", new ColumnDefinition("Id", System.Data.DbType.AnsiStringFixedLength, 36, 0, 0, new ColumnConstraint(false)));
            table.ColumnDefinitions.Add("Type", new ColumnDefinition("Type", System.Data.DbType.AnsiString, 255, 0, 0));
            commands.Add(table.TableName, table);

            //CREATE TABLE [Attribute]
            //([Name] varchar(255),
            //[ValueString] varchar(1024),
            //[ValueInt] int,
            //[ValueBool] bit,
            //[ValueFloat] float,
            //[ValueDateTime] datetime,
            //[ValueBin] image,
            //[FK_Entity] char(36) );

            table = new CreateTableSQLCommand(null, AttributeTableName);
            table.ColumnDefinitions.Add("Name", new ColumnDefinition("Name", System.Data.DbType.AnsiString, 255, 0, 0));
            table.ColumnDefinitions.Add("ValueString", new ColumnDefinition(SqlPersistenceEngine.StringValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.String.Name, true), MappingType.String.Size, 0, 0));
            table.ColumnDefinitions.Add("ValueInt", new ColumnDefinition(SqlPersistenceEngine.IntegerValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.Int.Name, true), 0, 0, 0));
            table.ColumnDefinitions.Add("ValueBool", new ColumnDefinition(SqlPersistenceEngine.BooleanValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.Bool.Name, true), 0, 0, 0));
            table.ColumnDefinitions.Add("ValueFloat", new ColumnDefinition(SqlPersistenceEngine.FloatValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.Float.Name, true), 0, 0, 0));
            table.ColumnDefinitions.Add("ValueDateTime", new ColumnDefinition(SqlPersistenceEngine.DateTimeValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.DateTime.Name, true), 0, 0, 0));
            table.ColumnDefinitions.Add("ValueBin", new ColumnDefinition(SqlPersistenceEngine.BinaryValueColumn, (DbType)Enum.Parse(typeof(DbType), MappingType.Binary.Name, true), 0, 0, 0));
            table.ColumnDefinitions.Add("FK_Entity", new ColumnDefinition("FK_Entity", System.Data.DbType.AnsiStringFixedLength, 36, 0, 0));
            commands.Add(table.TableName, table);

            //CREATE TABLE [Reference]
            //([Role] varchar(255),
            //[FK_Parent] char(36),
            //[FK_Child] char(36) );

            table = new CreateTableSQLCommand(null, ReferenceTableName);
            table.ColumnDefinitions.Add("Role", new ColumnDefinition("Role", System.Data.DbType.AnsiString, 255, 0, 0));
            table.ColumnDefinitions.Add("FK_Parent", new ColumnDefinition("FK_Parent", System.Data.DbType.AnsiStringFixedLength, 36, 0, 0));
            table.ColumnDefinitions.Add("FK_Child", new ColumnDefinition("FK_Child", System.Data.DbType.AnsiStringFixedLength, 36, 0, 0));
            commands.Add(table.TableName, table);


            //ALTER TABLE [dbo].[Entity] WITH NOCHECK ADD 
            //                     PRIMARY KEY  CLUSTERED 
            //                    (
            //                        [Id]
            //                    )  ON [PRIMARY]

            alter = new AlterTableSQLCommand(null, EntityTableName, AlterTypeEnum.Add);
            alter.PrimaryKey = new PrimaryKey(GetPKEntityColumnName(), new ColumnDefinition[] { new ColumnDefinition("Id", System.Data.DbType.AnsiStringFixedLength, 36, 0, 0) });
            commands.Add(alter.TableName + GetPKEntityColumnName(), alter);

            //ALTER TABLE [dbo].[Attribute] ADD CONSTRAINT Id_FK_Entity

            alter = new AlterTableSQLCommand(null, AttributeTableName, AlterTypeEnum.Add);
            alter.ForeignKeys.Add(new ForeignKey(GetFKEntityColumnName(), new string[] { "FK_Entity" }, EntityTableName, new string[] { "Id" }));
            commands.Add(alter.TableName + GetFKEntityColumnName(), alter);

            //ALTER TABLE [dbo].[Reference] ADD 
            //                CONSTRAINT [Id_FK_Child] FOREIGN KEY 
            //                (
            //                    [FK_Child]
            //                ) REFERENCES [dbo].[Entity] (
            //                    [Id]
            //                ),
            //                CONSTRAINT [Id_FK_Parent] FOREIGN KEY 
            //                (
            //                    [FK_Parent]
            //                ) REFERENCES [dbo].[Entity] (
            //                    [Id]
            //                )

            alter = new AlterTableSQLCommand(null, ReferenceTableName, AlterTypeEnum.Add);
            alter.ForeignKeys.Add(new ForeignKey(GetFKParentColumnName(), new string[] { "FK_Parent" }, EntityTableName, new string[] { "Id" }));
            alter.ForeignKeys.Add(new ForeignKey(GetFKChildColumnName(), new string[] { "FK_Child" }, EntityTableName, new string[] { "Id" }));
            commands.Add(alter.TableName + "Id_FK_Parent", alter);

            return commands;
        }

        private string GetFKEntityColumnName()
        {
            return string.Format("Id_FK_{0}", EntityTableName);
        }

        private string GetPKEntityColumnName()
        {
            return string.Format("PK_{0}", EntityTableName);
        }

        private string GetFKParentColumnName()
        {
            return string.Format("Id_FK_{0}_Parent", EntityTableName);
        }

        private string GetFKChildColumnName()
        {
            return string.Format("Id_FK_{0}_Child", EntityTableName);
        }

        #endregion

        #region ProcessCommands

        public override void ProcessCommands(Transaction tx)
        {
            _CommandProcessor = new SqlCommandProcessor(this);
            _Connection.Open();
            _Transaction = _Connection.BeginTransaction();
            _CommandProcessor.Transaction = _Transaction;

            try
            {
                foreach (Command cmd in tx.PendingCommands)
                {
                    cmd.Accept(_CommandProcessor);
                    tx.NewIds = new Hashtable();
                    foreach (string oldid in ((SqlCommandProcessor)_CommandProcessor).NewIds.Keys)
                        tx.NewIds.Add(oldid, ((SqlCommandProcessor)_CommandProcessor).NewIds[oldid]);
                }

                _Transaction.Commit();
            }
            catch
            {
                _Transaction.Rollback();
                throw;
            }
            finally
            {
                _Connection.Close();
            }
        }

        #endregion

        #region ImportAttribute

        private void ImportAttribute(Entity entity, string name, object value)
        {

            Type sType = _Model.GetAttribute(entity.Type, name, true).Type;

            object sValue = null;

            if (value != null)
            {
                if (sType == typeof(TimeSpan))
                {
                    sValue = new TimeSpan((long)value);
                }
                else if (sType != null && (sType.IsValueType || sType == typeof(String))) // type can be null if it is not a ValueType
                {
#if !EUSS11
                    if (sType.IsGenericType && sType.GetGenericTypeDefinition().Equals(typeof(Nullable<>)))
                    {
                        System.ComponentModel.NullableConverter nc = new System.ComponentModel.NullableConverter(sType);
                        sValue = nc.ConvertFrom(value);
                    }
                    else
                    {
                        sValue = Convert.ChangeType(value, sType);
                    }
#else
                    sValue = Convert.ChangeType(value, sType);
#endif

                }
                else if (sType == typeof(byte[]))
                {
                    sValue = (byte[])value;
                }
                else
                {
                    sValue = Utils.UnSerialize((byte[])value);
                    sType = value.GetType();
                }
            }

            entity.AddValue(name, sValue, sType, State.UpToDate);
        }

        #endregion

        #region Functions

        /// <summary>
        /// Return the final type of the opath
        /// </summary>
        /// <param name="query">opath to analyse</param>
        /// <returns>type of the object to return by the opath</returns>
        private Evaluant.Uss.Models.Entity GetFinalType(OPathQuery query)
        {
            Evaluant.Uss.Models.Entity parent = null;

            if (query.Path.Identifiers.Count == 1)
                parent = _Model.GetEntity(query.Path.Identifiers[0].Value);
            else
            {
                string refName = query.Path.Identifiers[query.Path.Identifiers.Count - 1].Value;

                string firstType = query.Path.Identifiers[0].Value;
                string lastType = firstType;
                string nextRefName = string.Empty;

                for (int i = 0; i < query.Path.Identifiers.Count - 1; i++)
                {
                    nextRefName = query.Path.Identifiers[i + 1].Value;

                    //	The next identifier isn't a reference but an attribute
                    if (_Model.GetReference(lastType, nextRefName) == null &&
                        _Model.GetAttribute(lastType, nextRefName) != null)
                        continue;

                    lastType = _Model.GetReference(lastType, nextRefName).ChildType;
                }

                parent = _Model.GetEntity(lastType);
            }
            return parent;
        }

        private string GetSqlAttributeType(string entityType, string attributeName)
        {
            switch (_Model.GetAttribute(entityType, attributeName).Type.Name)
            {
                case "Boolean":
                    return "bit";

                case "Byte[]":
                case "Char":
                case "String":
                    return "varchar(1024)";

                case "SByte":
                case "Byte":
                case "Int16":
                case "Unt16":
                case "Int32":
                case "Unt32":
                case "Int64":
                case "UInt64":
                    return "int";

                case "DateTime":
                case "Double":
                case "Single":
                case "Decimal":
                    return "float";

                default:
                    throw new PersistenceEngineException("Cannot sort an attribute of type '" + _Model.GetAttribute(entityType, attributeName).Type.FullName + "'");
            }
        }

        #endregion

        private string entityTableName;
        public string EntityTableName
        {
            get { return entityTableName; }
            set { entityTableName = value; }
        }

        private string attributeTableName;
        public string AttributeTableName
        {
            get { return attributeTableName; }
            set { attributeTableName = value; }
        }

        private string referenceTableName;
        public string ReferenceTableName
        {
            get { return referenceTableName; }
            set { referenceTableName = value; }
        }
    }

}
