using System;
using System.Reflection;

using Evaluant.Uss.Common;
using Evaluant.Uss.SqlMapper;

namespace Evaluant.Uss.Sql
{
	/// <summary>
	/// Description rsume de SqlProvider.
	/// </summary>
    public class SqlProvider : PersistenceEngineFactoryImplementation
    {
        #region Members

        protected string _ConnectionString;
        protected SqlMappingDataType _MappingType;

        protected string _MappingIntType = String.Empty;
        protected string _MappingBoolType = String.Empty;
        protected string _MappingFloatType = String.Empty;
        protected string _MappingDateTimeType = String.Empty;
        protected string _MappingBinaryType = String.Empty;
        protected string _MappingStringType = String.Empty;
        protected string _MappingStringSize = String.Empty;

        protected string _TablePrefix = String.Empty;
        protected const string ENTITY_TABLENAME = "Entity";
        protected const string ATTRIBUTE_TABLENAME = "Attribute";
        protected const string REFERENCE_TABLENAME = "Reference";

		private static object _synLock = new object();

        #endregion

        #region Ctor
        
        public SqlProvider()
		{
            _MappingType = new SqlMappingDataType();
		}

        #endregion

        #region Initialize

        public override void InitializeConfiguration()
        {
			lock(_synLock)
			{
				if(_MappingIntType != string.Empty)
					_MappingType.Int.Name = _MappingIntType;
            
				if(_MappingBoolType != string.Empty)
					_MappingType.Bool.Name = _MappingBoolType;

				if(_MappingFloatType != string.Empty)
					_MappingType.Float.Name = _MappingFloatType;

				if(_MappingDateTimeType != string.Empty)
					_MappingType.DateTime.Name = _MappingDateTimeType;
            
				if(_MappingBinaryType != string.Empty)
					_MappingType.Binary.Name = _MappingBinaryType;

				if(_MappingStringType != string.Empty)
					_MappingType.String.Name = _MappingStringType;

				if (_MappingStringSize != string.Empty)
					_MappingType.String.Size = _MappingStringSize == "*" ? 0 : int.Parse(_MappingStringSize);

			}
        }

        public override IPersistenceEngine CreatePersistenceEngine()
        {
			lock(_synLock)
			{
				SqlPersistenceEngine engine = new SqlPersistenceEngine(_ConnectionString, CreateDriver(_driver), CreateDialect(_dialect), _MappingType, null, _Model);
				engine.Culture = _Culture;
				engine.Dialect.Culture = _Culture;
                engine.EntityTableName = EntityTableName;
                engine.AttributeTableName = AttributeTableName;
                engine.ReferenceTableName = ReferenceTableName;
				engine.Initialize();
				return engine;
			}
        }

        #endregion

        public string EntityTableName
        {
            get
            {
                if (!string.IsNullOrEmpty(TablePrefix))
                    return string.Concat(TablePrefix, ENTITY_TABLENAME);

                return ENTITY_TABLENAME;
            }
        }

        public string AttributeTableName
        {
            get
            {
                if (!string.IsNullOrEmpty(TablePrefix))
                    return string.Concat(TablePrefix, ATTRIBUTE_TABLENAME);

                return ATTRIBUTE_TABLENAME;
            }
        }

        public string ReferenceTableName
        {
            get
            {
                if (!string.IsNullOrEmpty(TablePrefix))
                    return string.Concat(TablePrefix, REFERENCE_TABLENAME);

                return REFERENCE_TABLENAME;
            }
        }

        #region Properties

        public string TablePrefix
        {
            get { return _TablePrefix; }
            set { _TablePrefix = value; }
        }

        public string ConnectionString
        {
            get { return _ConnectionString; }
            set { _ConnectionString = value; }
        }

        private string _dialect;
        public string Dialect
        {
            get { return _dialect; }
			set	{ _dialect = value; }
        }

		protected DBDialect CreateDialect(string dialect)
		{
			Type type = Utils.GetType(dialect);

			if (type == null && dialect.Split(',').Length >= 2)
			{
				string[] values = dialect.Split(',');
				type = Assembly.LoadWithPartialName(values[1]).GetType(values[0]);
			}

			if (type == null)
				throw new PersistenceEngineException("The specified type could not be found: " + dialect);

			ConstructorInfo pc = type.GetConstructor(Type.EmptyTypes);
			return (DBDialect)pc.Invoke(new object[] { });
		}
        
		private string _driver;
		public string Driver
		{
			get { return _driver; }
			set { _driver = value; }
		}

		protected IDriver CreateDriver(string driver)
		{
			Type type = Utils.GetType(driver);

			if (type == null && driver.Split(',').Length >= 2)
			{
				string[] values = driver.Split(',');
				type = Assembly.LoadWithPartialName(values[1]).GetType(values[0]);
			}

			if (type == null)
				throw new PersistenceEngineException("The specified type could not be found: " + driver);

			ConstructorInfo pc = type.GetConstructor(Type.EmptyTypes);
			return (IDriver)pc.Invoke(new object[] { });
		}

        public string MappingIntType
        {
            get { return _MappingIntType; }
            set { _MappingIntType = value; }
        }

        public string MappingBoolType
        {
            get { return _MappingBoolType; }
            set { _MappingBoolType = value; }
        }

        public string MappingFloatType
        {
            get { return _MappingFloatType; }
            set { _MappingFloatType = value; }
        }

        public string MappingDateTimeType
        {
            get { return _MappingDateTimeType; }
            set { _MappingDateTimeType = value; }
        }

        public string MappingBinaryType
        {
            get { return _MappingBinaryType; }
            set { _MappingBinaryType = value; }
        }

        public string MappingStringType
        {
            get { return _MappingStringType; }
            set { _MappingStringType = value; }
        }

        public string MappingStringSize
        {
            get { return _MappingStringSize; }
            set { _MappingStringSize = value; }
        }

        #endregion
    }
}
