using System;
using System.Collections;
using System.Data;
using System.Reflection;
using System.Xml.Serialization;
using SQLObject;
using System.IO;
using System.Text.RegularExpressions;

using Evaluant.Uss.Common;
using Evaluant.Uss.Commands;

namespace Evaluant.Uss.SqlMapper
{
    /// <summary>
    /// Description rsume de SqlMapperFactory.
    /// </summary>
    public class SqlMapperProvider : PersistenceEngineFactoryImplementation
    {
        protected static object _synLock = new object();
        public static char IDSEP = ';';

        protected ArrayList _CacheEntityEntries;
        public ArrayList CacheEntityEntries
        {
            get { return _CacheEntityEntries; }
            set { _CacheEntityEntries = value; }
        }

        private CacheEntityEntry GetCacheEntityEntry(string type)
        {
            foreach (CacheEntityEntry entry in _CacheEntityEntries)
            {
                if (((EntityMapping)entry.Mapping).Type == type)
                    return entry;
            }

            return null;
        }

        public SqlMapperProvider()
        {
        }

        #region Membres de IPersistenceEngineFactory

        private string _TablePrefix;
        public string TablePrefix
        {
            get { return _TablePrefix; }
            set { _TablePrefix = value; }
        }

        //private ArrayList _MappingFilenames = new ArrayList();
        //public string[] MappingFilenames
        //{
        //    get 
        //    { 
        //        return (string[]) _MappingFilenames.ToArray(typeof(string)); 
        //    }
        //}

        public void AddMappingFilename(string filename)
        {
            TextReader tr = new StreamReader(filename);
            AddMappingContent(tr.ReadToEnd());
        }

        private ArrayList _MappingContents = new ArrayList();
        public string[] MappingContents
        {
            get
            {
                return (string[])_MappingContents.ToArray(typeof(string));
            }
        }

        public void AddMappingContent(string content)
        {
            _MappingContents.Add(content);
        }

        protected void InitializeMapping(string mappingContent)
        {
            string fileStrResult = Regex.Replace(mappingContent, @"\sxmlns[^""]*\""[^""]*\""", String.Empty, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);

            StringReader reader = new StringReader(fileStrResult);

            XmlSerializer deserialize = new XmlSerializer(typeof(Mapping));
            Mapping tmpMapping = (Mapping)deserialize.Deserialize(reader);

            foreach (EntityMapping em in tmpMapping.Entities)
            {
                EntityMapping rem = _Mapping.Entities[em.Type, false];
                if (rem == null)
                    _Mapping.Entities.Add(em);
                else
                {
                    rem.Table = em.Table;
                    if (em.Ids.Count > 0)
                        rem.Ids = em.Ids;

                    foreach (AttributeMapping am in em.Attributes)
                    {
                        AttributeMapping ram = rem.Attributes[am.Name, false];
                        if (ram == null)
                            rem.Attributes.Add(am);
                        else
                        {
                            ram.Field = am.Field;
                            ram.DbType = am.DbType;
                            if (am.ParentEntity != null)
                                ram.Table = am.Table;
                        }
                    }

                    foreach (ReferenceMapping rm in em.References)
                    {
                        ReferenceMapping rRm = rem.References[rm.Name, false];
                        if (rRm == null)
                            rem.References.Add(rm);
                        else
                            rRm.Rules = rm.Rules;
                    }
                }

            }
            reader.Close();
        }

        public override void InitializeConfiguration()
        {
            _Mapping = new Mapping();

            if (_MappingFileName != null && _MappingFileName != string.Empty)
            {
                TextReader tr = new StreamReader(_MappingFileName);
                InitializeMapping(tr.ReadToEnd());
            }

            foreach (string mappingContent in _MappingContents)
            {
                InitializeMapping(mappingContent);
            }

            _Mapping.Initialize(_TablePrefix);
            _Mapping.Normalize(CreateDialect(_dialect));
            _Mapping.Driver = CreateDriver(_driver);
        }

        private void InitializeInheritance()
        {
            foreach (Evaluant.Uss.Models.Entity e in _Model.Entities)
            {
                Evaluant.Uss.Models.Entity current = e;
                EntityMapping em = _Mapping.Entities[e.Type];

                if (em != null)
                {
                    if (em.Type != "*")
                    {

                        if (_Model.GetParent(_Model.GetEntity(em.Type)) != null)
                        {
                            string parent = _Model.GetParent(_Model.GetEntity(em.Type)).Type;

                            if (_Mapping.Entities[parent] != null)
                                foreach (PrimaryKeyMapping pkm in em.Ids)
                                {
                                    if (_Mapping.Entities[parent, true].Ids[em.GetIdField(pkm)] != null)
                                        pkm.Generator.Params = _Mapping.Entities[parent, true].Ids[em.GetIdField(pkm)].Generator.Params;
                                }

                            while (_Model.GetParent(current) != null)
                            {
                                current = _Model.GetParent(current);
                                foreach (PrimaryKeyMapping pkm in _Mapping.Entities[em.Type, true].Ids)
                                {
                                    if (pkm.Generator.Name == GeneratorMapping.GeneratorType.inherited)
                                    {
                                        foreach (AttributeMapping am in em.Attributes)
                                            am.ParentField = em.IdFields;

                                        foreach (Evaluant.Uss.Models.Attribute a in _Model.GetInheritedAttributes(current.Type))
                                        {
                                            if (em.Attributes[a.Name] == null)
                                            {
                                                Evaluant.Uss.Models.Attribute att = _Model.GetAttribute(current.Type, a.Name);
                                                EntityMapping currentEm = _Mapping.Entities[current.Type, true];

                                                AttributeMapping am = currentEm.Attributes[a.Name, true];
                                                AttributeMapping currentAm = new AttributeMapping(am.Name, am.Type, currentEm);
                                                currentAm.ParentField = em.IdFields;
                                                currentAm.DbType = am.DbType;
                                                currentAm.Table = am.Table;
                                                currentAm.Precision = am.Precision;
                                                currentAm.Scale = am.Scale;
                                                currentAm.Size = am.Size;
                                                currentAm.Field = am.Field;
                                                em.Attributes.Add(currentAm);
                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (_Mapping.Entities[current.Type] != null)
                                        {
                                            em.CopyNode = current.Type;
                                            _Mapping.CopyNodeTo(em);
                                        }
                                    }
                                }
                            }
                        }
                        foreach (AttributeMapping am in em.Attributes)
                        {
                            if (_Model.GetAttribute(em.Type, am.Name) != null)
                                am.Type = _Model.GetAttribute(em.Type, am.Name).Type;
                            else
                            {
                                Evaluant.Uss.Models.Entity cur = _Model.GetEntity(em.Type);
                                while (_Model.GetParent(cur) != null)
                                {
                                    cur = _Model.GetParent(cur);
                                    if (_Model.GetAttribute(cur.Type, am.Name) != null)
                                        am.Type = _Model.GetAttribute(cur.Type, am.Name).Type;
                                }
                            }

                        }
                    }
                }
            }
        }

        private void InitializeReference()
        {
            foreach (EntityMapping parentEntity in _Mapping.Entities)
            {
                CacheEntityEntry cacheEntityEntry = GetCacheEntityEntry(parentEntity.Type);

                if (parentEntity.References != null)
                {
                    foreach (ReferenceMapping reference in parentEntity.References)
                    {
                        EntityMapping childEntity = _Mapping.Entities[reference.EntityChild, true];

                        // clause insert if if not a entity table
                        CacheReferenceEntry referenceEntry = new CacheReferenceEntry(reference);
                        cacheEntityEntry.CreateReferenceEntries.Add(referenceEntry);

                        if (reference.DiscriminatorField != null)
                        {
                            if (!referenceEntry.ContainsQuery(reference.Rules[0].ChildTable))
                            {
                                referenceEntry.QueryEntries.Add(new CacheQueryEntry(new InsertCommand(reference.Rules[0], reference.Rules[0].ChildTable)));

                                if (reference.Rules[0].Constraint != String.Empty)
                                    referenceEntry.DisableTable.Add(reference.Rules[0].ChildTable);
                            }

                            CacheQueryEntry entry = referenceEntry.GetQueryFromTableName(reference.Rules[0].ChildTable);

                            if (!entry.ContainsParameter(reference.DiscriminatorField))
                            {
                                Parameter param = new Parameter(reference, reference.DiscriminatorField);
                                entry.Parameters.Add(param);
                                ((InsertCommand)entry.Query).ColumnValueCollection.Add(reference.DiscriminatorField, param);
                            }
                        }

                        // if the specified parentField is not the parent's Id, then the parent's record has to be set

                        if (parentEntity.IdFields != reference.Rules[0].ParentField)
                        {
                            UpdateCommand parentEntityQuery = new UpdateCommand(parentEntity, parentEntity.Table);

                            CacheQueryEntry query = new CacheQueryEntry(parentEntityQuery);

                            for (int i = 0; i < reference.Rules[0].ParentFields.Length; i++)
                            {
                                string parentField = reference.Rules[0].ParentFields[i];

                                if (parentEntity.Ids[parentField] == null)
                                {
                                    Parameter param = new Parameter(reference.Rules[0], parentField);
                                    query.Parameters.Add(param);
                                    parentEntityQuery.ColumnValueCollection.Add(parentField, param);
                                }
                            }

                            foreach (PrimaryKeyMapping pkm in parentEntity.Ids)
                            {
                                if (parentEntity.GetIdField(pkm) == parentEntity.DiscriminatorField)
                                {
                                    parentEntityQuery.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                        new Column(pkm, parentEntity.GetIdField(pkm)), BinaryLogicOperator.Equals, new Constant(parentEntity.DiscriminatorValue, DbType.AnsiString)));
                                    continue;
                                }


                                Parameter param = new Parameter(parentEntity, parentEntity.GetIdField(pkm));
                                param.UseParentValue = true; // mandatory as by default the child id is used
                                query.Parameters.Add(param);
                                parentEntityQuery.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                    new Column(pkm, parentEntity.GetIdField(pkm)), BinaryLogicOperator.Equals, param));
                            }

                            if (reference.Rules[0].Constraint != String.Empty)
                                referenceEntry.DisableTable.Add(parentEntity.Table);

                            referenceEntry.QueryEntries.Add(query);
                        }


                        // if the specified childField is not the child's Id, then the child's record has to be set
                        if (childEntity.IdFields != reference.Rules[reference.Rules.Count - 1].ChildField)
                        {
                            UpdateCommand childEntityQuery = new UpdateCommand(childEntity, reference.Rules[reference.Rules.Count - 1].ChildTable);
                            CacheQueryEntry query = new CacheQueryEntry(childEntityQuery);

                            for (int i = 0; i < reference.Rules[reference.Rules.Count - 1].ChildFields.Length; i++)
                            {
                                string childField = reference.Rules[reference.Rules.Count - 1].ChildFields[i];

                                if (childEntity.Ids[childField] == null)
                                {
                                    Parameter param = new Parameter(reference.Rules[reference.Rules.Count - 1], childField);
                                    query.Parameters.Add(param);
                                    childEntityQuery.ColumnValueCollection.Add(childField, param);
                                }
                            }

                            foreach (PrimaryKeyMapping pkm in childEntity.Ids)
                            {
                                if (childEntity.GetIdField(pkm) == childEntity.DiscriminatorField)
                                {
                                    childEntityQuery.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                        new Column(pkm, childEntity.GetIdField(pkm)), BinaryLogicOperator.Equals, new Constant(childEntity.DiscriminatorValue, DbType.AnsiString)));
                                    continue;
                                }

                                Parameter param = new Parameter(childEntity, childEntity.GetIdField(pkm));
                                query.Parameters.Add(param);
                                childEntityQuery.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                    new Column(pkm, childEntity.GetIdField(pkm)), BinaryLogicOperator.Equals, param));
                            }

                            if (reference.Rules[reference.Rules.Count - 1].Constraint != String.Empty)
                                referenceEntry.DisableTable.Add(childEntity.Table);

                            referenceEntry.QueryEntries.Add(query);
                        }

                        for (int index = 0; index < reference.Rules.Count; index++)
                        {
                            RuleMapping rule = reference.Rules[index];
                            if (index != 0 && rule.ParentTable != parentEntity.Table)
                            {
                                if (!referenceEntry.ContainsQuery(rule.ParentTable))
                                {
                                    if (rule.Constraint != String.Empty)
                                        referenceEntry.DisableTable.Add(rule.ParentTable);

                                    referenceEntry.QueryEntries.Add(new CacheQueryEntry(new InsertCommand(rule, rule.ParentTable)));
                                }

                                CacheQueryEntry entry = referenceEntry.GetQueryFromTableName(rule.ParentTable);

                                if (!entry.ContainsParameter(rule.ParentField))
                                {
                                    Parameter param = new Parameter(rule, rule.ParentField);
                                    entry.Parameters.Add(param);
                                    if (entry.Query is InsertCommand)
                                        ((InsertCommand)entry.Query).ColumnValueCollection.Add(rule.ParentField, param);
                                    else if (entry.Query is UpdateCommand)
                                        ((UpdateCommand)entry.Query).ColumnValueCollection.Add(rule.ParentField, param);
                                }
                            }

                            if (index != reference.Rules.Count - 1 && rule.ChildTable != childEntity.Table)
                            {
                                if (!referenceEntry.ContainsQuery(rule.ChildTable))
                                {
                                    if (rule.Constraint != String.Empty)
                                        referenceEntry.DisableTable.Add(rule.ChildTable);

                                    referenceEntry.QueryEntries.Add(new CacheQueryEntry(new InsertCommand(rule, rule.ChildTable)));
                                }

                                CacheQueryEntry entry = referenceEntry.GetQueryFromTableName(rule.ChildTable);

                                Parameter param = new Parameter(rule, rule.ChildField);
                                entry.Parameters.Add(param);
                                ((InsertCommand)entry.Query).ColumnValueCollection.Add(rule.ChildField, param);
                            }
                        }

                    }
                }
            }
        }

        /// <summary>
        /// Create one update query per tag attribute mapping
        /// </summary>
        private void InitializeCompoundUpdateCommands()
        {
            Parameter param = null;

            foreach (EntityMapping entity in _Mapping.Entities)
            {
                CacheEntityEntry cacheEntityEntry = GetCacheEntityEntry(entity.Type);

                if (entity.Attributes != null)
                {
                    // Loops through attributes so as to create as many SQL Update Command as Tables
                    foreach (AttributeMapping attribute in entity.Attributes)
                    {
                        if (entity.Ids[attribute.Field] == null)
                        {
                            CacheQueryEntry entry = null;

                            if (attribute.Discriminator == null)
                            {
                                // Creates an UpdateCommand for each Table and add the Where clause for the command
                                if (!cacheEntityEntry.UpdateCompoundQueryEntriesContains(attribute.Table))
                                {
                                    CacheQueryEntry cacheEntry = new CacheQueryEntry(new UpdateCommand(attribute, attribute.Table));
                                    cacheEntityEntry.UpdateCompoundQueryEntries.Add(cacheEntry);

                                    if ((attribute.ParentField == null || attribute.ParentField == string.Empty) && attribute.Table != entity.Table)
                                    {
                                        attribute.ParentField = entity.IdFields;
                                        param = new Parameter(attribute, "FK_Entity");
                                        cacheEntry.Parameters.Add(param);
                                        ((UpdateCommand)cacheEntry.Query).WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(attribute, attribute.ParentField), BinaryLogicOperator.Equals, param));
                                    }
                                }

                                entry = cacheEntityEntry.GetUpdateCompoundQueryEntries(attribute.Table);
                            }
                            else
                            {
                                // Initialization 
                                entry = new CacheQueryEntry(new UpdateCommand(attribute, attribute.Table));

                                // A tag attribute mapping which name = "*", is an implicit generic query (a mapping for all attributes don't describe)
                                // it is differente to a explicit generic mapping (a mapping foreach attribute)
                                entry.IsAttributeGenericQuery = attribute.Name == "*";
                                cacheEntityEntry.UpdateCompoundQueryEntries.Add(entry);
                            }

                            UpdateCommand query = (UpdateCommand)entry.Query;
                            ArrayList parameters = entry.Parameters;

                            if (!query.ColumnValueCollection.ContainsKey(attribute.Field))
                            {
                                param = new Parameter(attribute, attribute.Field);
                                parameters.Add(param);
                                query.ColumnValueCollection.Add(attribute.Field, param);

                                if (attribute.Discriminator != null)
                                {
                                    if (attribute.Name != "*")
                                    {
                                        query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                            new Column(attribute, attribute.Discriminator),
                                            BinaryLogicOperator.Equals,
                                            new Constant(attribute.DiscriminatorValue == null ? attribute.Name : attribute.DiscriminatorValue, DbType.String)));


                                        param = new Parameter(attribute, "FK_Entity");
                                        parameters.Add(param);
                                        query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(attribute, attribute.ParentField), BinaryLogicOperator.Equals, param));
                                    }
                                    else
                                    {
                                        param = new Parameter(attribute, "AttributeDiscriminator");
                                        parameters.Add(param);
                                        query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(attribute, attribute.Discriminator), BinaryLogicOperator.Equals, param));

                                        param = new Parameter(attribute, "FK_Entity");
                                        parameters.Add(param);
                                        query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(attribute, attribute.ParentField), BinaryLogicOperator.Equals, param));
                                    }
                                }

                                if (attribute.ParentField == null && !parameters.Contains("EntityId"))
                                {
                                    foreach (PrimaryKeyMapping pkm in entity.Ids)
                                    {
                                        if (entity.GetIdField(pkm) == entity.DiscriminatorField)
                                            continue;

                                        param = new Parameter(pkm, "EntityId");
                                        parameters.Add(param);

                                        query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(
                                            new Column(entity, entity.GetIdField(pkm)),
                                            BinaryLogicOperator.Equals,
                                            param));
                                    }
                                }
                            }
                        }
                    }

                    if (!string.IsNullOrEmpty(entity.DiscriminatorField))
                    {
                        CacheQueryEntry entry = cacheEntityEntry.GetUpdateCompoundQueryEntries(entity.Table);
                        if (entry != null)
                        {
                            UpdateCommand query = (UpdateCommand)entry.Query;
                            query.WhereClause.SearchCondition.Add(new BinaryLogicExpression(new Column(entity, entity.DiscriminatorField),
                                BinaryLogicOperator.Equals,
                                new Constant(entity.DiscriminatorValue, DbType.AnsiString)));
                        }
                    }
                }
            }
        }



        private void InitializeCompoundCreateCommands()
        {
            Parameter param = null;

            foreach (EntityMapping entity in _Mapping.Entities)
            {
                CacheEntityEntry cacheEntityEntry = GetCacheEntityEntry(entity.Type);

                // Initialization 
                if (!cacheEntityEntry.InsertCompoundQueryEntriesContains(entity.Table))
                    cacheEntityEntry.InsertCompoundQueryEntries.Add(new CacheQueryEntry(new InsertCommand(entity, entity.Table)));

                InsertCommand query = (InsertCommand)cacheEntityEntry.GetInsertCompoundQueryEntries(entity.Table).Query;
                ArrayList parameters = cacheEntityEntry.GetInsertCompoundQueryEntries(entity.Table).Parameters;

                if (entity.DiscriminatorField != null && !query.ColumnValueCollection.ContainsKey(entity.DiscriminatorField))
                {
                    if (entity.Type != "*")
                        query.ColumnValueCollection.Add(entity.DiscriminatorField,
                            new Constant(entity.DiscriminatorValue == null ? entity.Type : entity.DiscriminatorValue, DbType.String));
                    else
                    {
                        param = new Parameter(entity, "EntityDiscriminator");
                        parameters.Add(param);
                        query.ColumnValueCollection.Add(entity.DiscriminatorField, param);
                    }
                }

                foreach (PrimaryKeyMapping pkm in entity.Ids)
                {
                    if (pkm.Generator.Name != GeneratorMapping.GeneratorType.native
                        && pkm.Generator.Name != GeneratorMapping.GeneratorType.business
                        && !query.ColumnValueCollection.ContainsKey(entity.GetIdField(pkm)))
                    {
                        bool parentIsNative = false;
                        //	If generator is inherited => check if parent generator is not native too

                        Evaluant.Uss.Models.Entity current = _Model.GetEntity(entity.Type);

                        if (current != null)
                        {
                            Evaluant.Uss.Models.Entity parent = _Model.GetParent(current);
                            while (parent != null)
                            {
                                EntityMapping parentMapping = _Mapping.Entities[parent.Type];
                                parent = _Model.GetParent(parent);
                                if (parentMapping != null)
                                {
                                    foreach (PrimaryKeyMapping ppkm in parentMapping.Ids)
                                    {
                                        if (ppkm.Generator.Name == GeneratorMapping.GeneratorType.native)
                                        {
                                            parentIsNative = true;
                                            break;
                                        }
                                    }
                                }
                            }
                        }

                        if (!parentIsNative)
                        {
                            param = new Parameter(pkm, "EntityId");
                            parameters.Add(param);
                            query.ColumnValueCollection.Add(entity.GetIdField(pkm), param);
                        }
                    }
                }

                if (entity.Attributes != null)
                {
                    foreach (AttributeMapping attribute in entity.Attributes)
                    {
                        CacheQueryEntry entry = null;
                        // Attributes are in a same table : one update query
                        if (attribute.Discriminator == null)
                        {
                            // Initialization 
                            if (!cacheEntityEntry.InsertCompoundQueryEntriesContains(attribute.Table))
                                cacheEntityEntry.InsertCompoundQueryEntries.Add(new CacheQueryEntry(new InsertCommand(attribute, attribute.Table)));

                            entry = cacheEntityEntry.GetInsertCompoundQueryEntries(attribute.Table);
                        }
                        else
                        {
                            // Initialization 
                            entry = new CacheQueryEntry(new InsertCommand(attribute, attribute.Table));
                            entry.IsAttributeGenericQuery = false;
                            cacheEntityEntry.InsertCompoundQueryEntries.Add(entry);
                        }

                        query = (InsertCommand)entry.Query;
                        parameters = entry.Parameters;

                        if (!query.ColumnValueCollection.ContainsKey(attribute.Field))
                        {
                            param = new Parameter(attribute, attribute.Field);
                            parameters.Add(param);
                            query.ColumnValueCollection.Add(attribute.Field, param);

                            if (attribute.Discriminator != null)
                            {
                                query.ColumnValueCollection.Add(attribute.Discriminator,
                                    new Constant(attribute.DiscriminatorValue == null ? attribute.Name : attribute.DiscriminatorValue, DbType.String));
                            }

                            if (attribute.ParentField != null && !query.ColumnValueCollection.ContainsKey(attribute.ParentField))
                            {
                                param = new Parameter(attribute, "FK_Entity");
                                if (!parameters.Contains(param))
                                    parameters.Add(param);
                                query.ColumnValueCollection.Add(attribute.ParentField, param);
                            }
                        }
                    }
                }
            }
        }

        private bool _initialized = false;
        protected void EnsureInitialized()
        {
            if (_initialized)
                return;

            _initialized = true;

            InitializeInheritance();
            InitializeAttributeTypes();

            _CacheEntityEntries = new ArrayList();

            foreach (EntityMapping entity in _Mapping.Entities)
            {
                _CacheEntityEntries.Add(new CacheEntityEntry(entity));
            }

            InitializeCompoundCreateCommands();
            InitializeCompoundUpdateCommands();
            InitializeReference();

        }

        private void InitializeAttributeTypes()
        {
            foreach (EntityMapping em in _Mapping.Entities)
            {
                foreach (AttributeMapping am in em.Attributes)
                {
                    if (am.Name != "*")
                    {
                        //if (_Model.GetEntity(em.Type) == null)
                        //    continue;

                        Evaluant.Uss.Models.Attribute a = _Model.GetAttribute(em.Type, am.Name);
                        if (a != null)
                        {
                            am.Type = a.Type;
                        }
                        else
                        {
                            if (_Model.GetEntity(em.Type) != null)
                            {
                                Evaluant.Uss.Models.Entity current = _Model.GetEntity(em.Type);

                                if (_Model.GetParent(_Model.GetEntity(em.Type)) != null)
                                {
                                    string parent = _Model.GetParent(_Model.GetEntity(em.Type)).Type;

                                    while (_Model.GetParent(current) != null)
                                    {
                                        current = _Model.GetParent(current);
                                        a = _Model.GetAttribute(current.Type, am.Name);
                                        if (a != null)
                                        {
                                            am.Type = a.Type;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }


        public override IPersistenceEngine CreatePersistenceEngine()
        {
            lock (_synLock)
            {
                EnsureInitialized();

                SqlMapperPersistenceEngine sqlPE = new SqlMapperPersistenceEngine(_ConnectionString, CreateDriver(_driver), CreateDialect(_dialect), _Mapping, _CacheEntityEntries, _Model);
                sqlPE.Dialect.Culture = _Culture;
                sqlPE.Culture = _Culture;
                sqlPE.Initialize();

                return sqlPE;
            }
        }

        #endregion

        protected string _ConnectionString;
        public string ConnectionString
        {
            get { return _ConnectionString; }
            set { _ConnectionString = value; }
        }

        protected string _dialect;
        public string Dialect
        {
            get { return _dialect; }
            set { _dialect = value; }
        }

        protected DBDialect CreateDialect(string dialect)
        {
            Type type = Utils.GetType(dialect);

            if (type == null && dialect.Split(',').Length >= 2)
            {
                string[] values = dialect.Split(',');
                type = Assembly.LoadWithPartialName(values[1]).GetType(values[0]);
            }

            if (type == null)
                throw new PersistenceEngineException("The specified type could not be found: " + dialect);

            ConstructorInfo pc = type.GetConstructor(Type.EmptyTypes);
            DBDialect dbDialect = (DBDialect)pc.Invoke(new object[] { });

            //dbDialect.Schema = _Schema;

            if (options != null)
                dbDialect.Options = options.Split(' ');

            return dbDialect;
        }

        private string options;

        public string Options
        {
            get { return options; }
            set { options = value; }
        }


        protected string _driver;
        public string Driver
        {
            get { return _driver; }
            set { _driver = value; }
        }

        protected IDriver CreateDriver(string driver)
        {
            Type type = Utils.GetType(driver);

            if (type == null && driver.Split(',').Length >= 2)
            {
                string[] values = driver.Split(',');
                type = Assembly.LoadWithPartialName(values[1]).GetType(values[0]);
            }

            if (type == null)
                throw new PersistenceEngineException("The specified type could not be found: " + driver);

            ConstructorInfo pc = type.GetConstructor(Type.EmptyTypes);
            return (IDriver)pc.Invoke(new object[] { });
        }

        protected string _MappingFileName;
        public string MappingFileName
        {
            get { return _MappingFileName; }
            set { _MappingFileName = value; }
        }

        protected Mapping _Mapping;
        public Mapping Mapping
        {
            get { return _Mapping; }
            set { _Mapping = value; }
        }

        protected string _Schema;
        public string Schema
        {
            get { return _Schema; }
            set { _Schema = value; }
        }


    }
}
