<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">

<xsl:key name="classId" match="//Class[@xmi.id] | //AssociationClass[@xmi.id]" use="@xmi.id"/>
<xsl:key name="attributeId" match="//Enumeration[@xmi.id] | //Classifier[@xmi.id]" use="@xmi.id"/>
<xsl:key name="ImplementationId" match="//Interface[@xmi.id]" use="@xmi.id"/>
<xsl:key name="MultiplicityId" match="//Association/Association.connection/AssociationEnd[@xmi.id]/AssociationEnd.multiplicity/Multiplicity[@xmi.id]" use="@xmi.id"/>

<xsl:key name="stereotypeId" match="//Stereotype[@xmi.id]" use="@xmi.id"/>
<xsl:key name="DataTypeId" match="//DataType[@xmi.id] | //Class[@xmi.id]" use="@xmi.id"/>

<xsl:variable name="lowerCase" select="'abcdefghijklmnopqrstuvwxyz'"/>
<xsl:variable name="upperCase" select="'ABCDEFGHIJKLMNOPQRSTUVWXYZ'"/>

<xsl:template match="/">
<UMLEditor>
	<xsl:choose>
		<xsl:when test="//Model">
			<xsl:apply-templates select="//Model" />
		</xsl:when>
		<xsl:otherwise>
			<xsl:apply-templates select="//XMI.content/node()" mode="PartialModel" />
		</xsl:otherwise>
	</xsl:choose>
</UMLEditor>
</xsl:template>

<xsl:template match="*" mode="PartialModel">
	<xsl:element name="Model">
		<!-- Package -->
		<xsl:element name="Package">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
			
		<xsl:for-each select="//Class[@xmi.id]">
			<xsl:variable name="idclass" select="@xmi.id"/>

			<xsl:choose>
				<!-- test si la classe possède une stéréotype -->
				<xsl:when test="../Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]">
					
					<!-- Si oui : définition du stéréotype -->
					<xsl:variable name="stereotype" select="../Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]/../../ModelElement.name"/>
					
					<!-- Suivant le stéréotype .... -->
					<xsl:choose>
						<!-- Enumeration -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='enumeration'">
							<xsl:apply-templates select="." mode="Enumeration"/>
						</xsl:when>
						
						<!-- Singleton -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='singleton'">
							<xsl:apply-templates select="." mode="Singleton"/>
						</xsl:when>
					</xsl:choose>			
				</xsl:when>
				
				<xsl:otherwise>
					<!-- classe "normale" -->
					<xsl:apply-templates select="." mode="Class"/>
				</xsl:otherwise>
			</xsl:choose>
			
		</xsl:for-each>
		
		<!-- classe d'association -->
		<xsl:apply-templates select="//AssociationClass[@xmi.id]" mode="AssociationClass"/>
		
		<!-- interfaces -->
		<xsl:apply-templates select="//Interface[@xmi.id]" mode="Interface"/>
		
	</xsl:element>
		
		<!-- Héritage -->
		<xsl:apply-templates select="//Generalization[@xmi.id]/Generalization.parent/child::node()/../.." mode="Generalisation"/>
		
		<!-- Implémentation -->
		<!-- Together-->
		<xsl:apply-templates select="//Generalization[@xmi.id]/Generalization.parent/Interface/../.." mode="Implementation_Together"/>
		<!-- ArgoUML-->
		<xsl:apply-templates select="//Abstraction[@xmi.id]" mode="Implementation_ArgoUml"/>

		
		<!-- Association -->
		<xsl:apply-templates select="//Association[@xmi.id]" mode="Association"/>
		<xsl:apply-templates select="//AssociationClass[@xmi.id]" mode="Association"/>
	</xsl:element>
</xsl:template>

<xsl:template match="Model" >
	<xsl:element name="Model">
		<!-- Package -->
		<xsl:apply-templates select="." mode="defaultPackage" />
		
		<!-- Héritage -->
		<xsl:apply-templates select="//Generalization[@xmi.id]/Generalization.parent/child::node()/../.." mode="Generalisation"/>
		
		<!-- Implémentation -->
		<!-- Together-->
		<xsl:apply-templates select="//Generalization[@xmi.id]/Generalization.parent/Interface/../.." mode="Implementation_Together"/>
		<!-- ArgoUML-->
		<xsl:apply-templates select="//Abstraction[@xmi.id]" mode="Implementation_ArgoUml"/>

		
		<!-- Association -->
		<xsl:apply-templates select="//Association[@xmi.id]" mode="Association"/>
		<xsl:apply-templates select="//AssociationClass[@xmi.id]" mode="Association"/>
	</xsl:element>
</xsl:template>

<!-- Model (Package) -->
<xsl:template match="Model" mode="defaultPackage">
	<xsl:element name="Package">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
			
		<xsl:for-each select="//Class[@xmi.id]">
			<xsl:variable name="idclass" select="@xmi.id"/>

			<xsl:choose>
				<!-- test si la classe possède une stéréotype -->
				<xsl:when test="../Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]">
					
					<!-- Si oui : définition du stéréotype -->
					<xsl:variable name="stereotype" select="../Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]/../../ModelElement.name"/>
					
					<!-- Suivant le stéréotype .... -->
					<xsl:choose>
						<!-- Enumeration -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='enumeration' ">
							<xsl:apply-templates select="." mode="Enumeration"/>
						</xsl:when>
						
						<!-- Singleton -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='singleton'">
							<xsl:apply-templates select="." mode="Singleton"/>
						</xsl:when>
					</xsl:choose>			
				</xsl:when>
				
				<xsl:otherwise>
					<!-- classe "normale" -->
					<xsl:apply-templates select="." mode="Class"/>
				</xsl:otherwise>
			</xsl:choose>
			
		</xsl:for-each>
		
		<!-- classe d'association -->
		<xsl:apply-templates select="//AssociationClass[@xmi.id]" mode="AssociationClass"/>
		
		<!-- interfaces -->
		<xsl:apply-templates select="//Interface[@xmi.id]" mode="Interface"/>
		
	</xsl:element>
</xsl:template>

<!-- Model (Package) -->
<xsl:template match="Package">
	<xsl:element name="Package">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
			
		<xsl:for-each select="Namespace.ownedElement/Class[@xmi.id]">
			<xsl:variable name="idclass" select="@xmi.id"/>

			<xsl:choose>
				<!-- test si la classe possède une stéréotype -->
				<xsl:when test="ModelElement.stereotype/Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]">
					
					<!-- Si oui : définition du stéréotype -->
					<xsl:variable name="stereotype" select="../Stereotype[@xmi.id]/Stereotype.extendedElement/ModelElement[@xmi.idref=$idclass]/../../ModelElement.name"/>
					
					<!-- Suivant le stéréotype .... -->
					<xsl:choose>
						<!-- Enumeration -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='enumeration'">
							<xsl:apply-templates select="." mode="Enumeration"/>
						</xsl:when>
						
						<!-- Singleton -->
						<xsl:when test="translate($stereotype,$upperCase,$lowerCase)='singleton'">
							<xsl:apply-templates select="." mode="Singleton"/>
						</xsl:when>
					</xsl:choose>			
				</xsl:when>
				
				<xsl:otherwise>
					<!-- classe "normale" -->
					<xsl:apply-templates select="." mode="Class"/>
				</xsl:otherwise>
			</xsl:choose>
			
		</xsl:for-each>
		
		<!-- classe d'association -->
		<xsl:apply-templates select="//AssociationClass[@xmi.id]" mode="AssociationClass"/>
		
		<!-- interfaces -->
		<xsl:apply-templates select="//Interface[@xmi.id]" mode="Interface"/>
		
	</xsl:element>
</xsl:template>

<!-- Classes -->
<xsl:template match="Class" mode="Class">
	<xsl:element name="Class">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>		
		<xsl:apply-templates select="ModelElement.visibility | @visibility" mode="GetModelElementVisibility" />		
		<xsl:apply-templates select="GeneralizableElement.isAbstract | @isAbstract" mode="GetGeneralizableElementIsAbstract" />
		
		<xsl:attribute name="final">false</xsl:attribute>
		
		<xsl:apply-templates select="ModelElement.stereotype" mode="GetModelElementStereotype" />
		
		<xsl:attribute name="associationClass">false</xsl:attribute>
		
		<!-- Attributs -->
		<xsl:apply-templates select="Classifier.feature/Attribute[@xmi.id]" mode="Property"/>
		
		<!-- Operations -->
		<!--<xsl:apply-templates select="Classifier.feature/Operation[@xmi.id]" mode="Operation"/>-->
		
	</xsl:element>
</xsl:template>

<!-- Classe d'Association -->
<xsl:template match="AssociationClass" mode="AssociationClass">
	<xsl:element name="Class">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:apply-templates select="ModelElement.visibility | @visibility" mode="GetModelElementVisibility" />			
		<xsl:apply-templates select="GeneralizableElement.isAbstract | @isAbstract" mode="GetGeneralizableElementIsAbstract" />

		<xsl:attribute name="final">false</xsl:attribute>
		<xsl:apply-templates select="ModelElement.stereotype" mode="GetModelElementStereotype" />
		<xsl:attribute name="associationClass">true</xsl:attribute>
		
		<!-- Attributs -->
		<xsl:apply-templates select="Classifier.feature/Attribute[@xmi.id]" mode="Property"/>
		
		<!-- Operations -->
		<!--<xsl:apply-templates select="Classifier.feature/Operation[@xmi.id]" mode="Operation"/>-->
		
	</xsl:element>
</xsl:template>


<!-- Attributs -->
<xsl:template match="Attribute" mode="Property">
	<xsl:variable name="attributeName" select="ModelElement.name | @name"/>
	
	<xsl:if test="not( starts-with($attributeName, 'lnk'))">
		<xsl:element name="Property">
			<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
			<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"/>
			<xsl:apply-templates select="StructuralFeature.type/Classifier" mode="GetDataType" />
			<xsl:apply-templates select="ModelElement.visibility | @visibility" mode="GetModelElementVisibility" />
			<xsl:attribute name="static">???</xsl:attribute>
			<xsl:apply-templates select="Attribute.initialValue" mode="GetAttributeInitialValue" />
		</xsl:element>
	</xsl:if>
</xsl:template>

<!-- Operations -->
<xsl:template match="Operation" mode="Operation">
	<xsl:element name="Operation">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"/>
		
		<xsl:choose>
			<xsl:when test="BehavioralFeature.parameter/Parameter/Parameter.kind[@xmi.value='return']">
				<xsl:apply-templates select="BehavioralFeature.parameter/Parameter/Parameter.kind[@xmi.value='return']/../Parameter.type/Classifier" mode="GetDataType"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="type">void</xsl:attribute>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:apply-templates select="ModelElement.visibility | @visibility" mode="GetModelElementVisibility" />
		<xsl:attribute name="static">???</xsl:attribute>
		<xsl:attribute name="modifier">m_none</xsl:attribute>
		<xsl:attribute name="code"/>
		
		<xsl:apply-templates select="BehavioralFeature.parameter/Parameter/Parameter.kind[@xmi.value='in' or @xmi.value='inout']/.." mode="GetParameter" />		

	</xsl:element>
</xsl:template>


<!-- Interface -->
<xsl:template match="Interface" mode="Interface">
	<xsl:element name="Interface">
		<xsl:apply-templates select="ModelElement.name | @name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		
		<xsl:apply-templates select="ModelElement.visibility" mode="GetModelElementVisibility" />
		
		<xsl:attribute name="final">???</xsl:attribute>
		
		<!-- Attributs -->
		<xsl:apply-templates select="Classifier.feature/Attribute[@xmi.id]" mode="Property"/>
		
		<!-- Operations -->
		<!--<xsl:apply-templates select="Classifier.feature/Operation[@xmi.id]" mode="Operation"/>-->
		
	</xsl:element>
</xsl:template>


<!-- Enumeration -->
<xsl:template match="Class" mode="Enumeration">
	<xsl:element name="Enumeration">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 	
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		
		<xsl:apply-templates select="ModelElement.visibility" mode="GetModelElementVisibility" />		
		<xsl:attribute name="final">???</xsl:attribute>
		
		<!-- Literal -->
		<xsl:apply-templates select="Classifier.feature/Attribute[@xmi.id]" mode="Literal"/>
		
	</xsl:element>
</xsl:template>

<!-- literal -->
<xsl:template match="Attribute" mode="Literal">
	<xsl:element name="Property">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 			
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:attribute name="type"/>
		<xsl:attribute name="visibility"/>
		<xsl:attribute name="static"/>
		<xsl:attribute name="defaultValue"/>

	</xsl:element>
</xsl:template>

<!-- Singleton -->
<xsl:template match="Class" mode="Singleton">
	<xsl:element name="Class">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 		
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		
		<xsl:apply-templates select="ModelElement.visibility" mode="GetModelElementVisibility" />	
		
		<xsl:attribute name="final">???</xsl:attribute>
		<xsl:apply-templates select="GeneralizableElement.isAbstract" mode="GetGeneralizableElementIsAbstract" />
		<xsl:attribute name="final">false</xsl:attribute>
		<xsl:apply-templates select="ModelElement.stereotype" mode="GetModelElementStereotype" />
	
		<xsl:attribute name="associationClass">false</xsl:attribute>
		<xsl:attribute name="isSingleton">true</xsl:attribute>
		
		<!-- Attributs -->
		<xsl:apply-templates select="Classifier.feature/Attribute[@xmi.id]" mode="Property"/>
		
	</xsl:element>
</xsl:template>



<!-- l'héritage -->
<xsl:template match="Generalization" mode="Generalisation">
	<xsl:element name="Generalization">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:variable name="idvalue1" select="Generalization.child/child::node()/@xmi.idref"/>
		<xsl:variable name="idvalue2" select="Generalization.parent/child::node()/@xmi.idref"/>
		<xsl:attribute name="clientClassName"><xsl:value-of select="key('classId',$idvalue1)/ModelElement.name"/></xsl:attribute>
		<xsl:attribute name="supplierClassName"><xsl:value-of select="key('classId',$idvalue2)/ModelElement.name"/></xsl:attribute>
	</xsl:element>
</xsl:template>

<!-- l'implémentation (Together)-->
<xsl:template match="Generalization" mode="Implementation_Together">
	<xsl:element name="Implementation">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:variable name="idvalue1" select="Generalization.child/child::node()/@xmi.idref"/>
		<xsl:variable name="idvalue2" select="Generalization.parent/child::node()/@xmi.idref"/>
		<xsl:attribute name="clientClassName"><xsl:value-of select="key('classId',$idvalue1)/ModelElement.name"/></xsl:attribute>
		<xsl:attribute name="supplierClassName"><xsl:value-of select="key('ImplementationId',$idvalue2)/ModelElement.name"/></xsl:attribute>
	</xsl:element>
</xsl:template>

<!-- l'implémentation (ArgoUML) -->
<xsl:template match="Abstraction" mode="Implementation_ArgoUml">
	<xsl:element name="Implementation">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:variable name="idvalue1" select="Dependency.client/ModelElement/@xmi.idref"/>
		<xsl:variable name="idvalue2" select="Dependency.supplier/ModelElement/@xmi.idref"/>
		<xsl:attribute name="clientClassName"><xsl:value-of select="key('classId',$idvalue1)/ModelElement.name"/></xsl:attribute>
		<xsl:attribute name="supplierClassName"><xsl:value-of select="key('ImplementationId',$idvalue2)/ModelElement.name"/></xsl:attribute>
	</xsl:element>
</xsl:template>

<!-- l'Association -->
<xsl:template match="Association" mode="Association">
	<xsl:element name="Association">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 		
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:attribute name="type"><xsl:apply-templates select="Association.connection" mode="GetAssociationType"/></xsl:attribute>
		<xsl:attribute name="navigable"><xsl:apply-templates select="Association.connection" mode="GetAssociationNavigability"/></xsl:attribute>

		<xsl:variable name="client"><xsl:apply-templates select="Association.connection" mode="GetAssociationClient"/></xsl:variable>
		<xsl:variable name="supplier"><xsl:apply-templates select="Association.connection" mode="GetAssociationSupplier"/></xsl:variable>

		<xsl:attribute name="associationClassifier"/>
		
		<xsl:attribute name="clientClassName">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationClassName"><xsl:with-param name="role" select="$client"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="clientRole">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationClientRoleName"><xsl:with-param name="role" select="$client"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="clientMultiplicity">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationMultiplicity"><xsl:with-param name="role" select="$client"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="clientQualifierName"/>
		<xsl:attribute name="clientQualifierType"/>
		<xsl:attribute name="supplierClassName">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationClassName"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="supplierRole">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationRoleName"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="supplierMultiplicity">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationMultiplicity"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates>
		</xsl:attribute>
		<xsl:attribute name="supplierQualifierName"></xsl:attribute><!-- non défini -->
		<xsl:attribute name="supplierQualifierType"></xsl:attribute>
	</xsl:element>
</xsl:template>

<!-- l'Association avec classe d'association-->
<xsl:template match="AssociationClass" mode="Association">
	<xsl:element name="Association">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName"/> 
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation"	/>
		<xsl:attribute name="type"><xsl:apply-templates select="Association.connection" mode="GetAssociationType"/></xsl:attribute>
		<xsl:attribute name="navigable"><xsl:apply-templates select="Association.connection" mode="GetAssociationNavigability"/></xsl:attribute>
		<xsl:variable name="client">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationClient"/>
		</xsl:variable>
		<xsl:variable name="supplier">
			<xsl:apply-templates select="Association.connection" mode="GetAssociationSupplier"/>
		</xsl:variable>
		<xsl:attribute name="associationClassifier"><xsl:value-of select="ModelElement.name"/></xsl:attribute>
		<xsl:attribute name="clientClassName"><xsl:apply-templates select="Association.connection" mode="GetAssociationClassName"><xsl:with-param name="role" select="$client"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="clientRole"><xsl:apply-templates select="Association.connection" mode="GetAssociationClientRoleName"><xsl:with-param name="role" select="$client"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="clientMultiplicity"><xsl:apply-templates select="Association.connection" mode="GetAssociationMultiplicity"><xsl:with-param name="role" select="$client"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="clientQualifierName"/>
		<xsl:attribute name="clientQualifierType"/>
		<xsl:attribute name="supplierClassName"><xsl:apply-templates select="Association.connection" mode="GetAssociationClassName"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="supplierRole"><xsl:apply-templates select="Association.connection" mode="GetAssociationRoleName"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="supplierMultiplicity"><xsl:apply-templates select="Association.connection" mode="GetAssociationMultiplicity"><xsl:with-param name="role" select="$supplier"/></xsl:apply-templates></xsl:attribute>
		<xsl:attribute name="supplierQualifierName"/>
		<!-- non défini -->
		<xsl:attribute name="supplierQualifierType"/>
	</xsl:element>
</xsl:template>

<!-- retourne le nom de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationName">
	<xsl:choose>
		<xsl:when test="../ModelElement.name != '' ">
			<xsl:value-of select="../ModelElement.name"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="../@xmi.id"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- retourne le type de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationType">
	<xsl:variable name="class1" select="AssociationEnd[1]/AssociationEnd.aggregation/@xmi.value"/>
	<xsl:variable name="class2" select="AssociationEnd[2]/AssociationEnd.aggregation/@xmi.value"/>

	<xsl:choose>
		<xsl:when test="$class1='none' and $class2='none'">association</xsl:when>
		<xsl:when test="$class1='aggregate' or $class2='aggregate'">aggregation</xsl:when>
		<xsl:when test="$class1='composite' or $class2='composite'">composition</xsl:when>
	</xsl:choose>
</xsl:template>

<!-- retourne le client de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationClient">
	<xsl:variable name="class1" select="AssociationEnd[1]/AssociationEnd.aggregation/@xmi.value"/>

	<xsl:choose>
		<xsl:when test="$class1='none'">2</xsl:when>
		<xsl:otherwise>1</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- retourne le supplier de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationSupplier">
	<xsl:variable name="class1" select="AssociationEnd[1]/AssociationEnd.aggregation/@xmi.value"/>

	<xsl:choose>
		<xsl:when test="$class1='none'">1</xsl:when>
		<xsl:otherwise>2</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- retourne la navigabilité de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationNavigability">
	
	<xsl:variable name="client"><xsl:apply-templates select="Association.connection" mode="GetAssociationClient"/></xsl:variable>
	
	<xsl:variable name="clientNavigability" select="AssociationEnd[position()=$client]/AssociationEnd.isNavigable/@xmi.value"/>
	
	<xsl:variable name="associationType"><xsl:apply-templates select="." mode="GetAssociationType"/></xsl:variable>
	
	<xsl:choose>
		<xsl:when test="$associationType='association'">false</xsl:when>
		<xsl:otherwise>
			<xsl:choose>
				<xsl:when test="$clientNavigability='true'">false</xsl:when>
				<xsl:otherwise>true</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!-- retourne le nom de la classe client/supplier de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationClassName">
	<xsl:param name="role"/>
	
	<!-- XMI 1.3 ARGO UML-->
	<!--<xsl:variable name="clientId" select="AssociationEnd[position()=$role]/AssociationEnd.type/Classifier/@xmi.idref | AssociationEnd[position()=$role]/AssociationEnd.type/AssociationClass/@xmi.idref"/>-->

	<!-- XMI 1.3 TOGETHER -->
	<!--<xsl:variable name="clientId" select="AssociationEnd[position()=$role]/AssociationEnd.type/Class/@xmi.idref | AssociationEnd[position()=$role]/AssociationEnd.type/AssociationClass/@xmi.idref"/>-->

	<xsl:variable name="clientId" select="AssociationEnd[position()=$role]/AssociationEnd.participant/child::node()/@xmi.idref"/>

	<xsl:value-of select="key('classId', $clientId)/ModelElement.name"/>	
</xsl:template>

<!-- retourne le role du client/supplier de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationRoleName">
	<xsl:param name="role"/>
	
	<xsl:value-of select="AssociationEnd[position()=$role]/ModelElement.name"/>
</xsl:template>

<!-- retourne le role du client de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationClientRoleName">
	<xsl:param name="role"/>
	
	<xsl:choose>
		<xsl:when test="AssociationEnd[position()=$role]/ModelElement.name = 'undefined'">
			<xsl:variable name="roleSupplier"><xsl:apply-templates select="." mode="GetAssociationSupplier"/></xsl:variable>
			<xsl:apply-templates select="." mode="GetAssociationClassName"><xsl:with-param name="role" select="$role"/></xsl:apply-templates>
			<xsl:value-of select="AssociationEnd[position()=$roleSupplier]/ModelElement.name" />
			<xsl:text>Parent</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="AssociationEnd[position()=$role]/ModelElement.name"/>	
		</xsl:otherwise>
	</xsl:choose>
	
</xsl:template>


<!-- retourne la multiplicité du client/supplier de l'association -->
<xsl:template match="Association.connection" mode="GetAssociationMultiplicity">
	<xsl:param name="role"/>
	
	<xsl:variable name="lv">
		<!-- XMI 1.3 ARGO UML -->
		<xsl:if test="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity[@xmi.id or @xmi.idref]">
			<xsl:variable name="idvalue" select="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/@xmi.id | ./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/@xmi.idref"/>
	
			<xsl:value-of select="key('MultiplicityId',$idvalue)/Multiplicity.range/MultiplicityRange/MultiplicityRange.lower"/>
		</xsl:if>
		
		<!-- XMI 1.3 TOGETHER -->
		<xsl:if test="not(./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity[@xmi.id or @xmi.idref])">
			<xsl:value-of select="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/Multiplicity.range/MultiplicityRange/MultiplicityRange.lower"/>
		</xsl:if>
	</xsl:variable>
	
		<xsl:variable name="uv">
		<!-- XMI 1.3 ARGO UML -->
		<xsl:if test="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity[@xmi.id or @xmi.idref]">
			<xsl:variable name="idvalue" select="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/@xmi.id | ./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/@xmi.idref"/>
	
			<xsl:value-of select="key('MultiplicityId',$idvalue)/Multiplicity.range/MultiplicityRange/MultiplicityRange.upper"/>
		</xsl:if>
		
		<!-- XMI 1.3 TOGETHER -->
		<xsl:if test="not(./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity[@xmi.id or @xmi.idref])">
			<xsl:value-of select="./AssociationEnd[position()=$role]/AssociationEnd.multiplicity/Multiplicity/Multiplicity.range/MultiplicityRange/MultiplicityRange.upper"/>
		</xsl:if>
	</xsl:variable>
	
	
	<xsl:variable name="upperValue">
		<xsl:choose>
			<xsl:when test="$uv='-1'">*</xsl:when>
			<xsl:otherwise><xsl:value-of select="$uv"/></xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="xmiMultiplicity">
		<xsl:value-of select="$lv"/>
		<xsl:text>..</xsl:text>
		<xsl:value-of select="$upperValue"/>		
	</xsl:variable>

	<xsl:choose>
		<xsl:when test="$xmiMultiplicity='1..1'">1</xsl:when>
		<xsl:otherwise><xsl:value-of select="$xmiMultiplicity"/></xsl:otherwise>
	</xsl:choose>
	
</xsl:template>

<!-- ***************************************** DTD UML 1.3 ***************************************************************** -->
<xsl:template match="ModelElement.name | @name" mode="GetModelElementName" >
	<xsl:choose>
		<xsl:when test="current() = '' or current() = 'undefined' ">
			<xsl:attribute name="name"><xsl:value-of select="../@xmi.id"/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
			<xsl:attribute name="name"><xsl:value-of select="."/></xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="ModelElement.visibility" mode="GetModelElementVisibility" >
	<xsl:choose>
		<xsl:when test=".">
			<xsl:attribute name="visibility">v_<xsl:value-of select="@xmi.value"/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
			<xsl:attribute name="visibility">v_private</xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="@visibility" mode="GetModelElementVisibility" >
	<xsl:choose>
		<xsl:when test=".">
			<xsl:attribute name="visibility">v_<xsl:value-of select="."/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
			<xsl:attribute name="visibility">v_private</xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="GeneralizableElement.isRoot" mode="GetGeneralizableElementIsRoot" >
	<xsl:attribute name="isRoot"><xsl:value-of select="@xmi.value"/></xsl:attribute>
</xsl:template>

<xsl:template match="@isRoot" mode="GetGeneralizableElementIsRoot" >
	<xsl:attribute name="isRoot"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="GeneralizableElement.isAbstract" mode="GetGeneralizableElementIsAbstract" >
	<xsl:attribute name="abstract"><xsl:value-of select="@xmi.value"/></xsl:attribute>
</xsl:template>

<xsl:template match="@isAbstract" mode="GetGeneralizableElementIsAbstract" >
	<xsl:attribute name="abstract"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="GeneralizableElement.isLeaf" mode="GetGeneralizableElementIsLeaf" >
	<xsl:attribute name="isLeaf"><xsl:value-of select="@xmi.value"/></xsl:attribute>
</xsl:template>

<xsl:template match="@isLeaf" mode="GetGeneralizableElementIsLeaf" >
	<xsl:attribute name="isLeaf"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<xsl:template match="Class.isActive" mode="GetClassIsActive" >
	<xsl:attribute name="isActive"><xsl:value-of select="@xmi.value"/></xsl:attribute>
</xsl:template>

<xsl:template match="@isActive" mode="GetClassIsActive" >
	<xsl:attribute name="isActive"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<!-- retourne la documentation d'une entitée -->
<xsl:template match="ModelElement.taggedValue" mode="GetDocumentation">
	<xsl:attribute name="documentation">
		<xsl:value-of select="TaggedValue[TaggedValue.tag = 'documentation']/TaggedValue.value"/>
	</xsl:attribute>
</xsl:template>

<xsl:template match="ModelElement.stereotype" mode="GetModelElementStereotype">
	<xsl:variable name="idstereotype" select="Stereotype/@xmi.idref"/>
	<xsl:attribute name="stereotype"><xsl:value-of select="key('stereotypeId', $idstereotype)/ModelElement.name"/></xsl:attribute>
</xsl:template>

<xsl:template match="Attribute.initialValue" mode="GetAttributeInitialValue">
	<xsl:attribute name="defaultValue"><xsl:value-of 	select="Attribute.initialValue/Expression/Expression.body"/></xsl:attribute>	
</xsl:template>

<xsl:template match="Classifier" mode="GetDataType">
	<xsl:variable name="idvalue" select="@xmi.idref"/>
	<xsl:variable name="name" select="key('DataTypeId',$idvalue)/ModelElement.name | key('DataTypeId',$idvalue)/@name" />
					
	<xsl:choose>
		<!-- si le type n'est pas défini -->
		<xsl:when test="$name = '' or $name= '&amp;lt;undefined&amp;gt;'">
			<xsl:attribute name="type">void</xsl:attribute>
		</xsl:when>
		<xsl:when test="$name = 'integer'">
			<xsl:attribute name="type">int</xsl:attribute>
		</xsl:when>
		<xsl:when test="$name = 'boolean'">
			<xsl:attribute name="type">bool</xsl:attribute>
		</xsl:when>
		<xsl:when test="$name = 'real'">
			<xsl:attribute name="type">float</xsl:attribute>
		</xsl:when>
		<!-- si le type est défini -->
		<xsl:otherwise>
			<xsl:attribute name="type"><xsl:value-of select="$name"/></xsl:attribute>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="Parameter" mode="GetParameter">
	<xsl:element name="Parameter">
		<xsl:apply-templates select="ModelElement.name" mode="GetModelElementName" />
		<xsl:apply-templates select="Parameter.type/Classifier" mode="GetDataType" />
		<xsl:apply-templates select="ModelElement.taggedValue" mode="GetDocumentation" />		
	</xsl:element>
</xsl:template>




</xsl:stylesheet>
