using System;
using System.Collections;
using System.Collections.Specialized;
using Evaluant.Uss.Commands;

namespace Evaluant.Uss.Collections
{
	/// <summary>
	/// Description rsume de HashedArray.
	/// </summary>
	[Serializable]
	public class CommandCollection : ICollection
	{

		private static int MAX = 8;

		// Cuts all commands into predefined sections, using their ProcessOrder
		private HashedList[] _InnerLists = new HashedList[MAX];

		public CommandCollection()
		{
			for(int i=0; i<MAX; i++)
				_InnerLists[i] = new HashedList();
		}

		public bool Contains(Command command)
		{
			return _InnerLists[command.ProcessOrder].Contains(command);
		}

		public CommandCollection(IEnumerable commands) : this()
		{
            AddRange(commands);
		}

		public void AddRange(IEnumerable commands)
		{
			foreach(Command c in commands)
			{
				this.Add(c);
			}
		}

		public void Add(Command c)
		{
			if(c == null)
				throw new ArgumentNullException("A Command cannot be null");

			_InnerLists[c.ProcessOrder].Add(c);
		}

		#region Membres de IEnumerable

		public IEnumerator GetEnumerator()
		{
			return new Enumerator(this);
		}

		#endregion

		#region Membres de ICollection

		public bool IsSynchronized
		{
			get
			{
				return false;
			}
		}

		public int Count
		{
			get
			{
				int count = 0;
				for(int i=0; i<_InnerLists.Length; i++)
					count += _InnerLists[i].Count;

				return count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			throw new NotImplementedException();
		}

		public object SyncRoot
		{
			get
			{
				return _InnerLists;
			}
		}

		#endregion

		class Enumerator : IEnumerator
		{
			CommandCollection _Commands;
			int _Cursor;
			IEnumerator _Enumerator;

			public Enumerator(CommandCollection collection)
			{
				_Commands = collection;
				Reset();
			}

			#region Membres de IEnumerator

			public void Reset()
			{
				_Cursor = 0;
				_Enumerator = _Commands._InnerLists[_Cursor].GetEnumerator();
			}

			public object Current
			{
				get
				{
					return _Enumerator.Current;
				}
			}

			public bool MoveNext()
			{
				bool result = true;
				while(!_Enumerator.MoveNext())
				{
					if(_Cursor < _Commands._InnerLists.Length - 1)
					{
						_Cursor++;
						_Enumerator = _Commands._InnerLists[_Cursor].GetEnumerator();
					}
					else
					{
						result = false;
						break;
					}
				}

				return result;
			}

			#endregion

		}
	}
}
