using System;
using System.Collections;
using System.Collections.Specialized;

namespace Evaluant.Uss.Collections
{
	/// <summary>
	/// Description rsume de HashedArray.
	/// </summary>
	[Serializable]
	public class HashedList : IList
	{
		private ArrayList list = new ArrayList();
		private HybridDictionary index = new HybridDictionary();

		public int Add(object value)
		{
			index.Add(value, null);
			return list.Add(value);
		}

		public void Clear()
		{
			index.Clear();
			list.Clear();
		}

		public bool Contains(object value)
		{
			return index.Contains(value);
		}

		public int IndexOf(object value)
		{
			return list.IndexOf(value);
		}

		public void Insert(int index, object value)
		{
			list.Insert(index, value);
            this.index.Add(value, null);
		}

		public bool IsFixedSize
		{
			get { return list.IsFixedSize; }
		}

		public bool IsReadOnly
		{
			get { return list.IsReadOnly; }
		}

		public void Remove(object value)
		{
			index.Remove(value);
			list.Remove(value);
		}

		public void RemoveAt(int index)
		{
			this.index.Remove(list[index]);
			list.RemoveAt(index);
		}

		public object this[int index]
		{
			get
			{
				return list[index];
			}
			set
			{
				list[index] = value;
			}
		}

		public void CopyTo(Array array, int index)
		{
			list.CopyTo(array, index);
		}

		public int Count
		{
			get { return list.Count; }
		}

		public bool IsSynchronized
		{
			get { return list.IsSynchronized; }
		}

		public object SyncRoot
		{
			get { return list.SyncRoot; }
		}

		public IEnumerator GetEnumerator()
		{
			return list.GetEnumerator();
		}
	}
}
