using System;

namespace Evaluant.Uss
{
	/// <summary>
	/// Description rsume de EntityEntry.
	/// </summary>
	[Serializable]
	public class EntityEntry : ICloneable	
	{
		private string _Name;
 		private Type _Type;
		private object _Value;
		private State _State;
		private Entity _Parent;
		private bool _IsEntity;
		private TypeCode _TypeCode;
 
		public EntityEntry(string name, object value, Type type, State state, Entity parent)
		{
			this._Value = value;
			this._Name = name;
			this._Type = type;
			this._State = state;
			this._Parent = parent;
			this._IsEntity = value is Entity;
			this._TypeCode = Type.GetTypeCode(type);
		}
 
		public string Name
		{
			get	{ return this._Name; }
			set { _Name = value; }
		}
 
		public Type Type
		{
			get	{ return this._Type; }
			set { _Type = value; }
		}
 
		public TypeCode TypeCode
		{
			get	{ return this._TypeCode; }
			set { _TypeCode = value; }
		}
 
		public object Value
		{
			get	{ return this._Value; }
			set 
			{ 
				_Value = value;
				_IsEntity = value is Entity;

				if(_State == State.UpToDate)
					_State = State.Modified;

				if(_Parent.State == State.UpToDate)
					_Parent.State = State.Modified;
			}
		}

		public State State
		{
			get	{ return this._State; }
			set { _State = value; }
		}

		public Entity Parent
		{
			get	{ return this._Parent; }
			set { _Parent = value; }
		}

		public bool IsEntity
		{
			get { return _IsEntity; }
		}

        public override string ToString()
        {
            return String.Concat(_Name, " = ", _Value.ToString());
        }

		#region ICloneable Members

		public object Clone()
		{
			return new EntityEntry(_Name, _Value, _Type, _State, _Parent);
		}

		#endregion
	}
}
