using System;
using System.Collections;
using System.ComponentModel;

using Evaluant.Uss.Common;

namespace Evaluant.Uss
{
	/// <summary>
	/// Description rsume de TypedEntitySet.
	/// </summary>
	public class EntityView : EntitySet, IBindingList, ITypedList
	{
		private EntityView()
		{
		}

		private string _Type;
		public string Type
		{
			get { return _Type; }
			set { _Type = value; }
		}

		public EntityView(string type)
		{
			_Properties = new PropertyDescriptorCollection(null);
			_Type = type;
		}

		protected PropertyDescriptorCollection _Properties;

		public void AddProperty(string name, string path, Type type)
		{
			AddProperty(name, path, type, false);
		}

		public void AddProperty(string name, string path, Type type, bool readOnly)
		{
			_Properties.Add(new EntityDescriptor(name, readOnly, type, path));
		}

		public void AddRelation(string name, string path, EntityView childView)
		{
			_Properties.Add(new ReferenceDescriptor(name, typeof(EntityView), path));
		}
		
		#region IBindingList Members

		void IBindingList.AddIndex(PropertyDescriptor property)
		{
			throw new NotImplementedException();
		}

		bool IBindingList.AllowNew
		{
			get
			{
				return true;
			}
		}

		void IBindingList.ApplySort(PropertyDescriptor property, System.ComponentModel.ListSortDirection direction)
		{
			throw new NotImplementedException();
		}

		PropertyDescriptor IBindingList.SortProperty
		{
			get
			{
				return null;
			}
		}

		int IBindingList.Find(PropertyDescriptor property, object key)
		{
			return 0;
		}

		bool IBindingList.SupportsSorting
		{
			get
			{
				return false;
			}
		}

		bool IBindingList.IsSorted
		{
			get
			{
				return false;
			}
		}

		bool IBindingList.AllowRemove
		{
			get
			{
				return true;
			}
		}

		bool IBindingList.SupportsSearching
		{
			get
			{
				return false;
			}
		}

		System.ComponentModel.ListSortDirection IBindingList.SortDirection
		{
			get
			{
				return new System.ComponentModel.ListSortDirection ();
			}
		}

		public event System.ComponentModel.ListChangedEventHandler ListChanged;

		bool IBindingList.SupportsChangeNotification
		{
			get
			{
				return true;
			}
		}

		void IBindingList.RemoveSort()
		{
			throw new NotImplementedException();
		}

		object IBindingList.AddNew()
		{
			Entity e = new Entity(_Type);
			
			if(ListChanged != null)
				ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemAdded, this.Add(e))); 
			
			return e;
		}

		bool IBindingList.AllowEdit
		{
			get
			{
				return true;
			}
		}

		void IBindingList.RemoveIndex(PropertyDescriptor property)
		{
			throw new NotImplementedException();			
		}

		#endregion

		#region ITypedList Members

		public PropertyDescriptorCollection GetItemProperties(PropertyDescriptor[] listAccessors)
		{

			if (listAccessors == null)
			{
				PropertyDescriptorCollection pdc = new PropertyDescriptorCollection(null);
				foreach (PropertyDescriptor property in _Properties)
				{
					pdc.Add (property);
				}
				return pdc;
			}
			else
			{
				ReferenceDescriptor pd = listAccessors[listAccessors.Length-1] as ReferenceDescriptor;
				
				PropertyDescriptorCollection pdc = new PropertyDescriptorCollection(null);
//				foreach (PropertyDescriptor property in view.ViewModel[pd.ChildViewName].Properties )
//				{
//					pdc.Add (property);
//				}
				return pdc;
			}
		}

		public string GetListName(PropertyDescriptor[] listAccessors)
		{

			if (listAccessors == null)
			{
				return _Type;
			}
			else
			{
				ReferenceDescriptor pd = listAccessors[listAccessors.Length-1] as ReferenceDescriptor;
				return pd.DisplayName;
			}
		}		

		#endregion

		#region EntityDescriptor
		
		private class DescriptorBase : PropertyDescriptor
		{

			public DescriptorBase(string name) : base(name,null)
			{
			}

			public override bool CanResetValue(object component)
			{
				return false;
			}

			public override void ResetValue(object component)
			{

			}

			public override bool ShouldSerializeValue(object component)
			{
				return false;
			}

			public override Type ComponentType
			{
				get
				{
					return typeof(Entity);
				}
			}

			public override object GetValue(object component)
			{
				return null;
			}

			public override void SetValue(object component, object value)
			{
			
			}

			protected bool _IsReadOnly;
			public override bool IsReadOnly
			{
				get
				{
					return _IsReadOnly;
				}
			}

			protected Type _PropertyType;
			public override Type PropertyType
			{
				get
				{
					return _PropertyType;
				}
			}
		}

		private class EntityDescriptor : DescriptorBase
		{
			public EntityDescriptor(string name, bool isReadOnly, Type propertyType, string propertyPath) : base(name)
			{			
				this._IsReadOnly = IsReadOnly;
				this._PropertyType = propertyType;
			}
		}

		private class ReferenceDescriptor : DescriptorBase
		{
			public ReferenceDescriptor(string name, Type propertyType, string propertyPath) : base(name)
			{			
				this._PropertyType = propertyType;
			}
		}

		#endregion
	}
}
