using System;
using System.Collections.Generic;
using System.Text;

using DslModeling = global::Microsoft.VisualStudio.Modeling;
using DslDiagrams = global::Microsoft.VisualStudio.Modeling.Diagrams;

namespace Evaluant.EussDSL
{
    /// <summary>
    /// Redefinition of the serialization of the 1..* link. Instead of using the default behaviour, this one is used in order to
    /// construct a correct euss mapping file during a file-save, and to parse it correctly during a file-load.
    /// </summary>
    public partial class EntityReferencesTargetsSerializer
    {
        /// <summary>
        /// Rien du tout par dfaut dans ce constructeur
        /// </summary>
        private void CustomConstructor()
        {	// Do nothing.
            DefaultConstructor();
        }

        private string CustomXmlTagName
        {
            get { return DefaultXmlTagName; }
        }

        /// <summary>
        /// Lit l'entite serialisee.
        /// Appelle DefaultReadPropertiesFromAttributes pour lire en particulier les attrubuts
        /// Puis lis les elements sous-jacents
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="reader"></param>
        private void CustomRead(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            //DefaultRead(serializationContext, element, reader);

            // Read properties serialized as XML attributes.
            CustomReadPropertiesFromAttributes(serializationContext, element, reader);

            CustomReadTargetRolePlayer(serializationContext, element, reader);

            EntityReferencesTargets instanceOfEntityReferencesTargets = element as EntityReferencesTargets;
            global::System.Diagnostics.Debug.Assert(instanceOfEntityReferencesTargets != null, "Expecting an instance of EntityReferencesTargets");


            // on passe a <Rule>
            DslModeling::SerializationUtilities.SkipToFirstChild(reader);

            // on lit les 3 elements de <rule>
            // parentField
            if (!serializationContext.Result.Failed)
            {
                string attribparentField = reader.GetAttribute("parentField");
                if (attribparentField != null)
                {
                    global::System.String valueOfparentField;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribparentField), out valueOfparentField))
                    {
                        instanceOfEntityReferencesTargets.parentField = valueOfparentField;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "parentField", typeof(global::System.String), attribparentField);
                    }
                }
            }
            // childTable
            if (!serializationContext.Result.Failed)
            {
                string attribchildTable = reader.GetAttribute("childTable");
                if (attribchildTable != null)
                {
                    global::System.String valueOfchildTable;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribchildTable), out valueOfchildTable))
                    {
                        instanceOfEntityReferencesTargets.childTable = valueOfchildTable;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "childTable", typeof(global::System.String), attribchildTable);
                    }
                }
            }
            // childField
            if (!serializationContext.Result.Failed)
            {
                string attribchildField = reader.GetAttribute("childField");
                if (attribchildField != null)
                {
                    global::System.String valueOfchildField;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribchildField), out valueOfchildField))
                    {
                        instanceOfEntityReferencesTargets.childField = valueOfchildField;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "childField", typeof(global::System.String), attribchildField);
                    }
                }
            }

            // parentTable
            if (!serializationContext.Result.Failed)
            {
                string attribparentTable = reader.GetAttribute("parentTable");
                if (attribparentTable != null)
                {
                    global::System.String valueOfparentTable;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribparentTable), out valueOfparentTable))
                    {
                        instanceOfEntityReferencesTargets.parentTable = valueOfparentTable;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "parentTable", typeof(global::System.String), attribparentTable);
                    }
                }
            }

            DslModeling::SerializationUtilities.Skip(reader);
            DslModeling::SerializationUtilities.Skip(reader);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="reader"></param>
        private void CustomReadTargetRolePlayer(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            //DefaultReadTargetRolePlayer(serializationContext, element, reader);

            #region Check Parameters
            global::System.Diagnostics.Debug.Assert(serializationContext != null);
            if (serializationContext == null)
                throw new global::System.ArgumentNullException("serializationContext");
            global::System.Diagnostics.Debug.Assert(element != null);
            if (element == null)
                throw new global::System.ArgumentNullException("element");
            global::System.Diagnostics.Debug.Assert(reader != null);
            if (reader == null)
                throw new global::System.ArgumentNullException("reader");
            #endregion

            // Read the monikerized instance of target role-player Target
            DslModeling::Moniker targetRoleMoniker = null;
            DslModeling::DomainClassXmlSerializer targetRoleSerializer = serializationContext.Directory.GetSerializer(Entity.DomainClassId);
            global::System.Diagnostics.Debug.Assert(targetRoleSerializer != null, "Cannot find serializer for Entity!");

            while (!serializationContext.Result.Failed && !reader.EOF && reader.NodeType == global::System.Xml.XmlNodeType.Element)
            {
                targetRoleMoniker = targetRoleSerializer.TryCreateMonikerInstance(serializationContext, reader, ((EntityReferencesTargets)element).Source, EntityReferencesTargets.DomainClassId, element.Partition);
                if (targetRoleMoniker != null)
                {
                    // Attach the target role-player moniker.
                    DslModeling::DomainRoleInfo.SetRolePlayerMoniker(element as DslModeling::ElementLink, EntityReferencesTargets.TargetDomainRoleId, targetRoleMoniker);
                    // Moniker tag has no child XML elements in it, so just skip to the next element.
                    //DslModeling::SerializationUtilities.Skip(reader);
                    break;
                }
                // Encountered one unknown XML element, skip it and keep reading.
                EussDSLSerializationBehaviorSerializationMessages.UnexpectedXmlElement(serializationContext, reader);
                DslModeling::SerializationUtilities.Skip(reader);
            }
            if (targetRoleMoniker == null)
            {
                EussDSLSerializationBehaviorSerializationMessages.DanglingRelationship(serializationContext, reader, "EntityReferencesTargets");
            }
        }

        /// <summary>
        /// Lit les proprietes de l'entite une par une
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="reader"></param>
        private static void CustomReadPropertiesFromAttributes(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            //DefaultReadPropertiesFromAttributes(serializationContext, element, reader);

            EntityReferencesTargets instanceOfEntityReferencesTargets = element as EntityReferencesTargets;
            global::System.Diagnostics.Debug.Assert(instanceOfEntityReferencesTargets != null, "Expecting an instance of EntityReferencesTargets");

            // pour le nom : rien de spcial
            // name
            if (!serializationContext.Result.Failed)
            {
                string attribname = reader.GetAttribute("name");
                if (attribname != null)
                {
                    global::System.String valueOfname;
                    if (DslModeling::SerializationUtilities.TryGetValue<global::System.String>(DslModeling::SerializationUtilities.UnescapeXmlString(attribname), out valueOfname))
                    {
                        instanceOfEntityReferencesTargets.name = valueOfname;
                    }
                    else
                    {	// Invalid property value, ignored.
                        EussDSLSerializationBehaviorSerializationMessages.IgnoredPropertyValue(serializationContext, reader, "name", typeof(global::System.String), attribname);
                    }
                }
            }
        }

        /// <summary>
        /// Lit les elements sous_jacents de la classe (dans ce cas, rien puisqu'il n'y a aucun lien sous-jacent
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="reader"></param>
        private static void CustomReadElements(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlReader reader)
        {
            DefaultReadElements(serializationContext, element, reader);
        }

        /// <summary>
        /// Essaie de cre une instance de la classe en appelant le constructeur interne qui essaie de crer l'instance
        /// ou de regarder dans les classes hrites
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="reader"></param>
        /// <param name="partition"></param>
        /// <returns></returns>
        private DslModeling::ModelElement CustomTryCreateInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::Partition partition)
        {
            return DefaultTryCreateInstance(serializationContext, reader, partition);
        }

        /// <summary>
        /// Cre effectivement l'instance
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="reader"></param>
        /// <param name="partition"></param>
        /// <returns></returns>
        private DslModeling::ModelElement CustomCreateInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::Partition partition)
        {
            return DefaultCreateInstance(serializationContext, reader, partition);
        }

        /// <summary>
        /// Essaie de crer un guid pour le rfrencement unique
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="reader"></param>
        /// <param name="sourceRolePlayer"></param>
        /// <param name="relDomainClassId"></param>
        /// <param name="partition"></param>
        /// <returns></returns>
        private DslModeling::Moniker CustomTryCreateMonikerInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::ModelElement sourceRolePlayer, global::System.Guid relDomainClassId, DslModeling::Partition partition)
        {
            return DefaultTryCreateMonikerInstance(serializationContext, reader, sourceRolePlayer, relDomainClassId, partition);
        }

        /// <summary>
        /// Cre le moniker
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="reader"></param>
        /// <param name="sourceRolePlayer"></param>
        /// <param name="relDomainClassId"></param>
        /// <param name="partition"></param>
        /// <returns></returns>
        private DslModeling::Moniker CustomCreateMonikerInstance(DslModeling::SerializationContext serializationContext, global::System.Xml.XmlReader reader, DslModeling::ModelElement sourceRolePlayer, global::System.Guid relDomainClassId, DslModeling::Partition partition)
        {
            return DefaultCreateMonikerInstance(serializationContext, reader, sourceRolePlayer, relDomainClassId, partition);
        }

        /// <summary>
        /// Ecrit l'element, ses proprits puis les lments sous-jacents
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="writer"></param>
        /// <param name="rootElementSettings"></param>
        private void CustomWrite(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer, DslModeling::RootElementSettings rootElementSettings)
        {
            //DefaultWrite(serializationContext, element, writer, rootElementSettings);

            #region Check Parameters
            global::System.Diagnostics.Debug.Assert(serializationContext != null);
            if (serializationContext == null)
                throw new global::System.ArgumentNullException("serializationContext");
            global::System.Diagnostics.Debug.Assert(element != null);
            if (element == null)
                throw new global::System.ArgumentNullException("element");
            global::System.Diagnostics.Debug.Assert(writer != null);
            if (writer == null)
                throw new global::System.ArgumentNullException("writer");
            #endregion

            // Write start of element, including schema target namespace if specified.
            if (rootElementSettings != null && !string.IsNullOrEmpty(rootElementSettings.SchemaTargetNamespace))
                writer.WriteStartElement(this.XmlTagName, rootElementSettings.SchemaTargetNamespace);
            else
                writer.WriteStartElement(this.XmlTagName);

            // Write version info (in the format 1.2.3.4), if necessary
            if (rootElementSettings != null && rootElementSettings.Version != null)
                writer.WriteAttributeString("dslVersion", rootElementSettings.Version.ToString(4));

            // on ecrit juste le nom
            CustomWritePropertiesAsAttributes(serializationContext, element, writer);

            // Write the target role-player instance.
            EntityReferencesTargets instance = element as EntityReferencesTargets;
            global::System.Diagnostics.Debug.Assert(instance != null, "Expecting an instance of EntityReferencesTargets!");

            DslModeling::ModelElement targetElement = instance.Target;
            DslModeling::DomainClassXmlSerializer targetSerializer = serializationContext.Directory.GetSerializer(targetElement.GetDomainClass().Id);
            global::System.Diagnostics.Debug.Assert(targetSerializer != null, "Cannot find serializer for " + targetElement.GetDomainClass().Name + "!");
            targetSerializer.WriteMoniker(serializationContext, targetElement, writer, instance.Source, this);

            // maintenant, on ecrit <rule>, parentfield, childtable et childfield
            EntityReferencesTargets instanceOfEntityReferencesTargets = element as EntityReferencesTargets;
            global::System.Diagnostics.Debug.Assert(instanceOfEntityReferencesTargets != null, "Expecting an instance of EntityReferencesTargets");

            writer.WriteStartElement("Rule");
            // parentField
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfEntityReferencesTargets.parentField;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("parentField", propValue);
                }
            }
            // childTable
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfEntityReferencesTargets.childTable;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("childTable", propValue);
                }
            }
            // childField
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfEntityReferencesTargets.childField;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("childField", propValue);
                }
            }

            // parentTable
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfEntityReferencesTargets.parentTable;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("parentTable", propValue);
                }
            }

            writer.WriteEndElement();

            if (!serializationContext.Result.Failed)
            {
                // Write 1) properties serialized as nested XML elements and 2) child model elements into XML.
                DefaultWriteElements(serializationContext, element, writer);
            }

            writer.WriteEndElement();
        }

        /// <summary>
        /// Ecrit les proprits
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="writer"></param>
        private static void CustomWritePropertiesAsAttributes(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer)
        {
            //DefaultWritePropertiesAsAttributes(serializationContext, element, writer);

            EntityReferencesTargets instanceOfEntityReferencesTargets = element as EntityReferencesTargets;
            global::System.Diagnostics.Debug.Assert(instanceOfEntityReferencesTargets != null, "Expecting an instance of EntityReferencesTargets");

            // name
            if (!serializationContext.Result.Failed)
            {
                global::System.String propValue = instanceOfEntityReferencesTargets.name;
                if (!serializationContext.Result.Failed)
                {
                    if (!string.IsNullOrEmpty(propValue))
                        writer.WriteAttributeString("name", propValue);
                }
            }
        }

        /// <summary>
        /// Ecrit les lments sous-jacents
        /// </summary>
        /// <param name="serializationContext"></param>
        /// <param name="element"></param>
        /// <param name="writer"></param>
        private static void CustomWriteElements(DslModeling::SerializationContext serializationContext, DslModeling::ModelElement element, global::System.Xml.XmlWriter writer)
        {
            DefaultWriteElements(serializationContext, element, writer);
        }

        /// <summary>
        /// calcule le moniker (guid) pour l'instance courante
        /// </summary>
        /// <param name="directory"></param>
        /// <param name="element"></param>
        /// <returns></returns>
        private string CustomCalculateQualifiedName(DslModeling::DomainXmlSerializerDirectory directory, DslModeling::ModelElement element)
        {
            return DefaultCalculateQualifiedName(directory, element);
        }

        /// <summary>
        /// Obtient le guid pour l'instance courante
        /// </summary>
        /// <param name="directory"></param>
        /// <param name="element"></param>
        /// <returns></returns>
        private string CustomGetMonikerQualifier(DslModeling::DomainXmlSerializerDirectory directory, DslModeling::ModelElement element)
        {
            return DefaultGetMonikerQualifier(directory, element);
        }
    }
}
