using System;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss;


namespace NUnit.Mapping.AbstractClass
{
	/// <summary>
	/// Description rsume de Proxy.
	/// </summary>
    [TestClass]
	public class Fixture
	{
        protected ObjectContext oc;
        protected Evaluant.Uss.SqlMapper.SqlMapperProvider provider = new Evaluant.Uss.SqlMapper.SqlMapperProvider();

        public Fixture()
        {

            provider.ConnectionString = "Server=EUSS;DataBase=uss;Integrated Security=true";
            provider.Dialect = "Evaluant.Uss.SqlMapper.MsSqlDialect";
            provider.Driver = "Evaluant.Uss.SqlMapper.MsSqlDriver";
            provider.TablePrefix = "ABS_";
            provider.RegisterMetaData(MetaDataFactory.FromAssembly(typeof(Inhabitant).Assembly));

            Configure();

            provider.InitializeConfiguration();

            oc = new ObjectService(provider).CreateObjectContext();
        }

        public virtual void Configure()
        {
            provider.MappingFileName = @"..\..\..\NUnit\Mapping\AbstractClass\mapping_TPCC.xml";
        }

        [TestMethod]
        public void ShouldSerializeInheritedEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            h.Address = "1 Microsoft Way";

            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();
        }

        [TestMethod]
        public void ShouldLoadInheritedEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            h.Address = "1 Microsoft Way";

            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(1, oc.Load<Dog>(typeof(Dog), "[Name = 'Rox']").Count);
            Assert.AreEqual(1, oc.Load<Human>(typeof(Human)).Count);

            Assert.AreEqual(2, oc.Load<Inhabitant>(typeof(Inhabitant)).Count);

            Inhabitant inhabitant;

            inhabitant = oc.LoadSingle<Inhabitant>(typeof(Inhabitant), "[Name = 'Erik']");
            Assert.IsTrue(inhabitant is Human);
            Assert.AreEqual("Erik", inhabitant.Name);
            Assert.AreEqual("Smith", ((Human)inhabitant).LastName);
            Assert.AreEqual("1 Microsoft Way", ((Human)inhabitant).Address);

            inhabitant = oc.LoadSingle<Inhabitant>(typeof(Inhabitant), "[Name = 'Rox']");
            Assert.IsTrue(inhabitant is Dog);
            Assert.AreEqual("Rox", inhabitant.Name);
            Assert.AreEqual("Fox", ((Dog)inhabitant).Kind);
        }

        [TestMethod]
        public void ShouldCountEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            h.Address = "1 Microsoft Way";

            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.LoadScalar("count(NUnit:Mapping:AbstractClass:Inhabitant)"));
        }

        [TestMethod]
        public void ShouldPaginateEntities()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            h.Address = "1 Microsoft Way";

            oc.BeginTransaction();
            oc.Serialize(h);
            oc.Serialize(d);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(1, oc.Load<Inhabitant>(new Query(typeof(Inhabitant)) { MaxResults = 1 }).Count);
        }


        [TestMethod]
        public void ShouldLoadImplementedObjects()
        {
            oc.InitializeRepository();

            Dog d = new Dog();
            d.Kind = "Fox";
            d.Name = "Rox";

            Human h = new Human();
            h.Name = "Erik";
            h.LastName = "Smith";
            h.Address = "1 Microsoft Way";

            Zoo z = new Zoo();
            z.Inhabitants.Add(d);
            z.Inhabitants.Add(h);

            oc.BeginTransaction();
            oc.Serialize(z);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(2, oc.Load<Inhabitant>(typeof(Inhabitant)).Count);
            Assert.AreEqual(1, oc.Load<Human>(typeof(Human)).Count);
            Assert.AreEqual(1, oc.Load<Dog>(typeof(Dog)).Count);

            IList<Inhabitant> inhabitants = oc.Load<Inhabitant>(typeof(Zoo), ".Inhabitants");
            Assert.AreEqual(2, inhabitants.Count);
        }
    }
}
