using System;
using System.Collections.Generic;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss;


namespace NUnit.Mapping.Domain
{
	/// <summary>
	/// Description rsume de Proxy.
	/// </summary>
	[TestClass]
	public class Fixture
	{
        protected ObjectContext oc;

        public Fixture()
        {
            Evaluant.Uss.SqlMapper.SqlMapperProvider provider = new Evaluant.Uss.SqlMapper.SqlMapperProvider();
            
            provider.ConnectionString = @"Data Source=.;Initial Catalog=uss;Integrated Security=True";
            provider.Dialect = "Evaluant.Uss.SqlMapper.MsSqlDialect";
            provider.Driver = "Evaluant.Uss.SqlMapper.MsSqlDriver";
            provider.MappingFileName = @"..\..\..\NUnit\Mapping\Domain\mapping_4.xml";
            provider.TablePrefix = "OTH_";
            provider.RegisterMetaData(MetaDataFactory.FromAssembly(typeof(Person).Assembly));

            provider.InitializeConfiguration();

            oc = new ObjectService(provider).CreateObjectContext();
        }

        [TestMethod]
        public void ShouldSerializeInheritedEntities()
        {
            oc.InitializeRepository();

            Person p = new Person();
            p.FirstName = "pfs";
            p.LastName = "pls";

            Employee e = new Employee();
            e.FirstName = "efs";
            e.LastName = "els";
            e.Salary = 10;

            Consultant c = new Consultant();
            c.FirstName = "cfs";
            c.LastName = "cls";
            c.Company = "Evaluant";

            oc.BeginTransaction();
            oc.Serialize(p);
            oc.Serialize(e);
            oc.Serialize(c);
            oc.CommitTransaction();

        }

        [TestMethod]
        public void ShouldLoadInheritedEntities()
        {
            oc.InitializeRepository();

            Person p = new Person();
            p.FirstName = "pfs";
            p.LastName = "pls";

            Employee e = new Employee();
            e.FirstName = "efs";
            e.LastName = "els";
            e.Salary = 10;

            Consultant c = new Consultant();
            c.FirstName = "cfs";
            c.LastName = "cls";
            c.Company = "Evaluant";

            oc.BeginTransaction();
            oc.Serialize(p);
            oc.Serialize(e);
            oc.Serialize(c);
            oc.CommitTransaction();

            oc.Clear();

            Assert.AreEqual(3, oc.Load<Person>(typeof(Person)).Count);

        }

        [TestMethod]
        public virtual void KeyMultiple()
        {
            oc.InitializeRepository();

            Vehicule v1 = new Vehicule();
            v1.VehiculeId1 = "AAA"; //Guid.NewGuid().ToString();
            v1.VehiculeId2 = "BBB"; //Guid.NewGuid().ToString();

            oc.BeginTransaction();
            oc.Serialize(v1);
            oc.CommitTransaction();

            oc.Clear();
            v1 = oc.LoadSingle<Vehicule>(string.Format("[VehiculeId1 = 'AAA']"));
            Assert.IsNotNull(v1);

            Passager p1 = new Passager();
            p1.Name = "p1";

            v1.Passagers.Add(p1);

            oc.BeginTransaction();
            oc.Serialize(v1);
            oc.CommitTransaction();

            oc.Clear();
            v1 = oc.LoadSingle<Vehicule>(string.Format("[VehiculeId1 = 'AAA']"));
            Assert.IsNotNull(v1);

            Assert.AreEqual(1, v1.Passagers.Count);
          

           



        }
    }
}
