using System;
using NUnit.Framework;
using Evaluant.Uss;

namespace TestUSS
{
	/// <summary>
	/// Description rsume de CommonSqlMapperAssignedId.
	/// </summary>
	[TestFixture]
	public abstract class CommonSqlMapperAssignedId : CommonSqlMapperGeneratorId
	{
		[Test]
		public override void BackRef()
		{
			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			p1.AddValue("Partners", p2);

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			Assert.AreEqual(1, pe.Load(String.Concat("Person[ count(Partners[Id = '", p2.Id, "']) > 0]")).Count);
			Assert.AreEqual("p1", pe.Load(String.Concat("Person[ count(Partners[Id = '", p2.Id, "']) > 0]"))[0]["Firstname"]);
		}

		[Test]
		public override void BusinessId()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "Firstname1");
			p1.Id = "1";

			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Firstname2");
			p2.Id = "2";

			Entity p3 = new Entity("Person");
			p3.AddValue("Firstname", "Firstname3");
			p3.Id = "3";

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			Assert.IsNotNull(pe.LoadWithId("Person", "1"));
			Assert.IsNotNull(pe.LoadWithId("Person", "2"));
			Assert.IsNotNull(pe.LoadWithId("Person", "3"));

			Assert.AreEqual("Firstname1", pe.LoadWithId("Person", "1")["Firstname"]);
			Assert.AreEqual("Firstname2", pe.LoadWithId("Person", "2")["Firstname"]);
			Assert.AreEqual("Firstname3", pe.LoadWithId("Person", "3")["Firstname"]);

			Assert.AreEqual("Firstname1", pe.Load("Person[Id = " + p1.Id + "]")[0]["Firstname"]);

		}

		[Test]
		public override void DeleteAttribute()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			Assert.AreEqual(1, pe.Load("Person[Firstname = 'p1']").Count);

			p1["Firstname"] = null;
			
			t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			
			Assert.AreEqual(0, pe.Load("Person[Firstname = 'p1']").Count);
		}

		[Test]
		public override void DeleteEntity()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "p1");
			p1.Id = "1";

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			t = new Transaction();
			t.Delete(p1);
			t.Commit(pe);

			pe.Initialize();
			
			Assert.AreEqual(0, pe.Load("Person").Count);
		}



		[Test]
		public override void Simple()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "Thomas");
			p1.AddValue("Lastname", "Gil");
			p1.AddValue("Age", 27);

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "Sbastien");
			p2.AddValue("Lastname", "Ros");
			p2.AddValue("Age", 27);

			p1.AddValue("Partners", p2);
			p2.AddValue("Partners", p1); // To test cyclic references

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			p1.DeleteElement("Firstname");

			t = new Transaction();
			t.Serialize(p1);

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "Nicolas");
			p3.AddValue("Lastname", "Chabrier");
			p3.AddValue("Age", 27);
			p3.AddValue("Prout", p2);

			t.Serialize(p3);
			t.Commit(pe);

			t = new Transaction();
			t.Delete(p1);
			t.Serialize(p3);
			t.Commit(pe);
		}

		[Test]
		public override void Create()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);
		}

		[Test]
		public override void Update()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			p1["Firstname"] = "new_name";

			Entity p4 = new Entity("Person");
			p4.Id = "4";
			p4["Firstname"] = "p4";
			p1.AddValue("Brother", p4);

			t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			EntitySet e = pe.Load("Person[Firstname = 'new_name']");
			Assert.AreEqual(1, e.Count, "Error 'Person[Firstname = new_name]'");

			pe.LoadReference(e[0], new string[]{ "Brother" });
			Assert.IsNotNull(e[0].GetEntity("Brother"), "Reference 'Brother' not found");
		}

		[Test]
		public override void LoadReference()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			Entity p = pe.Load("Person[Firstname = 'p1']")[0];
			pe.LoadReference(p, new string[]{ "Partners" });

			Assert.AreEqual(State.UpToDate, p.State);
			foreach(Entity e in p.GetEntitySet("Partners"))
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			Assert.AreEqual(2, p.GetEntitySet("Partners").Count, "Reference 'Partners' not found");

			Entity s1 = new Entity("Company");
			s1.Id = "11";
			s1.AddValue("Name","evaluant");

			Entity s2 = new Entity("Company");
			s2.Id = "12";
			s2.AddValue("Name","microsoft");

			p1.AddValue("MyCompany", s1);

			p2.AddValue("MyCompany", s1);
			p2.AddValue("MyCompany", s2);

			p3.AddValue("MyCompany", s1);

			t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			p = pe.Load("Person[Firstname = 'p1']")[0];
			pe.LoadReference(p, new string[]{ "MyCompany" });

			Assert.AreEqual(State.UpToDate, p.State);
			foreach(EntityEntry e in p)
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			foreach(Entity e in p.GetEntitySet("MyCompany"))
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			Assert.AreEqual(1, p.GetEntitySet("MyCompany").Count, "Reference 'MyCompany' not found");

			p = pe.Load("Person[Firstname = 'p2']")[0];
			pe.LoadReference(p, new string[]{ "MyCompany" });

			Assert.AreEqual(State.UpToDate, p.State);
			foreach(EntityEntry e in p)
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			foreach(Entity e in p.GetEntitySet("MyCompany"))
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			Assert.AreEqual(2, p.GetEntitySet("MyCompany").Count, "Reference 'MyCompany' not found");

			p = pe.Load("Person[Firstname = 'p3']")[0];
			pe.LoadReference(p, new string[]{ "MyCompany" });

			Assert.AreEqual(State.UpToDate, p.State);
			foreach(EntityEntry e in p)
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			foreach(Entity e in p.GetEntitySet("MyCompany"))
			{
				Assert.AreEqual(State.UpToDate, e.State);
			}
			Assert.AreEqual(1, p.GetEntitySet("MyCompany").Count, "Reference 'MyCompany' not found");
		}

		[Test]
		public override void Delete()
		{

			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			
			// Removes an attribute
			p2["Firstname"] = null;
			p1.AddValue("Lastname", "TestValue");

			t = new Transaction();
			t.Delete(p3);
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();

			Assert.AreEqual(2, pe.Load("Person").Count);
			Assert.AreEqual("TestValue", pe.Load("Person[Firstname = 'p1']")[0]["Lastname"]);
			
			// Check the attribute "Name" is deleted from p2
			Assert.IsNull(p2["Firstname"]);

			p1.DeleteElement("Lastname");
			
			t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			// Check the attribute "Test" is deleted from p1
			Assert.IsNull(pe.Load("Person[Firstname = 'p1']")[0]["Lastname"]);
		}

		[Test]
		public override void ManyToMany()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "Thomas");

			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "Sbastien");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "Nicolas");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			EntitySet people = pe.Load("Person");

			Assert.AreEqual(people.Count, 3);
		}

		[Test]
		public virtual void LazyLoading()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "Thomas");
			p1.Id = "1";

			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Sbastien");
			p2.Id = "2";

			Entity p3 = new Entity("Person");
			p3.AddValue("Firstname", "Nicolas");
			p3.Id = "3";

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);

			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			EntitySet people = pe.Load("Person");
			pe.LoadReference(people, new string[] { "Partners" });
		}

		[Test]
		public override void OPath()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.Id = "1";
			p1.AddValue("Firstname", "p1");
			
			Entity p2 = new Entity("Person");
			p2.Id = "2";
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.Id = "3";
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			// A constraint can't begin with a relation's name: must be attribute or commmand. Ex: Person[Partners] is not allowed
			// Person.Firstname can't be used in a constraint, but Person[Firstname]

			// TODO : if a constraint starts with a Path, include it in an exists() clause
			
			// id() management
			EntitySet coll = pe.Load("Person[Id = " + p1.Id + "].Partners");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + "].Partners'");
			Assert.IsTrue(Contains(coll, p2));
			Assert.IsTrue(Contains(coll, p3));

			coll = pe.Load("Person[Id = " + p1.Id + "].Partners.Partners");
			Assert.AreEqual(1, coll.Count, "Error 'Person[Id = " + p1.Id + "].Partners.Partners'");
			Assert.IsTrue(Contains(coll, p3));

			coll = pe.Load("Person[exists(Partners[Id = " + p2.Id + "])]");
			Assert.AreEqual(1, coll.Count, "Error 'Person[exists(Partners[Id = " + p2.Id + "])]'");
			Assert.IsTrue(Contains(coll, p1));

			coll = pe.Load("Person[Id = " + p1.Id + " or Id = " + p2.Id + "]");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + " or Id = " + p2.Id + "]'");
			Assert.IsTrue(Contains(coll, p1));
			Assert.IsTrue(Contains(coll, p2));

			Assert.AreEqual(0, pe.Load("Person[Id = " + p1.Id + " and Id = " + p2.Id + "]").Count, "Error 'Person[Id = " + p1.Id + " and Id = " + p2.Id + "]'");
			
			coll = pe.Load("Person[Id = " + p1.Id + " or exists(Partners[Id = " + p3.Id + "])]");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + " or exists(Partners[Id = " + p3.Id + "])]'");
			Assert.IsTrue(Contains(coll, p1));
			Assert.IsTrue(Contains(coll, p2));
		}

		public override void Inheritance()
		{
			pe.InitializeRepository();

			Entity bill = new Entity("Employee");
			bill.AddValue("Firstname", "Bill Gates");
			bill.AddValue("Salary", 99999);
			bill.Id = "1";

			Entity redmond = new Entity("Address");
			redmond.AddValue("City", "Redmond");
			redmond.Id = "1";
			bill.AddValue("Address", redmond);

			Entity paloalto = new Entity("Address");
			paloalto.AddValue("City", "Palo Alto");
			paloalto.Id = "2";

			Entity steve = new Entity("Person");
			steve.AddValue("Firstname", "Steve Jobs");
			steve.AddValue("Address", paloalto);
			steve.Id = "2";

			bill.AddValue("Partners", steve);
            
			Transaction t = new Transaction();
			t.Serialize(bill);
			t.Commit(pe);

			Assert.AreEqual(2, pe.Load("Person").Count);
			Assert.AreEqual(2, pe.Load("Address").Count);

			bill = pe.Load("Employee[Firstname = 'Bill Gates']")[0];
			Assert.IsNotNull(bill);
			Assert.AreEqual(99999, bill["Salary"]);

			pe.LoadReference(bill, new string[] { "Partners", "Address" });

			Assert.AreEqual("Steve Jobs", bill.GetEntitySet("Partners")[0]["Firstname"]);
			Assert.AreEqual("Redmond", bill.GetEntity("Address")["City"]);
		}

		public override void Composition()
		{
			pe.InitializeRepository();

			Entity paloalto = new Entity("Address");
			paloalto.AddValue("City", "Palo Alto");
			paloalto.Id = "1";

			Entity steve = new Entity("Person");
			steve.AddValue("Firstname", "Steve Jobs");
			steve.AddValue("Address", paloalto);
			steve.Id = "1";

			Transaction t = new Transaction();
			t.Serialize(steve);
			t.Commit(pe);

			Entity p = (Entity) pe.Load("Person")[0];

			Assert.AreEqual(1, pe.Load("Person").Count);
			Assert.AreEqual(1, pe.Load("Address").Count);

			EntitySet es = pe.Load("Person.Address");

			t = new Transaction();
			t.Delete(steve);
			t.Commit(pe);

			Assert.AreEqual(0, pe.Load("Person").Count);
			Assert.AreEqual(0, pe.Load("Address").Count);
		}

		public override void LoadReference2()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "p1");
			p1.Id = "1";

			Entity s1 = new Entity("Company");
			s1.AddValue("Name","evaluant");
			s1.Id = "1";

			Entity s2 = new Entity("Company");
			s2.AddValue("Name","toto");
			s2.Id = "2";

			p1.AddValue("MyCompany", s1);
			p1.AddValue("MyCompany", s2);

			Transaction t = new Transaction();
			t.Serialize(s1);
			t.Serialize(s2);
			t.Serialize(p1);
			t.Commit(pe);

			pe.Initialize();
			Entity p = pe.Load("Person[count(MyCompany[id('"+s1.Id+"')]) > 0]")[0];
			pe.LoadReference(p, new string[] { "MyCompany" });
		}

		
	}
}
