using NUnit.Framework;
using Evaluant.Uss;

namespace TestUSS
{
	/// <summary>
	/// Description rsume de TestSqlMapperGeneratorId.
	/// </summary>
	[TestFixture]
	public abstract class CommonSqlMapperGeneratorId : Common
	{
		[Test]
		public override void OPath()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "p1");
			
			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "p2");

			Entity p3 = new Entity("Person");
			p3.AddValue("Firstname", "p3");

			p1.AddValue("Partners", p2);
			p1.AddValue("Partners", p3);
			p2.AddValue("Partners", p3);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Commit(pe);

			// A constraint can't begin with a relation's name: must be attribute or commmand. Ex: Person[Partners] is not allowed
			// Person.Firstname can't be used in a constraint, but Person[Firstname]

			// TODO : if a constraint starts with a Path, include it in an exists() clause
			
			// id() management
			EntitySet coll = pe.Load("Person[Id = " + p1.Id + "].Partners");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + "].Partners'");
			Assert.IsTrue(Contains(coll, p2));
			Assert.IsTrue(Contains(coll, p3));

			coll = pe.Load("Person[Id = " + p1.Id + "].Partners.Partners");
			Assert.AreEqual(1, coll.Count, "Error 'Person[Id = " + p1.Id + "].Partners.Partners'");
			Assert.IsTrue(Contains(coll, p3));

			coll = pe.Load("Person[exists(Partners[Id = " + p2.Id + "])]");
			Assert.AreEqual(1, coll.Count, "Error 'Person[exists(Partners[Id = " + p2.Id + "])]'");
			Assert.IsTrue(Contains(coll, p1));

			coll = pe.Load("Person[Id = " + p1.Id + " or Id = " + p2.Id + "]");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + " or Id = " + p2.Id + "]'");
			Assert.IsTrue(Contains(coll, p1));
			Assert.IsTrue(Contains(coll, p2));

			Assert.AreEqual(0, pe.Load("Person[Id = " + p1.Id + " and Id = " + p2.Id + "]").Count, "Error 'Person[Id = " + p1.Id + " and Id = " + p2.Id + "]'");
			
			coll = pe.Load("Person[Id = " + p1.Id + " or exists(Partners[Id = " + p3.Id + "])]");
			Assert.AreEqual(2, coll.Count, "Error 'Person[Id = " + p1.Id + " or exists(Partners[Id = " + p3.Id + "])]'");
			Assert.IsTrue(Contains(coll, p1));
			Assert.IsTrue(Contains(coll, p2));
		}

		#region	All method ignore in the Generator Id Test.

		public override void BusinessId()
		{
			base.BusinessId ();
		}

		public override void ValueTypes()
		{
			base.ValueTypes();
		}

		public override void SerializableTypes()
		{
			base.SerializableTypes();
		}

		public override void ComplexTypes()
		{
			base.ComplexTypes();
		}

		public override void References()
		{
			base.References();
		}

		public override void References2()
		{
			
			base.References2();
		}

		public override void LoadState()
		{
			base.LoadState();
		}

		public override void NullArgument()
		{
			base.NullArgument ();
		}
		#endregion
	}
}
