using System;
using System.Xml.Serialization;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;

using NUnit.Framework;
using Evaluant.Uss;
using Evaluant.Uss.MetaData;
using Evaluant.Uss.Common;

namespace NUnit.PersistenceEngine
{
	[TestFixture]
	public class Internals
	{

		[Test]
		public void GetFullName()
		{
			// Checks a System type is serialized with its fullname
			Assert.AreEqual("System.String", Utils.GetFullName(typeof(string)));
			Assert.AreEqual(typeof(string), Utils.GetType("System.String"));
			Assert.AreEqual(typeof(System.Collections.Specialized.StringCollection), Utils.GetType("System.Collections.Specialized.StringCollection, System"));

			// Checks a Business type is serialized with its Assembly "short" name
			Assert.AreEqual("Evaluant.Uss.State, Evaluant.Uss", Utils.GetFullName(typeof(State)));
			Assert.AreEqual(typeof(State), Utils.GetType("Evaluant.Uss.State, Evaluant.Uss"));
		}

		[Test]
		public void SerializeEntity()
		{
			Entity p1 = new Entity("Person");
			p1.AddValue("Firstname", "Sbastien");
			p1.AddValue("Age", 28);
			
			Entity p2 = new Entity("Person");
			p2.AddValue("Firstname", "Fabien");
			
			p1.AddValue("Colleague", p2);

			MemoryStream ms = new MemoryStream();
			
			BinaryFormatter bf = new BinaryFormatter();
			bf.Serialize(ms, p1);
			
			ms.Seek(0, SeekOrigin.Begin);
			p1 = bf.Deserialize(ms) as Entity;

			Assert.IsNotNull(p1);
		}

		[Test]
		public void SerializeMetaData()
		{
			PropertyMetaData pm = new PropertyMetaData("Person", "Age", typeof(int));

			MemoryStream ms = new MemoryStream();
			
			BinaryFormatter bf = new BinaryFormatter();
			bf.Serialize(ms, pm);
			
			ms.Seek(0, SeekOrigin.Begin);
			pm = bf.Deserialize(ms) as PropertyMetaData;

			Assert.IsNotNull(pm);
			Assert.AreEqual(typeof(int), pm.PropertyType);
		}
	}
}
