using System;
using System.Collections.Specialized;
using NUnit.Framework;
using UniversalStorageServices;
using UniversalStorageServices.Commands;

namespace TestUSS
{
	/// <summary>
	/// Description rsume de Common.
	/// </summary>
	[TestFixture]
	public class MappingInheritance
	{
		protected IPersistenceEngine pe;
		protected IPersistenceEngineFactory pef ;
			
		public MappingInheritance()
		{
			CreatePersistenceEngine();
			pef.RegisterMetaData(MetaDataFactory.FromMetaDataFile(@"..\..\PersistenceEngine\Models\MappingInheritance.xml"));
			pe = pef.CreatePersistenceEngine();
		}

		[SetUp]
		public void SetUp()
		{
			pe.InitializeRepository();
		}

		protected virtual void CreatePersistenceEngine()
		{
			pef = XmlConfigLoader.LoadXmlConfig("engines.config", "TablePerHierarchy");
		}

		[Test]
		public void Simple()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Person");
			p1.AddValue("Name", "Robert King");
			p1.AddValue("Age", 28);

			Entity p2 = new Entity("Employee");
			p2.AddValue("Name", "John Smith");
			p2.AddValue("Age", 20);
			p2.AddValue("Salary", 200);

			Entity p3 = new Entity("Worker");
			p3.AddValue("Name", "Sven Ottlieb");
			p3.AddValue("Age", 25);
            p3.AddValue("Salary", 500);
			p3.AddValue("WorkingPlace", "London");

			Entity p4 = new Entity("Secretary");
			p4.AddValue("Name", "Pedro Afonso");
			p4.AddValue("Age", 25);
			p4.AddValue("Salary", 500);
			p4.AddValue("Bonus", 100);
			
			Transaction t = new Transaction();
			t.Serialize(p1);
			t.Serialize(p2);
			t.Serialize(p3);
			t.Serialize(p4);
			t.Commit(pe);

			EntitySet es = pe.Load("Person");
			
			Assert.AreEqual(4, es.Count);
			
			es = pe.Load("Person[Name = 'Sven Ottlieb']");

			Assert.AreEqual(1, es.Count);
		}

		[Test]
		public void ReferencesBases()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Employee");
			p1.AddValue("Name", "Gil");
			p1.AddValue("Age", 28);

			Entity p2 = new Entity("Person");
			p2.AddValue("Name", "Ros");
			p2.AddValue("Age", 27);

			Entity c1 = new Entity("Company");
			c1.AddValue("Name", "Alfreds Futterkiste");

			p1.AddValue("Partners", p2);
			p2.AddValue("Partners", p1);

			c1.AddValue("Employees", p1);

			Transaction t = new Transaction();
			t.Serialize(c1);
			t.Commit(pe);

			Entity company = (Entity) pe.Load("Company[Name = 'Alfreds Futterkiste']")[0];

			Assert.IsNotNull(company);
			Assert.AreEqual("Alfreds Futterkiste", company["Name"]);

			pe.LoadReference(company, new string[]{ "Employees" });

			Entity employee = company.GetEntitySet("Employees")[0];
			
			Assert.IsNotNull(employee);
			Assert.AreEqual("Gil", employee["Name"]);
			Assert.AreEqual(28, employee["Age"]);

			pe.LoadReference(employee, new string[]{ "Partners" });

			Entity partner = employee.GetEntitySet("Partners")[0];

			Assert.IsNotNull(partner);
			Assert.AreEqual("Ros", partner["Name"]);
			Assert.AreEqual(27, partner["Age"]);
		}

		[Test]
		public void InheritedReferences()
		{
			pe.InitializeRepository();

			Entity p1 = new Entity("Worker");
			p1.AddValue("Name", "Sven Ottlieb");
			p1.AddValue("Age", 25);
			p1.AddValue("Salary", 500);
			p1.AddValue("WorkingPlace", "London");

			Entity c1 = new Entity ("Company");
			c1.AddValue("Name", "Ernst Handel");
			c1.AddValue("Employees", p1);

			Transaction t = new Transaction();
			t.Serialize(c1);
			t.Commit(pe);
		}
	}
}
