using System;
using System.Collections;
using System.Runtime.Serialization.Formatters.Soap;
using System.IO;

using NUnit.Framework;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss;
using Evaluant.Uss.ObjectContext.Descriptors;

using NUnit.DynamicProxy;

namespace NUnit.PersistenceManager
{
	/// <summary>
	/// Description rsume de TestNavigationComparer.
	/// </summary>
	[TestFixture]
	public class TestNavigation
	{

		public Evaluant.Uss.ObjectContext.ObjectContext pm;

		public TestNavigation()
		{
			Configure();
		}

		public virtual void Configure()
		{
			ObjectService factory = new ObjectService("engines.config", "Xml");
			factory.PersistentDescriptor = new ReflectionDescriptor();
			factory.AddAssembly("NUnit");
			factory.RegisterMetaData(MetaDataFactory.FromMetaDataFile(@"..\..\PersistenceEngine\Models\Person.xml"));
			factory.RegisterMetaData(MetaDataFactory.FromPropertyDescriptor(typeof(Person), factory.PersistentDescriptor, false));
			this.pm = factory.CreateObjectContext();
		}

		[Test]
		public virtual void NavigationComparer()
		{
			pm.InitializeRepository();

			Person p1 = new Person();
			p1.PersonId = "1";
			p1.Name = "Sebastien";
			p1.Birth = DateTime.Today;

			Person p2 = new Person();
			p2.PersonId = "2";
			p2.Name = "Carole";
			p2.Birth = DateTime.Today.AddDays(1);

			Person p3 = new Person();
			p3.PersonId = "3";
			p3.Name = "Nicolas";
			p3.Birth = DateTime.Today;

			Person p4 = new Person();
			p4.PersonId = "4";
			p4.Name = "Carole";
			p4.Birth = DateTime.Today.AddDays(1);

			p4.Partners.Add(p3);
			p1.Partners.Add(p2);
			p3.Partners.Add(p2);

			pm.BeginTransaction();
			pm.Serialize(p1);
			pm.Serialize(p2);
			pm.Serialize(p3);
			pm.Serialize(p4);
			pm.CommitTransaction();

			IList coll = pm.Load(typeof(Person));
			IPersistable[] orderedColl = new IPersistable[coll.Count];
			coll.CopyTo(orderedColl, 0);
   
			Evaluant.Uss.ObjectContext.Navigator.NavigationComparer comparer 
										= new Evaluant.Uss.ObjectContext.Navigator.NavigationComparer("Name, Birth, PersonId desc");
			Array.Sort(orderedColl, comparer);

			Assert.AreEqual("4", ((Person)orderedColl[0]).PersonId);
			Assert.AreEqual("2", ((Person)orderedColl[1]).PersonId);
			Assert.AreEqual("3", ((Person)orderedColl[2]).PersonId);
			Assert.AreEqual("1", ((Person)orderedColl[3]).PersonId);

			comparer = new Evaluant.Uss.ObjectContext.Navigator.NavigationComparer("Name desc, PersonId");
			Array.Sort(orderedColl, comparer);

			Assert.AreEqual("1", ((Person)orderedColl[0]).PersonId);
			Assert.AreEqual("3", ((Person)orderedColl[1]).PersonId);
			Assert.AreEqual("2", ((Person)orderedColl[2]).PersonId);
			Assert.AreEqual("4", ((Person)orderedColl[3]).PersonId);

			comparer = new Evaluant.Uss.ObjectContext.Navigator.NavigationComparer("Partners.Name desc, PersonId");
			Array.Sort(orderedColl, comparer);

			Assert.AreEqual("4", ((Person)orderedColl[0]).PersonId);
			Assert.AreEqual("1", ((Person)orderedColl[1]).PersonId);
			Assert.AreEqual("3", ((Person)orderedColl[2]).PersonId);
			Assert.AreEqual("2", ((Person)orderedColl[3]).PersonId);
		}

		[Test]
		public virtual void NavRelationToOne()
		{
			pm.InitializeRepository();

			Country fr = new Country("France");
			Country us = new Country("USA");

			Person p1 = new Person();
			p1.Name = "Robert";
			p1.Address = new Address("Paris");
			p1.Address.Country = fr;

			Person p2 = new Person();
			p2.Name = "John";
			p2.Address = new Address("New-York");
			p2.Address.Country = us;

			Person p3 = new Person();
			p3.Name = "Marcel";
			p3.Address = new Address("Paris");
			p3.Address.Country = fr;

			Person p4 = new Person();
			p4.Name = "Sims";
			p4.Address = new Address("SimsCity");

			Person p5 = new Person();
			p5.Name = "Sdf";

			ArrayList personColl = new ArrayList();
			personColl.Add(p1);
			personColl.Add(p2);
			personColl.Add(p3);
			personColl.Add(p4);
			personColl.Add(p5);

			Evaluant.Uss.ObjectContext.Navigator.Navigator nav = new Evaluant.Uss.ObjectContext.Navigator.Navigator(personColl);

			object[] res = nav.SelectElements("this[exists(Address)]");
			Assert.AreEqual(4, res.Length);

			res = nav.SelectElements("this.Address");
			Assert.AreEqual(4, res.Length);

			res = nav.SelectElements("this.Address.Country");
			Assert.AreEqual(2, res.Length);

			res = nav.SelectElements("this.Address[City = 'Paris']");
			Assert.AreEqual(2, res.Length);

			res = nav.SelectElements("this.Address.Country[Name = 'USA']");
			Assert.AreEqual(1, res.Length);
			Assert.AreEqual("USA", ((Country)res[0]).Name);

			res = nav.SelectElements("this[Name = 'Sdf'].Address");
			Assert.AreEqual(0, res.Length);
		}
	}
}
