using System;
using System.IO;
using System.Collections;
using System.Reflection;
using System.Xml;
using System.Xml.Xsl;
using System.Text;

using Microsoft.CSharp;
using System.CodeDom.Compiler;

using Microsoft.VisualStudio.TestTools.UnitTesting;

using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;
using Evaluant.Uss.Xmi;

namespace NUnit.PersistenceManagerGenerator
{
	/// <summary>
	/// Description rsume de Poseidon.
	/// </summary>
	/// 
	[TestClass]
	public class CommonXMI
	{
		protected string _Code;
		protected Assembly _Assembly;

		protected StreamReader xmiReader;

		protected void GenerateCode()
		{
			string content = xmiReader.ReadToEnd();

			XmiVersion ver = Converter.GetVersion(content);
			Exporter exp = Converter.GetExporter(content);

			string simple = Converter.ConvertToSimple(content, ver, exp);

			XmlDocument xmlDoc = new XmlDocument();
			xmlDoc.LoadXml(simple);

			XmlDocument xslDoc = new XmlDocument();
			xslDoc.Load("..\\..\\..\\PersistenceLayerGen\\xsl\\AssociationClassConverter.xslt");

			StringBuilder sb = new StringBuilder();
			XslTransform xslt = new XslTransform();
			xslt.Load(xslDoc, null, null);
			xslt.Transform(xmlDoc, null,  new StringWriter(sb), null);

			XmlDocument xmlResult = new XmlDocument();
			xmlResult.LoadXml(sb.ToString());

			if (System.Environment.Version.ToString().StartsWith("1.1"))
				xslDoc.Load("..\\..\\..\\PersistenceLayerGen\\xsl\\CSPersistenceManagerv11.xslt");
			else
				xslDoc.Load("..\\..\\..\\PersistenceLayerGen\\xsl\\CSPersistenceManagerv20.xslt");


			sb = new StringBuilder();

			xslt.Load(xslDoc, null, null);
			xslt.Transform(xmlResult, null, new StringWriter(sb), null);

			_Code = sb.ToString();
		}

		private static int count = 0;
		public void CompileCode()
		{
			if (_Code == null || _Code == string.Empty)
				return;

			// The C# code holder
			CSharpCodeProvider codeProvider = new CSharpCodeProvider();

			// The MSIL compiler
			ICodeCompiler icc = codeProvider.CreateCompiler();
			CompilerParameters parameters = new CompilerParameters();

			parameters.ReferencedAssemblies.Add("System.dll");

			parameters.GenerateExecutable = false;
			parameters.GenerateInMemory = false;
			parameters.OutputAssembly = "c:\\test" + count++ + ".dll";
			parameters.ReferencedAssemblies.Add("..\\..\\..\\bin\\Debug\\Evaluant.Uss.dll");

			CompilerResults results = icc.CompileAssemblyFromSource(parameters, _Code);
			
			if (results.Errors.HasErrors)
				return;
			if (results.CompiledAssembly != null)
				_Assembly = results.CompiledAssembly;
			else
				_Assembly = null;
		}

		[TestMethod]
		public void Test()
		{
			DirectoryInfo dir = new DirectoryInfo("..\\..\\..\\NUnit\\PersistenceManagerGenerator\\XmiModels");

			foreach (FileInfo f in dir.GetFiles())
			{
				xmiReader = new StreamReader(f.FullName);
				GenerateCode();
				CompileCode();

				Assert.IsNotNull(_Assembly);

				int types = _Assembly.GetTypes().Length;

				Assert.IsFalse(types == 0, f.Name);
			}
		}
	}
}
