using System;
using System.Collections;

using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;

using InheritanceStrategiesModel;

using NUnit.Framework;

namespace UseCases.InheritanceStrategies
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		private ObjectService pmf;
		private ObjectContext pm;
		public virtual string StrategyName
		{
			get
			{
				return "TablePerHierarchyStrategyMapping";
			}
		}

		public void TestStrategy()
		{
			
		}

		[SetUp]
		public void SetUp()
		{
			pmf = new ObjectService("UseCases.config", StrategyName);
			pmf.AddAssembly("NUnit");
			pm = pmf.CreateObjectContext();
			pm.InitializeRepository();
			SerializeObjects();
		}

		public void SerializeObjects()
		{
			A a1 = new A();
			a1.A_Prop = "a1 object";

			A a2 = new A();
			a2.A_Prop = "a2 object";

			B b1 = new B();
			b1.B_Prop = "b1 object";

			B b2 = new B();
			b2.B_Prop = "b2 object";

			C c1 = new C();
			c1.B_Prop = "b1 (c1) object";
			c1.C_Prop = "c1 object";

			C c2 = new C();
			c2.B_Prop = "b2 (c2) object";
			c2.C_Prop = "c2 object";

			D d1 = new D();
			d1.B_Prop = "b1 (d1) object";
			d1.D_Prop = "d1 object";

			E e1 = new E();
			e1.B_Prop = "b1 (e1) object";
			e1.C_Prop = "c1 (e1) object";
			e1.E_Prop = "e1 object";

			E e2 = new E();
			e2.B_Prop = "b2 (e2) object";
			e2.C_Prop = "c2 (e2) object";
			e2.E_Prop = "e2 object";

			F f1 = new F();
			f1.B_Prop = "b1 (f1) object";
			f1.C_Prop = "c1 (f1) object";
			f1.F_Prop = "f1 object";

			G g1 = new G();
			g1.G_Prop = "g1 object";

			a1.Bs.Add(b1);
			a1.Bs.Add(d1);

			a2.Bs.Add(b2);
			a2.Bs.Add(f1);

			g1.Cs.Add(c1);
			g1.Cs.Add(c2);
			g1.Cs.Add(e1);
			g1.Cs.Add(e2);

			pm.BeginTransaction();
			pm.Serialize(a1);
			pm.Serialize(a2);
			pm.Serialize(g1);
			pm.CommitTransaction();
		}

		[Test]
		public void TestAObjects()
		{
			pm.Clear();

			IList as1 = pm.Load(typeof(A), "[A_Prop='a1 object']");
			Assert.AreEqual("a1 object", ((A)as1[0]).A_Prop);
			Assert.AreEqual(1, as1.Count);
			Assert.AreEqual(2, ((A)as1[0]).Bs.Count);
			
			pm.Clear();

			IList as2 = pm.Load(typeof(A), "[A_Prop='a2 object']");
			Assert.AreEqual("a2 object", ((A)as2[0]).A_Prop);
			Assert.AreEqual(1, as2.Count);
			Assert.AreEqual(2, ((A)as2[0]).Bs.Count);
		}

		[Test]
		public void TestBObjects()
		{
			pm.Clear();

			IList bs1 = pm.Load(typeof(B), "[B_Prop='b1 object']");
			Assert.AreEqual(1, bs1.Count);
			Assert.AreEqual("b1 object", ((B)bs1[0]).B_Prop);
			B b1Loaded = (B)bs1[0];
			Assert.AreEqual(typeof(B), b1Loaded.GetType());

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(A), b1Loaded.Parents.GetA().GetType());
//			Assert.AreEqual("a1 object", b1Loaded.Parents.GetA().A_Prop);

			pm.Clear();

			IList bs2 = pm.Load(typeof(B), "[B_Prop='b2 object']");
			Assert.AreEqual(1, bs2.Count);			
			Assert.AreEqual("b2 object", ((B)bs2[0]).B_Prop);
			B b2Loaded = (B)bs2[0];
			Assert.AreEqual(typeof(B), b2Loaded.GetType());
			
			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(A), b2Loaded.Parents.GetA().GetType());
//			Assert.AreEqual("a2 object", b2Loaded.Parents.GetA().A_Prop);
		}

		[Test]
		public void TestCObjects()
		{
			pm.Clear();

			IList cs1 = pm.Load(typeof(C), "[C_Prop='c1 object']");
			Assert.AreEqual("c1 object", ((C)cs1[0]).C_Prop);
			Assert.AreEqual("b1 (c1) object", ((C)cs1[0]).B_Prop);
			C c1Loaded = (C)cs1[0];
			Assert.IsTrue(c1Loaded.GetType().IsSubclassOf(typeof(B)));
			Assert.AreEqual(typeof(C), c1Loaded.GetType());

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(G), c1Loaded.Parents.GetG());
//			Assert.AreEqual("g1 object", c1Loaded.Parents.GetG().G_Prop);

			IList cs2 = pm.Load(typeof(C), "[C_Prop='c2 object']");
			Assert.AreEqual("c2 object", ((C)cs2[0]).C_Prop);
			Assert.AreEqual("b2 (c2) object", ((C)cs2[0]).B_Prop);
			C c2Loaded = (C)cs2[0];
			Assert.IsTrue(c2Loaded.GetType().IsSubclassOf(typeof(B)));
			Assert.AreEqual(typeof(C), c2Loaded.GetType());
			
			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(G), c2Loaded.Parents.GetG());
//			Assert.AreEqual("g1 object", c2Loaded.Parents.GetG().G_Prop);
		}

		[Test]
		public void TestDObjects()
		{
			pm.Clear();

			IList ds = pm.Load(typeof(D), "[D_Prop='d1 object']");
			Assert.AreEqual("b1 (d1) object", ((D)ds[0]).B_Prop);
			Assert.AreEqual("d1 object", ((D)ds[0]).D_Prop);
			Assert.AreEqual(1, ds.Count);
			D dLoaded = (D)ds[0];
			Assert.IsTrue(dLoaded.GetType().IsSubclassOf(typeof(B)));
			Assert.AreEqual(typeof(D), dLoaded.GetType());
//			Assert.AreEqual(typeof(A), dLoaded.Parents.GetA());
//			Assert.AreEqual("a2 object", dLoaded.Parents.GetA().A_Prop);
		}

		[Test]
		public void TestEObjects()
		{
			pm.Clear();

			IList es1 = pm.Load(typeof(E), "[E_Prop='e1 object']");
			Assert.AreEqual("b1 (e1) object", ((E)es1[0]).B_Prop);
			Assert.AreEqual("c1 (e1) object", ((E)es1[0]).C_Prop);
			Assert.AreEqual("e1 object", ((E)es1[0]).E_Prop);
			Assert.AreEqual(1, es1.Count);
			E e1Loaded = (E)es1[0];
			Assert.IsTrue(e1Loaded.GetType().IsSubclassOf(typeof(B)));
			Assert.IsTrue(e1Loaded.GetType().IsSubclassOf(typeof(C)));
			Assert.AreEqual(typeof(E), e1Loaded.GetType());
//			Assert.AreEqual(typeof(A), e1Loaded.Parents.GetA());
//			Assert.AreEqual(typeof(G), e1Loaded.Parents.GetG());
//			Assert.AreEqual("g1 object", e1Loaded.Parents.GetG().G_Prop);

			pm.Clear();

			IList es2 = pm.Load(typeof(E), "[E_Prop='e2 object']");
			Assert.AreEqual("b2 (e2) object", ((E)es2[0]).B_Prop);
			Assert.AreEqual("c2 (e2) object", ((E)es2[0]).C_Prop);
			Assert.AreEqual("e2 object", ((E)es2[0]).E_Prop);
			Assert.AreEqual(1, es2.Count);
			E e2Loaded = (E)es2[0];
			Assert.IsTrue(e2Loaded.GetType().IsSubclassOf(typeof(B)));
			Assert.IsTrue(e2Loaded.GetType().IsSubclassOf(typeof(C)));
			Assert.AreEqual(typeof(E), e2Loaded.GetType());
//			Assert.AreEqual(typeof(A), e2Loaded.Parents.GetA());
//			Assert.AreEqual(typeof(G), e2Loaded.Parents.GetG());
//			Assert.AreEqual("g1 object", e2Loaded.Parents.GetG().G_Prop);
		}

		[Test]
		public void TestFObjects()
		{
			pm.Clear();

			IList fs = pm.Load(typeof(F), "[F_Prop='f1 object']");
			Assert.AreEqual("b1 (f1) object", ((F)fs[0]).B_Prop);
			Assert.AreEqual("c1 (f1) object", ((F)fs[0]).C_Prop);
			Assert.AreEqual("f1 object", ((F)fs[0]).F_Prop);
			Assert.AreEqual(1, fs.Count);
			F fLoaded = (F)fs[0];
			Assert.IsTrue(fLoaded.GetType().IsSubclassOf(typeof(B)));
			Assert.IsTrue(fLoaded.GetType().IsSubclassOf(typeof(C)));
			Assert.AreEqual(typeof(F), fLoaded.GetType());
//			Assert.AreEqual(typeof(A), e2Loaded.Parents.GetA());
//			Assert.AreEqual(typeof(G), e2Loaded.Parents.GetG());
//			Assert.AreEqual("g1 object", fLoaded.Parents.GetG().G_Prop);
		}

		[Test]
		public void TestGObjects()
		{
			pm.Clear();

			IList gs1 = pm.Load(typeof(G), "[G_Prop='g1 object']");
			Assert.AreEqual("g1 object", ((G)gs1[0]).G_Prop);
			Assert.AreEqual(1, gs1.Count);
			Assert.AreEqual(4, ((G)gs1[0]).Cs.Count);
		}
	}
}
