using System;
using System.Collections;
using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;

using LinkClass;

using NUnit.Framework;

namespace UseCases.LinkClassModel
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		private ObjectService pmf;
		private ObjectContext pm;

		[SetUp]
		public void SetUp()
		{
			pmf = new ObjectService("UseCases.config", "LinkClassModel");
			pmf.AddAssembly(typeof(Job).Assembly);
			pm = pmf.CreateObjectContext();

			pm.InitializeRepository();

			Company c1 = new Company();
			c1.Name = "Comrcio Mineiro";
			c1.Chairman = "Pedro Afonso";
			c1.Address = "Av. dos Lusadas, 23";
			c1.City = "Sao Paulo";

			Employee e1 = new Employee();
			e1.Name = "Accorti";
			e1.Firstname = "Paolo";
			e1.Birthday = new DateTime(1982, 06, 12);

			Employee e2 = new Employee();
			e2.Name = "Cruz";
			e2.Firstname = "Aria";
			e2.Birthday = new DateTime(1978, 10, 05);

			Job j1 = new Job();
			j1.Title = "Accountant";
			j1.StartDate = new DateTime(2003, 04, 01);

			Job j2 = new Job();
			j2.Title = "Worker";
			j2.StartDate = new DateTime(2003, 05, 03);

			j1.Employee = e1;
			j2.Employee = e2;

			c1.EmployeeJob.Add(j1);
			c1.EmployeeJob.Add(j2);

			pm.BeginTransaction();
			pm.Serialize(c1);
			pm.Serialize(j1);
			pm.Serialize(j2);
			pm.CommitTransaction();
		}

		[Test]
		public void TestJobs()
		{
			pm.Clear();

			IList job1List = pm.Load(typeof(Job), "[Title='Accountant']");
			Assert.AreEqual(1, job1List.Count);
			Job job1 = (Job)job1List[0];
			Assert.AreEqual("Accountant", job1.Title);
			Assert.AreEqual("Accorti", job1.Employee.Name);
			Assert.AreEqual("Comrcio Mineiro", ((Company)pm.LoadParents(typeof(Company), job1, "EmployeeJob")[0]).Name);

			IList job2List = pm.Load(typeof(Job), "[Title='Worker']");
			Assert.AreEqual(1, job1List.Count);
			Job job2 = (Job)job2List[0];
			Assert.AreEqual("Worker", job2.Title);
			Assert.AreEqual("Cruz", job2.Employee.Name);
			Assert.AreEqual("Comrcio Mineiro", ((Company)pm.LoadParents(typeof(Company), job2, "EmployeeJob")[0]).Name);
		}

		[Test]
		public void TestEmployees()
		{
			pm.Clear();

			IList employee1List = pm.Load(typeof(Employee), "[Name='Accorti']");
			Assert.AreEqual(1, employee1List.Count);
			Employee employee1 = (Employee)employee1List[0];
			Assert.AreEqual("Accorti", employee1.Name);
			
			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(Job), employee1.Parents.GetCompanyEmployeesJob().GetType());
//			Assert.AreEqual("Accountant", ((Job)employee1.Parents.GetCompanyEmployeesJob()).Title);

			IList employee2List = pm.Load(typeof(Employee), "[Name='Cruz']");
			Assert.AreEqual(1, employee2List.Count);
			Employee employee2 = (Employee)employee2List[0];
			Assert.AreEqual("Cruz", employee2.Name);
			
			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(typeof(Job), employee2.Parents.GetCompanyEmployeesJob().GetType());
//			Assert.AreEqual("Worker", ((Job)employee2.Parents.GetCompanyEmployeesJob()).Title);
		}

		[Test]
		public void TestCompanies()
		{
			pm.Clear();

			IList companies = pm.Load(typeof(Company), "[Name='Comrcio Mineiro']");
			Assert.AreEqual(1, companies.Count);
			Assert.AreEqual("Comrcio Mineiro", ((Company)companies[0]).Name);
			Company company1 = (Company)companies[0];
			Assert.AreEqual(2, company1.EmployeeJob.Count);
		}
	}
}
