using System;
using System.Collections;
using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;
using ManyToMany;

using NUnit.Framework;

namespace UseCases.ManyToManyModel
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		private ObjectService pmf;
		private ObjectContext pm;

		[SetUp]
		public void SetUp()
		{
			pmf = new ObjectService("UseCases.config", "ManyToManyModel");
			pmf.AddAssembly(typeof(Company).Assembly);
			pm = pmf.CreateObjectContext();

			Company c1 = new Company();
			c1.Name = "Hanari Carnes";
			c1.Chairman = "Martn Sommer";
			c1.Address = "Rua do Pao, 67";
			c1.City = "Rio de Janeiro";

			Company c2 = new Company();
			c2.Name = "Que Delcia";
			c2.Chairman = "Bernardo Batista";
			c2.Address = "Rua da Panificadora, 12";
			c2.City = "Lisboa";

			Shareholder s1 = new Shareholder();
			s1.Name = "King Robert";
			s1.SharesNumber = 10;

			Shareholder s2 = new Shareholder();
			s2.Name = "Buchanan Steven";
			s2.SharesNumber = 78;

			c1.Shareholders.Add(s1);
			c1.Shareholders.Add(s2);

			c2.Shareholders.Add(s1);
			c2.Shareholders.Add(s2);

			pm.InitializeRepository();

			pm.BeginTransaction();
			pm.Serialize(c1);
			pm.Serialize(c2);
			pm.CommitTransaction();
		}

		[Test]
		public void TestCompanies()
		{
			pm.Clear();

			IList company1List = pm.Load(typeof(Company), "[Name='Hanari Carnes']");
			Assert.AreEqual(1, company1List.Count);
			Company company1 = (Company)company1List[0];
			Assert.AreEqual("Hanari Carnes", company1.Name);
			
			Assert.AreEqual(2, company1.Shareholders.Count);
			
			pm.Clear();

			IList company2List = pm.Load(typeof(Company), "[Name='Que Delcia']");
			Assert.AreEqual(1, company2List.Count);
			Company company2 = (Company)company2List[0];
			Assert.AreEqual("Que Delcia", company2.Name);
			Assert.AreEqual(2, company2.Shareholders.Count);
		}

		[Test]
		public void TestShareholders()
		{
			pm.Clear();

			IList shareholder1List = pm.Load(typeof(Shareholder), "[Name='King Robert']");
			Assert.AreEqual(1, shareholder1List.Count);
			Shareholder shareholder1 = (Shareholder)shareholder1List[0];
			Assert.AreEqual("King Robert", shareholder1.Name);

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(2, shareholder1.Parents.GetCompanies().Length);

			pm.Clear();

			IList shareholder2List = pm.Load(typeof(Shareholder), "[Name='King Robert']");
			Assert.AreEqual(1, shareholder2List.Count);
			Shareholder shareholder2 = (Shareholder)shareholder2List[0];
			Assert.AreEqual("King Robert", shareholder2.Name);

			/// TODO: Add tests for LoadParents
//			Assert.AreEqual(2, shareholder2.Parents.GetCompanies().Length);
		}
	}
}
