using System;
using System.Collections;
using Evaluant.Uss;
using Evaluant.Uss.ObjectContext;
using ObjectModelGeneration;

using NUnit.Framework;

namespace UseCases.ObjectModelGeneration
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		private ObjectService pmf;
		private ObjectContext pm;

		[SetUp]
		public void SetUp()
		{
			pmf = new ObjectService("UseCases.config", "ObjectModelGeneration");
			pmf.AddAssembly(typeof(Person).Assembly);

			pm = pmf.CreateObjectContext();

			//Person Serialisation
			Person p = new Person();
			p.Name = "Moos";

			pm.InitializeRepository();

			pm.BeginTransaction();
			pm.Serialize(p);
			pm.CommitTransaction();

			pm.Clear();
		}

		[Test]
		public void Test()
		{
			pm.Clear();

			//Persons updating
			IList personsToUpdate = pm.Load(typeof(Person));
			
			Assert.AreEqual(1, personsToUpdate.Count);
			Assert.AreEqual("Moos", ((Person)personsToUpdate[0]).Name);
			
			foreach(Person currentPerson in personsToUpdate)
			{
				currentPerson.Name = "new_name";

				pm.BeginTransaction();
				pm.Serialize(currentPerson);
				pm.CommitTransaction();
			}

			pm.Clear();

			//Persons updating
			IList updatedPersons = pm.Load(typeof(Person));
			
			Assert.AreEqual(1, updatedPersons.Count);
			Assert.AreEqual("new_name", ((Person)updatedPersons[0]).Name);

			pm.Clear();

			//Persons removing
			IList personsToRemove = pm.Load(typeof(Person));
			
			foreach(Person currentPerson in personsToRemove)
			{
				pm.BeginTransaction();
				pm.Delete(currentPerson);
				pm.CommitTransaction();
			}

			IList persons = pm.Load(typeof(Person));

			Assert.AreEqual(0, persons.Count);
		}
	}
}
