using System;
using Evaluant.Uss;

using NUnit.Framework;

namespace UseCases.ReferencesLoading
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	/// 
	[TestFixture]
	public class Program
	{
		[Test]
		public void Main()
		{
			IPersistenceProvider pef = 
				XmlConfigLoader.LoadXmlConfig("UseCases.config", "ReferencesLoading");
			IPersistenceEngine pe = pef.CreatePersistenceEngine();

			Entity partner1 = new Entity("Partner");
			partner1.AddValue("Name", "Moos");
			partner1.AddValue("FirstName", "Robert");

			Entity partner2 = new Entity("Partner");
			partner2.AddValue("Name", "Kuamar");
			partner2.AddValue("FirstName", "Harry");

			Entity order1 = new Entity("Order");
			order1.AddValue("ProductName", "Coffee");
			order1.AddValue("Quantity", 1);

			Entity order2 = new Entity("Order");
			order2.AddValue("ProductName", "Chocolade");
			order2.AddValue("Quantity", 5);

			Entity customer = new Entity("Customer");
			customer.AddValue("Name", "Ottlieb");
			customer.AddValue("FirstName", "Sven");

			customer.AddValue("Orders", order1);
			customer.AddValue("Orders", order2);

			customer.AddValue("Partners", partner1);
			customer.AddValue("Partners", partner2);

			pe.InitializeRepository();

			Transaction t = new Transaction();
			t.Serialize(customer);
			t.Commit(pe);

			//Loading all references in the same time as the parent
			EntitySet es1 = pe.Load("Customer", new string[] { "Orders", "Partners" });
			
			Assert.AreEqual(1, es1.Count);
			Assert.AreEqual(2, ((Entity)es1[0]).GetEntitySet("Orders").Count);
			Assert.AreEqual(2, ((Entity)es1[0]).GetEntitySet("Partners").Count);

			//Just in time attributes loading (only the Name attribute)

			EntitySet es2 = pe.Load("Customer");
			
			Assert.AreEqual(1, es2.Count);

			pe.LoadReference(es2[0], new string[] { "Orders" });
			pe.LoadReference(es2[0], new string[] { "Partners" });

			Assert.AreEqual(2, ((Entity)es1[0]).GetEntitySet("Orders").Count);
			Assert.AreEqual(2, ((Entity)es1[0]).GetEntitySet("Partners").Count);
		}
	}
}
