using System;

using Pdb.Model;
using Pdb.Tools;

using System.Reflection;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

using Microsoft.Win32;
using Mono.Cecil;

namespace ReadWritePdb
{
	/// <summary>
	/// Description rsume de Program.
	/// </summary>
	public class Program
	{
		[STAThread]
		public static int Main(string[] args)
		{
			//string sourceAssembly = String.Empty;
			string assembly = String.Empty;
			string model = String.Empty;
			//bool write = false;

			ArgParser parser = new ArgParser(args);
			
			if(parser.Exists("?"))
			{
				Help();
				return 0;
			}

			if(!parser.Exists("assembly"))
			{
				Console.WriteLine("Incorrect parameters : no assembly is defined");
				Help();
				return 1;
			}

			if(!parser.Exists("model"))
			{
				Console.WriteLine("Incorrect parameters : no pdb model is defined");
				Help();
				return 1;
			}

			if(!parser.Exists("w") && !parser.Exists("r"))
			{
				Console.WriteLine("Incorrect parameters : no read / write operation is defined");
				Help();
				return 1;
			}

//			sourceAssembly = parser["source"].Value;
//			destAssembly = parser["dest"].Value;
			assembly = parser["assembly"].Value;
			model = parser["model"].Value;

			if(!System.IO.File.Exists(assembly))
			{
				Console.WriteLine("The source file does not exist.");
				return 2;           
			}

//			if(!System.IO.File.Exists(destAssembly))
//			{
//				Console.WriteLine("The destination does not exist.");
//				return 2;           
//			}

            try
            {
                if (parser.Exists("r") && !parser.Exists("w"))
                {
                    PdbReader pReader = new PdbReader(assembly);

                    PdbInfos pdb = pReader.GetPdbModel();

                    XmlTextWriter writer = new XmlTextWriter(model, System.Text.Encoding.UTF8);

                    writer.Formatting = Formatting.Indented;

                    XmlSerializer serializer = new XmlSerializer(typeof(PdbInfos));

                    serializer.Serialize(writer, pdb);
                    writer.Flush();
                    writer.Close();
                }

                if (parser.Exists("w") && !parser.Exists("r"))
                {
                    XmlTextReader reader = new XmlTextReader(model);
                    XmlSerializer serializer = new XmlSerializer(typeof(PdbInfos));

                    PdbInfos pdb = (PdbInfos)serializer.Deserialize(reader);
                    pdb.AssemblyName = assembly;

                    reader.Close();

                    foreach (TypeDefinition t in AssemblyFactory.GetAssembly(assembly).MainModule.Types)
                    {
                        foreach (MethodDefinition m in t.Methods)
                        {
                            int newToken = 0x6000000 + (int)m.MetadataToken.RID;
                            Method pdbMethod = pdb.Methods.GetMethodByFullName(m.ToString());
                            if (pdbMethod != null)
                            {
                                if (pdbMethod.Token != newToken)
                                    pdbMethod.Token = newToken;
                            }
                        }
                    }

                    PdbWriter pWriter = new PdbWriter(assembly);

                    pWriter.Write(pdb);

                    XmlTextWriter writer = new XmlTextWriter(model, System.Text.Encoding.UTF8);

                    writer.Formatting = Formatting.Indented;

                    serializer.Serialize(writer, pdb);
                    writer.Flush();
                    writer.Close();
                }

                if (parser.Exists("w") && parser.Exists("r"))
                {
                    throw new Exception("Cannot read and write in a single operation");
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Source + " " + e.Message);
            }
			return 0;
		}

		private static string EUSS_INSTALLATION_DIR_KEY = "SOFTWARE\\Evaluant\\EUSS";

		static public void Help()
		{
			Console.WriteLine(@"

Generates a pdb file for a weaved assembly

ReadWritePdb.exe /assembly:assembly_file_name /model:model_file_name /r /w

	assembly_file_name
		Specify the assembly 

	model_file_name
		Specify the weaved assembly

	/r
		Extract the pdb model of the assembly to the model file

	/w
		Generates the pdb file for the assembly according to the pdb model which is defined in the model file
");

		}
	}
}
