using System;
using System.IO;
using System.Collections;

using Evaluant.Uss.SqlMapper;

using Evaluant.Uss.Commands;
using Evaluant.Uss.Common;
using Evaluant.Uss.SqlMapper.SqlObjectModel.LDD;
using SQLObject;
using SQLObject.Renderer;
using System.Reflection;

namespace Evaluant.Uss.ScriptGenerator
{
	/// <summary>
	/// Description rsume de ScriptGenerator.
	/// </summary>
	public class ScriptGenerator
	{
		private Mapping _Mapping;

		/// <summary>
		/// The mapping which is used to generate script
		/// </summary>
		public Mapping Mapping
		{
			get { return _Mapping; }
			set { _Mapping = value; }
		}

		private string _FileName;

		/// <summary>
		/// The file which contains the script
		/// </summary>
		public string FileName
		{
			get { return _FileName; }
			set { _FileName = value; }
		}

        private DBDialect _Dialect;

        /// <summary>
        /// The dialect of the script
        /// </summary>
        public DBDialect Dialect
        {
            get { return _Dialect; }
            set { _Dialect = value; }
        }

		#region Ctor

		public ScriptGenerator()
		{
		
		}

        public ScriptGenerator(Mapping mapping, string dialect)
            : this(mapping, string.Empty, dialect)
		{
		
		}

        public ScriptGenerator(string filename, string dialect)
            : this(null, filename, dialect)
		{
		
		}

		public ScriptGenerator(Mapping mapping, string filename, string dialect)
        {
            _Mapping = mapping;
            _FileName = filename;
            _Dialect = GetDialectByName(dialect);
        }
		
		#endregion

		public DBDialect GetDialectByName(string dialect)
		{
            Type type = Utils.GetType(dialect);

            if (type == null && dialect.Split(',').Length >= 2)
            {
                string[] values = dialect.Split(',');
                type = Assembly.LoadWithPartialName(values[1]).GetType(values[0]);
            }

            if (type == null)
                throw new PersistenceEngineException("The specified type could not be found: " + dialect);

            ConstructorInfo pc = type.GetConstructor(Type.EmptyTypes);
            return (DBDialect)pc.Invoke(new object[] { });
        }

		public void GenerateScript()
		{
			if(_Mapping == null)
				throw new ScriptGeneratorException("Mapping is not defined");

			foreach(EntityMapping em in _Mapping.Entities)
			{
				foreach(AttributeMapping am in em.Attributes)
				{
					if(am.ParentEntity == null)
						am.ParentEntity = em;
					if(am.Table == null || am.Table == string.Empty)
						am.Table = em.Table;
				}
				foreach(ReferenceMapping rm in em.References)
				{
					for(int i = 0; i < rm.Rules.Count; i++)
					{
						RuleMapping rule = rm.Rules[i];
						if(i == 0)
							rule.ParentTable = em.Table;
						else
							rule.ParentTable = rm.Rules[i-1].ChildTable;
					}
					
					if(rm.EntityParent == null)
						rm.EntityParent = em;
				}
			}

			SqlMapperPersistenceEngine pe = new SqlMapperPersistenceEngine();
			pe.Mapping = _Mapping;
			pe.Dialect = _Dialect;

			ArrayList cmds = pe.CreateInitCommands();

            using (StreamWriter writer = new StreamWriter(_FileName))
            {
                foreach (ISQLExpression expression in cmds)
                    foreach (string query in Dialect.RenderQueries(expression))
                    {
                        writer.Write(query);
                        writer.WriteLine(";");
                    }
            }
        }
	}
}
